/**
 * Articulo Controller.
 *
 * Responde a eventos referente a la CLASE ARTICULO ,invoca peticiones al
 * 'modelo' cuando se hace alguna solicitud sobre la información (por ejemplo,
 * editar un documento o un registro en una base de datos).
 *
 * También puede enviar comandos a su 'vista' asociada si se solicita un cambio
 * en la forma en que se presenta de 'modelo' (por ejemplo, desplazamiento o
 * scroll por un documento o por los diferentes registros de una base de datos),
 * por tanto se podría decir que el 'controlador' hace de intermediario entre la
 * 'vista' y el 'modelo'.
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws Throws in DAO
 */
package controller;

import com.opensymphony.xwork2.ActionSupport;
import com.opensymphony.xwork2.ModelDriven;
import java.util.ArrayList;
import model.dao.ArticuloDAO;
import model.entities.Articulo;
import model.pdf.ObjectPdf;
import model.util.MessageUtil;

public class ArticuloController extends ActionSupport implements ModelDriven<Articulo> {

    private Articulo articulo = new Articulo();
    private ArrayList<Articulo> listArticulo = new ArrayList<Articulo>();
    private ArticuloDAO articuloDAO = new ArticuloDAO();
    private String option = null;
    private MessageUtil message = new MessageUtil();
    private String msg = null;

    /**
     * Implementacion del ModelDriven
     *
     * Permite enlazar tags de Struts 2 con el objeto del Action
     *
     * @return Retorna el Objeto del ModelDriven
     */
    @Override
    public Articulo getModel() {
        return articulo;
    }

    /**
     * Getter del Articulo
     *
     * @return Retorna el Articulo
     */
    public Articulo getArticulo() {
        return articulo;
    }

    /**
     * Setter del Articulo
     *
     * @param articulo Assigna el Articulo
     */
    public void setArticulo(Articulo articulo) {
        this.articulo = articulo;
    }

    /**
     * Getter de la lists de Articulos
     *
     * @return Retorna la lista de Articulos
     */
    public ArrayList<Articulo> getListArticulo() {
        return listArticulo;
    }

    /**
     * Setter de la lista de Articulos
     *
     * @param listArticulo Lista de Articulos
     */
    public void setListArticulo(ArrayList<Articulo> listArticulo) {
        this.listArticulo = listArticulo;
    }

    /**
     * Getter de Mensage
     *
     * @return Retorna el mensage destinado al Usuario
     */
    public String getMsg() {
        return msg;
    }

    /**
     * Setter de Mensage
     *
     * @param msg Mensage destinado al Usuario
     */
    public void setMsg(String msg) {
        this.msg = msg;
    }

    /**
     * Getter de Option
     *
     * @return
     */
    public String getOption() {
        return option;
    }

    /**
     * Setter de Option
     *
     * @param option Opcion Elegida por el Usuario
     */
    public void setOption(String option) {
        this.option = option;
    }

    /**
     * Insertar Articulo
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String insertArticulo() {
        if (this.validateInsert()) {
            if (articuloDAO.insertArticulo(articulo)) {
                msg = "El Articulo ha sido Insertado Correctamente.";
                //message = new MessageUtil(msg);
                articulo = new Articulo();
                return "OK";
            } else {
                msg = "Ha Ocurrido un Error al Insertar el Articulo.";
                message = new MessageUtil(msg);
                return "KO";
            }
        } else {
            msg = "Ha Ocurrido un Error al Validar el Formulario.";
            message = new MessageUtil(msg);
            addActionError(msg);
            return "KO";
        }
    }

    /**
     * Selecciona el Articulo
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectArticulo() {
        if (this.validateSelect()) {

            if (option.equals("Buscar por ID")) {
                articulo = articuloDAO.selectArticulo(articulo.getIdArticulo());
                if (articulo != null) {
                    listArticulo.add(articulo);
                }
            } else {
                listArticulo = articuloDAO.selectByExample(articulo);
            }

            if (listArticulo.size() > 0) {
                articulo = listArticulo.get(0);
                msg = "El Articulo se ha Seleccionado Correctamente.";
                //message = new MessageUtil(msg);
                return "OK";
            } else {
                msg = "El Articulo no Existe.";
                message = new MessageUtil(msg);
                addActionError(msg);
                return "KO";
            }
        } else {
            msg = "Ha Ocurrido un Error al Validar el Formulario.";
            message = new MessageUtil(msg);
            addActionError(msg);
            return "KO";
        }
    }

    /**
     * Selecciona todos los Articulos
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectAllArticulo() {
        listArticulo = articuloDAO.selectAll();
        if (listArticulo.size() > 0) {
            msg = "Los Articulos se ha Seleccionado Correctamente.";
        } else {
            msg = "La base de datos de Articulos esta Vacia.";
            message = new MessageUtil(msg);
            return "KO";
        }
        return "OK";
    }

    /**
     * Selecciona los Articulos que coinciden con el Ejemplo
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectByExampleArticulo() {
        if (this.validateExample()) {
            listArticulo = articuloDAO.selectByExample(articulo);

            if (listArticulo.size() > 0) {
                System.out.println(listArticulo.toString()+" en el select");
                msg = "El Articulo se ha Seleccionado Correctamente.";
                return "OK";
            } else {
                msg = "Ha Ocurrido un Error al Seleccionar los Articulo.";
                message = new MessageUtil(msg);
                return "KO";
            }
        } else {
            System.out.println("pasa a select all");
            msg = "Todos los Campos estan Vacios.";
            return this.selectAllArticulo();
        }
    }

    /**
     * Elimina el Articulo
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String deleteArticulo() {
        msg = "Desea Eliminar el Articulo??";
        if (message.showConfirmationMessage(msg) != -1) {
            if (articuloDAO.deleteArticulo(articulo)) {
                msg = "El Articulo ha sido Eliminado Correctamente.";
                message = new MessageUtil(msg);
                articulo = new Articulo();
                return "OK";
            } else {
                msg = "Ha Ocurrido un Error al Eliminar el Articulo.";
                message = new MessageUtil(msg);
                return "KO";
            }
        }
        msg = "Processo Abortado.";
        return "KO";
    }

    /**
     * Modifica el Articulo
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String updateArticulo() {
        msg = "Desea Modificar el Articulo??";
        if (message.showConfirmationMessage(msg) != -1) {
            if (articuloDAO.updateArticulo(articulo)) {
                msg = "El Articulo ha sido Modificado Correctamente.";
                message = new MessageUtil(msg);
                return "OK";
            } else {
                msg = "Ha Ocurrido un Error al Modificado el Articulo.";
                message = new MessageUtil(msg);
                return "KO";
            }
        }
        msg = "Processo Abortado.";
        return "KO";
    }

    /**
     * Imprime un Articulo
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printArticulo() {
        if (articulo != null) {
            ObjectPdf objPdf = new ObjectPdf(this.articulo);

            return "OK";
        } else {
            message = new MessageUtil("No hay datos para imprimir");
            return "KO";
        }
    }

    /**
     * Imprime la lista de Articulos
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printArticulos() {
        
        if (this.selectByExampleArticulo().equals("OK")) {
            System.out.println(listArticulo.toString());
            if (listArticulo != null) {
                ObjectPdf objPdf = new ObjectPdf(this.listArticulo);

                return "OK";
            } else {
                message = new MessageUtil("No hay datos para imprimir");
                return "KO";
            }
        }else{
            message = new MessageUtil("No hay datos para imprimir");
            return "KO";
        }
    }

    /**
     * Imprime la lista de Articulos
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printAllArticulos() {

        if (this.selectAllArticulo().equals("OK")) {

            if (listArticulo != null) {
                ObjectPdf objPdf = new ObjectPdf(this.listArticulo);

                return "OK";
            } else {
                message = new MessageUtil("No hay datos para imprimir");
                return "KO";
            }
        } else {
            message = new MessageUtil("No hay datos para imprimir");
            return "KO";
        }
    }

    //-------------------------------------------------------------
    //------------------------VALIDATES----------------------------
    //-------------------------------------------------------------
    /**
     * Valida los campos de Insercion de datos
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateInsert() {
        boolean bool = true;

        if (this.articulo.getDescripcion1().equals("")) {
            addFieldError("", "El campo Descripcion 1 es Obligatorio");
            bool = false;
        }

        if (this.articulo.getDescripcion2().equals("")) {
            addFieldError("", "El campo Descripcion 2 es Obligatorio");
            bool = false;
        }

        if (this.articulo.getDescripcion3().equals("")) {
            articulo.setDescripcion3(null);
        }

        if (this.articulo.getPrecio() == null) {
            addFieldError("", "El campo Precio es Obligatorio");
            bool = false;
        }

        if (this.articulo.getEan().equals("")) {
            addFieldError("", "El campo Ean es Obligatorio");
            bool = false;
        }

        if (this.articulo.getCodInterno().equals("")) {
            articulo.setCodInterno(null);
        }

        return bool;
    }

    /**
     * Valida los campos de Seleccion de Articulos
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateSelect() {
        boolean bool = true;

        articulo.onlyParameter(option);

        if (option.equals("Buscar por ID")) {
            if (this.articulo.getIdArticulo() == null) {
                bool = false;
            }
        }

        if (option.equals("Buscar por EAN")) {
            if (this.articulo.getEan().equals("")) {
                bool = false;
            }
        }

        if (option.equals("Buscar por SMS")) {
            if (this.articulo.getCodInterno().equals("")) {
                bool = false;
            }
        }

        if (option.equals("Buscar por Tecla")) {
            if (this.articulo.getTecla() == null) {
                bool = false;
            }
        }

        return bool;
    }

    /**
     * Valida los campos de Seleccion por Ejemplo
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateExample() {
        boolean bool = true;
        int n = 0;

        if (this.articulo.getUnidad().equals("")) {
            n = n + 1;
            articulo.setUnidad(null);
        }

        if (this.articulo.getDescripcion1().equals("")) {
            n = n + 1;
            articulo.setDescripcion1(null);
        }

        if (this.articulo.getDescripcion2().equals("")) {
            n = n + 1;
            articulo.setDescripcion2(null);
        }

        if (this.articulo.getDescripcion3().equals("")) {
            n = n + 1;
            articulo.setDescripcion3(null);
        }

        if (this.articulo.getPrecio() == null) {
            n = n + 1;
        }

        if (this.articulo.getPrecioOferta() == null) {
            n = n + 1;
        }

        if (n == 6) {
            bool = false;
        }
        return bool;
    }

    //-------------------------------------------------------------
    //--------------------------PANTALLAS--------------------------
    //-------------------------------------------------------------
    /**
     * Abrir Pantalla de Seleccion de Articulo
     *
     * @return Retorna "PANTALLA" dando el "OK" para cargar la Pagina
     */
    public String screenSelectArticulo() {
        articulo = new Articulo();
        return "PANTALLA";
    }

    /**
     * Abrir Pantalla de Modificacion de Articulo
     *
     * @return Retorna "PANTALLA" dando el "OK" para cargar la Pagina
     */
    public String screenUpdateArticulo() {
        return "PANTALLA";
    }

    /**
     * Abrir Pantalla de Seleccion de Articulo mediante Ejemplo
     *
     * @return Retorna "PANTALLA" dando el "OK" para cargar la Pagina
     */
    public String screenSelectByExampleArticulo() {
        articulo = new Articulo();
        return "PANTALLA";
    }
}
