/**
 * Cartel Controller.
 *
 * Responde a eventos referente a la CLASE CARTEL ,invoca peticiones al 'modelo'
 * cuando se hace alguna solicitud sobre la información (por ejemplo, editar un
 * documento o un registro en una base de datos).
 *
 * También puede enviar comandos a su 'vista' asociada si se solicita un cambio
 * en la forma en que se presenta de 'modelo' (por ejemplo, desplazamiento o
 * scroll por un documento o por los diferentes registros de una base de datos),
 * por tanto se podría decir que el 'controlador' hace de intermediario entre la
 * 'vista' y el 'modelo'.
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws Throws in DAO
 */
package controller;

import com.opensymphony.xwork2.ActionSupport;
import com.opensymphony.xwork2.ModelDriven;
import java.util.ArrayList;
import java.util.Iterator;
import model.dao.ArticuloDAO;
import model.dao.CartelDAO;
import model.dao.PlantillaDAO;
import model.entities.Articulo;
import model.entities.Cartel;
import model.entities.Plantilla;
import model.pdf.ObjectPdf;
import model.util.MessageUtil;

public class CartelController extends ActionSupport implements ModelDriven<Cartel> {

    private Cartel cartel = new Cartel();
    private ArrayList<Cartel> listCartel = new ArrayList<Cartel>();
    private CartelDAO cartelDAO = new CartelDAO();
    private ArrayList<Plantilla> listPlantilla = new ArrayList<Plantilla>();
    private ArrayList<Articulo> listArticulo = new ArrayList<Articulo>();
    private ArrayList<String> listTipo = new ArrayList<String>();
    private ArrayList<String> listTamano = new ArrayList<String>();
    private String option = null;
    private MessageUtil message = new MessageUtil();
    private String msg = null;

    /**
     * Implementacion del ModelDriven
     *
     * Permite enlazar tags de Struts 2 con el objeto del Action
     *
     * @return Retorna el Objeto del ModelDriven
     */
    @Override
    public Cartel getModel() {
        return cartel;
    }

    /**
     * Getter de la Lista de Tipos de Plantilla
     *
     * @return Retorna la lista de Tipos de Plantilla Disponibles en el sistema.
     */
    public ArrayList<String> getListTipo() {
        return listTipo;
    }

    /**
     * Setter de la lista de Tipos de Plantilla
     *
     * @param listTipo Assigna una lista de Tipos de Plantilla
     */
    public void setListTipo(ArrayList<String> listTipo) {
        this.listTipo = listTipo;
    }

    /**
     * Getter de la Lista de Tamaños de Plantilla
     *
     * @return Retorna la lista de Tamaños de Plantilla Disponibles en el
     * sistema.
     */
    public ArrayList<String> getListTamano() {
        return listTamano;
    }

    /**
     * Setter de la lista de Tamaños de Plantilla
     *
     * @param listTipo Assigna una lista de Tamaños de Plantilla
     */
    public void setListTamano(ArrayList<String> listTamano) {
        this.listTamano = listTamano;
    }

    /**
     * Getter del Objeto Cartel
     *
     * @return Retorna el cartel
     */
    public Cartel getCartel() {
        return cartel;
    }

    /**
     * Setter del Objeto Cartel
     *
     * @param cartel Assigna el Cartel
     */
    public void setCartel(Cartel cartel) {
        this.cartel = cartel;
    }

    /**
     * Getter de la lista de Carteles
     *
     * @return Retorna la lista de Carteles
     */
    public ArrayList<Cartel> getListCartel() {
        return listCartel;
    }

    /**
     * Setter de la lista de Carteles
     *
     * @param listCartel Assigna la lista de Carteles
     */
    public void setListCartel(ArrayList<Cartel> listCartel) {
        this.listCartel = listCartel;
    }

    /**
     * Getter de Mensage
     *
     * @return Retorna el mensage destinado al Usuario
     */
    public String getMsg() {
        return msg;
    }

    /**
     * Setter de Mensage
     *
     * @param msg Mensage destinado al Usuario
     */
    public void setMsg(String msg) {
        this.msg = msg;
    }

    /**
     * Getter de Option
     *
     * @return
     */
    public String getOption() {
        return option;
    }

    /**
     * Setter de Option
     *
     * @param option Opcion Elegida por el Usuario
     */
    public void setOption(String option) {
        this.option = option;
    }

    /**
     * Funcion Auxiliar que busca los tipos de plantillas que hay en el sistema
     */
    private void listTypes() {
        PlantillaDAO planDAO = new PlantillaDAO();
        ArrayList<Plantilla> listPlan = planDAO.selectAll();

        Iterator it = listPlan.listIterator();

        while (it.hasNext()) {
            Plantilla plan = (Plantilla) it.next();

            if (!this.listTamano.contains(plan.getTamano())) {
                listTamano.add(plan.getTamano());
            }

            if (!this.listTipo.contains(plan.getTipo())) {
                listTipo.add(plan.getTipo());
            }
        }
    }

    /**
     * Insertar Cartel
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String insertCartel() {
        this.listTypes();
        if (this.validateInsert()) {
            ArticuloDAO artDAO = new ArticuloDAO();
            if (artDAO.selectArticulo(cartel.getArticulo()) == null) {
                artDAO.insertArticulo(cartel.getArticulo());
            }
            if (cartelDAO.insertCartel(cartel)) {
                msg = "El Cartel ha sido Insertado Correctamente.";
                cartel = new Cartel();
                return "OK";
            } else {
                msg = "El Cartel Ya Existe.";
                message = new MessageUtil(msg);
                return "KO";
            }
        } else {
            msg = "Ha Ocurrido un Error al Validar el Formulario.";
            message = new MessageUtil(msg);
            addActionError(msg);
            return "KO";
        }
    }

    /**
     * Selecciona el Cartel mediante ID
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectCartelID() {
        this.listTypes();
        if (cartel.getIdCartel() != null) {
            cartel = cartelDAO.selectCartel(cartel.getIdCartel());
            if (cartel != null) {

                msg = "El Cartel se ha Seleccionado Correctamente.";

                return "OK";
            } else {
                msg = "El Cartel no Existe.";
                message = new MessageUtil(msg);
                addActionError(msg);
                return "KO";
            }
        } else {
            msg = "Ha Ocurrido un Error al Validar el Formulario..";
            message = new MessageUtil(msg);
            addActionError(msg);
            addFieldError("", "Introduzca el ID");
            return "KO";
        }
    }

    /**
     * Selecciona el Cartel
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectCartel() {
        this.listTypes();
        if (this.validateSelect()) {
            Cartel car = cartelDAO.selectCartel(cartel);
            if (car != null) {
                cartel = car;

                msg = "El Cartel se ha Seleccionado Correctamente.";

                return "OK";
            } else {
                msg = "El Cartel no Existe.";
                message = new MessageUtil(msg);
                addActionError(msg);
                return "KO";
            }
        } else {
            msg = "Ha Ocurrido un Error al Validar el Formulario.";
            message = new MessageUtil(msg);
            addActionError(msg);
            return "KO";
        }
    }

    /**
     * Selecciona todos los Carteles
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectAllCartel() {
        listCartel = cartelDAO.selectAll();
        if (listCartel.size() > 0) {
            msg = "Los Cartels se ha Seleccionado Correctamente.";
        } else {
            msg = "La base de datos de Cartels esta Vacia.";
            message = new MessageUtil(msg);
            return "KO";
        }
        return "OK";
    }

    /**
     * Selecciona los Carteles que coinciden con el Ejemplo
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectByExampleCartel() {
        this.listTypes();
        boolean bool = false;

        if (this.validateExampleArticulo()) {

            Iterator it = listArticulo.listIterator();

            while (it.hasNext()) {
                Cartel car = cartelDAO.selectCartel((Articulo) it.next());
                listCartel.add(car);
            }
            bool = true;
        }

        if (validateExamplePlantilla()) {

            Iterator it2 = listPlantilla.listIterator();

            while (it2.hasNext()) {
                Cartel car = cartelDAO.selectCartel((Plantilla) it2.next());

                if (!listCartel.contains(car)) {
                    listCartel.add(car);
                }
            }
            bool = true;
        }

        if (bool) {
            if (listCartel.size() > 0) {

                msg = "El Cartel se ha Seleccionado Correctamente.";
                return "OK";
            } else {
                msg = "No hay ningun cartel Con estas Caracteristicas.";
                message = new MessageUtil(msg);
                return "KO";
            }
        } else {
            msg = "Todos los Campos estan Vacios.";
            return selectAllCartel();
        }
    }

    /**
     * Elimina el Cartel
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String deleteCartel() {
        msg = "Desea Eliminar el Cartel??";
        if (message.showConfirmationMessage(msg) != -1) {
            if (cartelDAO.deleteCartel(cartel)) {
                msg = "El Cartel ha sido Eliminado Correctamente.";
                message = new MessageUtil(msg);
                cartel = new Cartel();
                return "OK";
            } else {
                msg = "Ha Ocurrido un Error al Eliminar el Cartel.";
                message = new MessageUtil(msg);
                return "KO";
            }
        }
        msg = "Processo Abortado.";
        return "KO";
    }

    /**
     * Modifica el Cartel
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String updateCartel() {
        this.listTypes();
        msg = "Desea Modificar el Cartel??";
        if (message.showConfirmationMessage(msg) != -1) {

            PlantillaDAO planDAO = new PlantillaDAO();
            Plantilla plan = planDAO.selectPlantilla(cartel.getPlantilla());

            cartel.setPlantilla(plan);

            if (cartelDAO.updateCartel(cartel)) {
                msg = "El Cartel ha sido Modificado Correctamente.";
                message = new MessageUtil(msg);
                return "OK";
            } else {
                msg = "Ha Ocurrido un Error al Modificado el Cartel.";
                message = new MessageUtil(msg);
                return "KO";
            }
        }
        msg = "Processo Abortado.";
        return "KO";
    }

    /**
     * Imprime un Cartel (formato texto)
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printCartel() {
        if (cartel != null) {
            ObjectPdf objPdf = new ObjectPdf(this.cartel);

            return "OK";
        } else {
            message = new MessageUtil("No hay datos para imprimir");
            return "KO";
        }
    }

    /**
     * Imprime la lista de Carteles (formato texto)
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printCarteles() {
        if (this.selectByExampleCartel().equals("OK")) {

            if (listCartel != null) {
                ObjectPdf objPdf = new ObjectPdf(this.listCartel);

                return "OK";
            } else {
                message = new MessageUtil("No hay datos para imprimir");
                return "KO";
            }
        } else {
            message = new MessageUtil("No hay datos para imprimir");
            return "KO";
        }
    }

    /**
     * Imprime la lista de Carteles (formato texto)
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printAllCarteles() {
        if (this.selectAllCartel().equals("OK")) {
            if (listCartel != null) {
                ObjectPdf objPdf = new ObjectPdf(this.listCartel);

                return "OK";
            } else {
                message = new MessageUtil("No hay datos para imprimir");
                return "KO";
            }
        } else {
            message = new MessageUtil("No hay datos para imprimir");
            return "KO";
        }
    }

    //-------------------------------------------------------------
    //------------------------VALIDATES----------------------------
    //-------------------------------------------------------------
    /**
     * Valida los campos de Insercion de datos
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateInsert() {
        boolean bool = true;

        if (validateArticulo() == false) {
            bool = false;
        }

        if (validatePlantilla() == false) {
            bool = false;
        }

        return bool;
    }

    /**
     * Funcion Auxiliar para la Validacion de Carteles
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateArticulo() {
        boolean bool = true;

        //validamos this.cartel.getArticulo()
        if (this.cartel.getArticulo().getDescripcion1().equals("")) {
            addFieldError("", "El campo Descripcion 1 es Obligatorio");
            bool = false;
        }

        if (this.cartel.getArticulo().getDescripcion2().equals("")) {
            addFieldError("", "El campo Descripcion 2 es Obligatorio");
            bool = false;
        }

        if (this.cartel.getArticulo().getDescripcion3().equals("")) {
            this.cartel.getArticulo().setDescripcion3(null);
        }

        if (this.cartel.getArticulo().getPrecio() == null) {
            addFieldError("", "El campo Precio es Obligatorio");
            bool = false;
        }

        if (this.cartel.getArticulo().getEan().equals("")) {
            addFieldError("", "El campo Ean es Obligatorio");
            bool = false;
        }

        if (this.cartel.getArticulo().getCodInterno().equals("")) {
            this.cartel.getArticulo().setCodInterno(null);
        }

        return bool;
    }

    /**
     * Funcion Auxiliar para la Validacion de Carteles
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validatePlantilla() {
        boolean bool = true;

        if (this.cartel.getPlantilla().getTipo().equals("")) {
            addFieldError("", "El campo Tipo es Obligatorio");
            bool = false;
        }

        if (this.cartel.getPlantilla().getTamano().equals("")) {
            addFieldError("", "El campo Tamaño es Obligatorio");
            bool = false;
        }

        if (!bool) {
            return false;
        }

        PlantillaDAO planDAO = new PlantillaDAO();

        Plantilla plan = planDAO.selectPlantilla(cartel.getPlantilla());
        if (plan == null) {
            addFieldError("", "La Plantilla No Existe.");
            this.cartel.getPlantilla().reset();
            return false;
        } else {
            cartel.setPlantilla(plan);
            return true;
        }
    }

    /**
     * Valida los campos de Seleccion de Cartels
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateSelect() {
        boolean bool = true;
        int n = 0;

        if (!this.validatePlantilla()) {
            bool = false;
        }

        if (this.cartel.getArticulo().getIdArticulo() == null) {
            n = n + 1;
        } else {
            ArticuloDAO artDAO = new ArticuloDAO();
            Articulo art = artDAO.selectArticulo(cartel.getArticulo().getIdArticulo());

            if (art != null) {
                cartel.setArticulo(art);
                return true;
            } else {
                addFieldError("", "El Articulo No Existe.");
                this.cartel.getArticulo().reset();
                return false;
            }
        }

        if (this.cartel.getArticulo().getEan().equals("")) {
            n = n + 1;
        }

        if (this.cartel.getArticulo().getCodInterno().equals("")) {
            n = n + 1;
        }

        if (this.cartel.getArticulo().getTecla() == null) {
            n = n + 1;
        }

        if (n != 4) {
            ArticuloDAO artDAO = new ArticuloDAO();

            listArticulo = artDAO.selectByExample(cartel.getArticulo());

            if (listArticulo.size() > 0) {
                cartel.setArticulo(listArticulo.get(0));
            } else {
                addFieldError("", "El Articulo No Existe.");
                this.cartel.getArticulo().reset();
                bool = false;
            }
        } else {
            addFieldError("", "Todos los campos Referentes a Articulo estan Vacios.");
            this.cartel.getArticulo().reset();
            bool = false;
        }

        return bool;
    }

    /**
     * Funcion Auxiliar para la validacion de Carteles elegidos por Ejemplo
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateExamplePlantilla() {
        int p = 0;


        if (this.cartel.getPlantilla().getTamano().equals("0")) {
            cartel.getPlantilla().setTamano(null);
            p = p + 1;
        }

        if (this.cartel.getPlantilla().getTipo().equals("0")) {
            cartel.getPlantilla().setTipo(null);
            p = p + 1;
        }

        if (p == 0) {
            PlantillaDAO planDAO = new PlantillaDAO();
            this.listPlantilla = planDAO.selectByExample(this.cartel.getPlantilla());
            return true;
        }
        return false;
    }

    /**
     * Funcion Auxiliar para la validacion de Carteles elegidos por Ejemplo
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateExampleArticulo() {
        int n = 0;

        if (this.cartel.getArticulo().getUnidad().equals("")) {
            n = n + 1;
            cartel.getArticulo().setUnidad(null);
        }

        if (this.cartel.getArticulo().getDescripcion1().equals("")) {
            n = n + 1;
            cartel.getArticulo().setDescripcion1(null);
        }

        if (this.cartel.getArticulo().getDescripcion2().equals("")) {
            n = n + 1;
            cartel.getArticulo().setDescripcion2(null);
        }

        if (this.cartel.getArticulo().getDescripcion3().equals("")) {
            n = n + 1;
            cartel.getArticulo().setDescripcion3(null);
        }

        if (this.cartel.getArticulo().getPrecio() == null) {
            n = n + 1;
        }

        if (this.cartel.getArticulo().getPrecioOferta() == null) {
            n = n + 1;
        }

        if (n == 0) {
            ArticuloDAO artDAO = new ArticuloDAO();
            this.listArticulo = artDAO.selectByExample(this.cartel.getArticulo());
            return true;
        }

        return false;
    }

    //-------------------------------------------------------------
    //--------------------------PANTALLAS--------------------------
    //-------------------------------------------------------------
    /**
     * Abrir Pantalla de Seleccion de Cartel
     *
     * @return Retorna "PANTALLA" dando el "OK" para cargar la Pagina
     */
    public String screenSelectCartel() {
        this.cartel = new Cartel();
        this.listTypes();
        return "PANTALLA";
    }

    /**
     * Abrir Pantalla de Insercion de Cartel
     *
     * @return Retorna "PANTALLA" dando el "OK" para cargar la Pagina
     */
    public String screenInsertCartel() {
        this.listTypes();
        return "PANTALLA";
    }

    /**
     * Abrir Pantalla de Modificacion de Cartel
     *
     * @return Retorna "PANTALLA" dando el "OK" para cargar la Pagina
     */
    public String screenUpdateCartel() {
        this.listTypes();
        return "PANTALLA";
    }

    /**
     * Abrir Pantalla de Seleccion de Cartel mediante Ejemplo
     *
     * @return Retorna "PANTALLA" dando el "OK" para cargar la Pagina
     */
    public String screenSelectByExampleCartel() {
        this.cartel = new Cartel();

        this.listTypes();
        return "PANTALLA";
    }
}
