/**
 * Departamento Controller.
 *
 * Responde a eventos referente a la CLASE DEPARTAMENTO ,invoca peticiones al
 * 'modelo' cuando se hace alguna solicitud sobre la información (por ejemplo,
 * editar un documento o un registro en una base de datos).
 *
 * También puede enviar comandos a su 'vista' asociada si se solicita un cambio
 * en la forma en que se presenta de 'modelo' (por ejemplo, desplazamiento o
 * scroll por un documento o por los diferentes registros de una base de datos),
 * por tanto se podría decir que el 'controlador' hace de intermediario entre la
 * 'vista' y el 'modelo'.
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws Throws in DAO
 */
package controller;

import com.opensymphony.xwork2.ActionSupport;
import com.opensymphony.xwork2.ModelDriven;
import java.util.ArrayList;
import model.dao.DepartamentoDAO;
import model.entities.Departamento;
import model.pdf.ObjectPdf;
import model.util.MessageUtil;

public class DepartamentoController extends ActionSupport implements ModelDriven<Departamento> {

    private Departamento departamento = new Departamento();
    private ArrayList<Departamento> listDepartamento = new ArrayList<Departamento>();
    private DepartamentoDAO DepartamentoDAO = new DepartamentoDAO();
    private String option = null;
    private MessageUtil message = new MessageUtil();
    private String msg = null;

    /**
     * Implementacion del ModelDriven
     *
     * Permite enlazar tags de Struts 2 con el objeto del Action
     *
     * @return Retorna el Objeto del ModelDriven
     */
    @Override
    public Departamento getModel() {
        return departamento;
    }

    /**
     * Getter del Objeto Departamento
     *
     * @return Retorna el cartel
     */
    public Departamento getDepartamento() {
        return departamento;
    }

    /**
     * Setter del Objeto Departamento
     *
     * @param cartel Assigna el Departamento
     */
    public void setDepartamento(Departamento departamento) {
        this.departamento = departamento;
    }

    /**
     * Getter de la lista de Departamentos
     *
     * @return Retorna la lista de Departamentos
     */
    public ArrayList<Departamento> getListDepartamento() {
        return listDepartamento;
    }

    /**
     * Setter de la lista de Departamentos
     *
     * @param listDepartamento Assigna la lista de Departamentos
     */
    public void setListDepartamento(ArrayList<Departamento> listDepartamento) {
        this.listDepartamento = listDepartamento;
    }

    /**
     * Getter de Option
     *
     * @return
     */
    public String getOption() {
        return option;
    }

    /**
     * Setter de Option
     *
     * @param option Opcion Elegida por el Usuario
     */
    public void setOption(String option) {
        this.option = option;
    }

    /**
     * Getter de Mensage
     *
     * @return Retorna el mensage destinado al Usuario
     */
    public String getMsg() {
        return msg;
    }

    /**
     * Setter de Mensage
     *
     * @param msg Mensage destinado al Usuario
     */
    public void setMsg(String msg) {
        this.msg = msg;
    }

    /**
     * Insertar Departamento
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String insertDepartamento() {

        if (this.validateInsert()) {
            if (DepartamentoDAO.insertDepartamento(departamento)) {
                msg = "El Departamento ha sido Insertada Correctamente.";
                departamento = new Departamento();
                return "OK";
            } else {
                msg = "Ha Ocurrido un Error al Insertar la Departamento.";
                message = new MessageUtil(msg);
                return "KO";
            }
        } else {
            msg = "Ha Ocurrido un Error al Validar el Formulario.";
            message = new MessageUtil(msg);
            addActionError(msg);
            return "KO";
        }
    }

    /**
     * Selecciona el Departamento
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectDepartamento() {

        if (this.validateSelect()) {
            Departamento dep = new Departamento();

            if (option.equals("Buscar por ID")) {
                dep = DepartamentoDAO.selectDepartamento(departamento.getIdDepartamento());

            } else {
                listDepartamento = DepartamentoDAO.selectByExample(departamento);

                if (listDepartamento != null) {
                    dep = listDepartamento.get(0);
                }
            }

            if (dep != null) {

                this.departamento.setIdDepartamento(dep.getIdDepartamento());
                this.departamento.setDepartamento(dep.getDepartamento());
                this.departamento.setCodigoDepartamento(dep.getCodigoDepartamento());

                msg = "La Departamento se ha Seleccionado Correctamente.";

                return "OK";
            } else {
                this.departamento = null;
                msg = "Ha Ocurrido un Error al Consultar la Departamento.";
                addActionError(msg);
                msg = "La Departamento no Existe.";
                addFieldError("", msg);
                message = new MessageUtil(msg);
                return "KO";
            }
        } else {
            this.departamento = null;
            msg = "Ha Ocurrido un Error al Validar el Formulario.";
            message = new MessageUtil(msg);
            addActionError(msg);
            return "KO";
        }
    }

    /**
     * Selecciona todos los Departamentoes
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectAllDepartamento() {
        listDepartamento = DepartamentoDAO.selectAll();
        if (listDepartamento.size() > 0) {
            msg = "Las Departamentos se ha Seleccionado Correctamente.";
        } else {
            msg = "La base de datos de Departamentos esta Vacia.";
            message = new MessageUtil(msg);
            return "KO";
        }
        return "OK";
    }

    /**
     * Selecciona los Departamentoes que coinciden con el Ejemplo
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectByExampleDepartamento() {
        if (this.validateExample()) {
            listDepartamento = DepartamentoDAO.selectByExample(departamento);

            if (listDepartamento != null) {
                msg = "La Departamento se ha Seleccionado Correctamente.";
                return "OK";
            } else {
                msg = "No Existe una Departamento con esas Caracteristicas.";
                message = new MessageUtil(msg);
                return "KO";
            }
        } else {
            msg = "Todos los Campos estan Vacios.";
            return this.selectAllDepartamento();
        }
    }

    /**
     * Elimina el Departamento
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String deleteDepartamento() {
        msg = "Desea Eliminar el Departamento??";
        if (message.showConfirmationMessage(msg) != -1) {

            if (DepartamentoDAO.deleteDepartamento(departamento)) {
                msg = "La Departamento ha sido Eliminado Correctamente.";
                message = new MessageUtil(msg);
                departamento = new Departamento();
                return "OK";
            } else {
                msg = "Ha Ocurrido un Error al Eliminar la Departamento.";
                message = new MessageUtil(msg);
                return "KO";
            }
        }
        msg = "Processo Abortado.";
        return "KO";
    }

    /**
     * Modifica el Departamento
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String updateDepartamento() {
        msg = "Desea Modificar el Departamento??";
        if (message.showConfirmationMessage(msg) != -1) {
            if (DepartamentoDAO.updateDepartamento(departamento)) {
                msg = "El Departamento ha sido Modificado Correctamente.";
                message = new MessageUtil(msg);
                return "OK";
            } else {
                msg = "Ha Ocurrido un Error al Modificado la Departamento.";
                message = new MessageUtil(msg);
                return "KO";
            }
        }
        msg = "Processo Abortado.";
        return "KO";
    }

    /**
     * Imprime un Departamento
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printDepartamento() {
        if (departamento != null) {
            ObjectPdf objPdf = new ObjectPdf(this.departamento);

            return "OK";
        } else {
            message = new MessageUtil("No hay datos para imprimir");
            return "KO";
        }
    }

    /**
     * Imprime la lista de Departamentos
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printDepartamentos() {

        if (this.selectByExampleDepartamento().equals("OK")) {

            if (listDepartamento != null) {
                ObjectPdf objPdf = new ObjectPdf(this.listDepartamento);

                return "OK";
            } else {
                message = new MessageUtil("No hay datos para imprimir");
                return "KO";
            }
        } else {
            message = new MessageUtil("No hay datos para imprimir");
            return "KO";
        }
    }

    /**
     * Imprime la lista de Departamentos
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printAllDepartamentos() {
        if (this.selectAllDepartamento().equals("OK")) {

            if (listDepartamento != null) {
                ObjectPdf objPdf = new ObjectPdf(this.listDepartamento);

                return "OK";
            } else {
                message = new MessageUtil("No hay datos para imprimir");
                return "KO";
            }
        } else {
            message = new MessageUtil("No hay datos para imprimir");
            return "KO";
        }
    }

    //-------------------------------------------------------------
    //------------------------VALIDATES----------------------------
    //-------------------------------------------------------------
    /**
     * Valida los campos de Insercion de datos
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateInsert() {
        boolean bool = true;

        if (this.departamento.getCodigoDepartamento().equals("")) {
            addFieldError("", "El campo Codigo Departamento es Obligatorio");
            bool = false;
        }
        return bool;
    }

    /**
     * Valida los Campos de seleccion del Departamento
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateSelect() {
        boolean bool = true;

        departamento.onlyParameter(option);

        if (option.equals("Buscar por ID")) {
            if (this.departamento.getIdDepartamento() == null) {
                addFieldError("", "Introduzca el ID");
                bool = false;
            }
        }

        if (option.equals("Buscar por CodigoDepartamento")) {
            if (this.departamento.getCodigoDepartamento().equals("")) {
                addFieldError("", "Introduzca el Codigo del Departamento");
                bool = false;
            }
        }

        return bool;
    }

    /**
     * Funcion Auxiliar para la validacion de Departamentoes elegidos por
     * Ejemplo
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateExample() {

        if (this.departamento.getDepartamento().equals("")) {
            return false;
        } else {
            return true;
        }
    }

    //-------------------------------------------------------------
    //--------------------------PANTALLAS--------------------------
    //-------------------------------------------------------------
    /**
     * Abrir Pantalla de Seleccion de Departamento
     *
     * @return Retorna "PANTALLA" dando el "OK" para cargar la Pagina
     */
    public String screenSelectDepartamento() {

        if (departamento != null) {
            departamento = new Departamento();
        }
        return "PANTALLA";
    }

    /**
     * Abrir Pantalla de Modificacion de Departamento
     *
     * @return Retorna "PANTALLA" dando el "OK" para cargar la Pagina
     */
    public String screenUpdateDepartamento() {
        return "PANTALLA";
    }

    /**
     * Abrir Pantalla de Seleccion de Departamento mediante Ejemplo
     *
     * @return Retorna "PANTALLA" dando el "OK" para cargar la Pagina
     */
    public String screenSelectByExampleDepartamento() {
        departamento = new Departamento();
        return "PANTALLA";
    }
}
