/**
 * Documento Controller.
 *
 * Responde a eventos referente a la CLASE DOCUMENTO ,invoca peticiones al
 * 'modelo' cuando se hace alguna solicitud sobre la información (por ejemplo,
 * editar un documento o un registro en una base de datos).
 *
 * También puede enviar comandos a su 'vista' asociada si se solicita un cambio
 * en la forma en que se presenta de 'modelo' (por ejemplo, desplazamiento o
 * scroll por un documento o por los diferentes registros de una base de datos),
 * por tanto se podría decir que el 'controlador' hace de intermediario entre la
 * 'vista' y el 'modelo'.
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws Throws in DAO
 */
package controller;

import com.opensymphony.xwork2.ActionSupport;
import com.opensymphony.xwork2.ModelDriven;
import java.util.ArrayList;
import model.dao.DocumentoDAO;
import model.entities.Documento;
import model.pdf.ObjectPdf;
import model.util.MessageUtil;

public class DocumentoController extends ActionSupport implements ModelDriven<Documento> {

    private Documento documento = new Documento();
    private ArrayList<Documento> listDocumento = new ArrayList<Documento>();
    private DocumentoDAO documentoDAO = new DocumentoDAO();
    private String option = null;
    private MessageUtil message = new MessageUtil();
    private String msg = null;

    /**
     * Implementacion del ModelDriven
     *
     * Permite enlazar tags de Struts 2 con el objeto del Action
     *
     * @return Retorna el Objeto del ModelDriven
     */
    @Override
    public Documento getModel() {
        return documento;
    }

    /**
     * Getter del Objeto Documento
     *
     * @return Retorna el cartel
     */
    public Documento getDocumento() {
        return documento;
    }

    /**
     * Setter del Objeto Documento
     *
     * @param cartel Assigna el Documento
     */
    public void setDocumento(Documento documento) {
        this.documento = documento;
    }

    /**
     * Getter de la lista de Documentos
     *
     * @return Retorna la lista de Documentos
     */
    public ArrayList<Documento> getListDocumento() {
        return listDocumento;
    }

    /**
     * Setter de la lista de Documentos
     *
     * @param listDocumento Assigna la lista de Documentos
     */
    public void setListDocumento(ArrayList<Documento> listDocumento) {
        this.listDocumento = listDocumento;
    }

    /**
     * Getter de Mensage
     *
     * @return Retorna el mensage destinado al Usuario
     */
    public String getMsg() {
        return msg;
    }

    /**
     * Setter de Mensage
     *
     * @param msg Mensage destinado al Usuario
     */
    public void setMsg(String msg) {
        this.msg = msg;
    }

    /**
     * Getter de Option
     *
     * @return
     */
    public String getOption() {
        return option;
    }

    /**
     * Setter de Option
     *
     * @param option Opcion Elegida por el Usuario
     */
    public void setOption(String option) {
        this.option = option;
    }

    @Override
    public String toString() {
        return "DocumentoController{" + documento.toString() + '}';
    }

    /**
     * Insertar Documento
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String insertDocumento() {
        if (this.validateInsert()) {
            if (documentoDAO.insertDocumento(documento)) {
                msg = "El Documento ha sido Insertado Correctamente.";

                documento = new Documento();
                return "OK";
            } else {
                msg = "Ha Ocurrido un Error al Insertar el Documento.";
                message = new MessageUtil(msg);
                return "KO";
            }
        } else {
            msg = "Ha Ocurrido un Error al Validar el Formulario.";
            message = new MessageUtil(msg);
            addActionError(msg);
            return "KO";
        }
    }

    /**
     * Selecciona el Documento
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectDocumento() {
        if (this.validateSelect()) {

            if (option.equals("Buscar por ID")) {
                documento = documentoDAO.selectDocumento(documento.getIdDocumento());
                if (documento != null) {
                    listDocumento.add(documento);
                }
            } else {
                listDocumento = documentoDAO.selectByExample(documento);
            }

            if (listDocumento.size() > 0) {
                documento = listDocumento.get(0);
                msg = "El Documento se ha Seleccionado Correctamente.";
                //message = new MessageUtil(msg);
                return "OK";
            } else {
                msg = "El Documento no Existe.";
                message = new MessageUtil(msg);
                addActionError(msg);
                return "KO";
            }
        } else {
            msg = "Ha Ocurrido un Error al Validar el Formulario.";
            message = new MessageUtil(msg);
            addActionError(msg);
            return "KO";
        }
    }

    /**
     * Selecciona todos los Documentoes
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectAllDocumento() {
        listDocumento = documentoDAO.selectAll();
        if (listDocumento.size() > 0) {
            msg = "Los Documentos se ha Seleccionado Correctamente.";
        } else {
            msg = "Ha Ocurrido un Error al Seleccionar los Documentos.";
            message = new MessageUtil(msg);
            return "KO";
        }
        return "OK";
    }

    /**
     * Selecciona los Documentoes que coinciden con el Ejemplo
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectByExampleDocumento() {
        if (this.validateExample()) {
            listDocumento = documentoDAO.selectByExample(documento);

            if (listDocumento.size() > 0) {
                msg = "El Documento se ha Seleccionado Correctamente.";
                return "OK";
            } else {
                msg = "Ha Ocurrido un Error al Seleccionar los Documento.";
                message = new MessageUtil(msg);
                return "KO";
            }
        } else {
            msg = "Todos los Campos estan Vacios.";

            return this.selectAllDocumento();
        }
    }

    /**
     * Elimina el Documento
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String deleteDocumento() {
        msg = "Desea Eliminar el Documento??";
        if (message.showConfirmationMessage(msg) != -1) {
            if (documentoDAO.deleteDocumento(documento)) {
                msg = "El Documento ha sido Eliminado Correctamente.";
                message = new MessageUtil(msg);
                documento = new Documento();
                return "OK";
            } else {
                msg = "Ha Ocurrido un Error al Eliminar el Documento.";
                message = new MessageUtil(msg);
                return "KO";
            }
        }
        msg = "Processo Abortado.";
        return "KO";
    }

    /**
     * Modifica el Documento
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String updateDocumento() {
        msg = "Desea Modificar el Documento??";
        if (message.showConfirmationMessage(msg) != -1) {
            if (documentoDAO.updateDocumento(documento)) {
                msg = "El Documento ha sido Modificado Correctamente.";
                message = new MessageUtil(msg);
                return "OK";
            } else {
                msg = "Ha Ocurrido un Error al Modificado el Documento.";
                message = new MessageUtil(msg);
                return "KO";
            }
        }
        msg = "Processo Abortado.";
        return "KO";
    }

    /**
     * Imprime un Documento
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printDocumento() {
        if (documento != null) {
            ObjectPdf objPdf = new ObjectPdf(this.documento);

            return "OK";
        } else {

            message = new MessageUtil("No hay datos para imprimir");
            return "KO";
        }
    }

    /**
     * Imprime la lista de Documentos
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printDocumentos() {

        if (this.selectByExampleDocumento().equals("OK")) {
            if (listDocumento != null) {
                ObjectPdf objPdf = new ObjectPdf(this.listDocumento);

                return "OK";
            } else {

                message = new MessageUtil("No hay datos para imprimir");
                return "KO";
            }
        } else {
            message = new MessageUtil("No hay datos para imprimir");
            return "KO";
        }
    }

    /**
     * Imprime la lista de Documentos
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printAllDocumentos() {
        if (this.selectAllDocumento().equals("OK")) {
            if (listDocumento != null) {
                ObjectPdf objPdf = new ObjectPdf(this.listDocumento);

                return "OK";
            } else {

                message = new MessageUtil("No hay datos para imprimir");
                return "KO";
            }
        } else {
            message = new MessageUtil("No hay datos para imprimir");
            return "KO";
        }
    }

    //-------------------------------------------------------------
    //------------------------VALIDATES----------------------------
    //-------------------------------------------------------------
    /**
     * Valida los campos de Insercion de datos
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateInsert() {
        boolean bool = true;

        if (this.documento.getNumeroDocumento().equals("")) {
            addFieldError("", "El campo NumeroDocumento es Obligatorio");
            bool = false;
        }

        if (this.documento.getTipoDocumento().equals("")) {
            addFieldError("", "El campo Tipo Documento es Obligatorio");
            bool = false;
        }

        return bool;
    }

    /**
     * Funcion Auxiliar para la Validacion de Documentoes
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateSelect() {
        boolean bool = true;

        documento.onlyParameter(option);

        if (option.equals("Buscar por ID")) {
            if (this.documento.getIdDocumento() == null) {
                addFieldError("", "Introduzca el ID");
                bool = false;
            }
        }

        if (option.equals("Buscar por NumeroDocumento")) {
            if (this.documento.getNumeroDocumento().equals("")) {
                addFieldError("", "Introduzca la NumeroDocumento");
                bool = false;
            }
        }

        if (option.equals("Buscar por Tipo Documento")) {
            if (this.documento.getTipoDocumento().equals("")) {
                addFieldError("", "Introduzca el Tipo de Documento");
                bool = false;
            }
        }

        return bool;
    }

    /**
     * Funcion Auxiliar para la validacion de Documentoes elegidos por Ejemplo
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateExample() {
        boolean bool = true;
        int n = 0;

        if (this.documento.getNumeroDocumento().equals("")) {
            n = n + 1;
            documento.setNumeroDocumento(null);
        }

        if (this.documento.getTipoDocumento().equals("")) {
            n = n + 1;
            documento.setTipoDocumento(null);
        }

        if (n == 2) {
            bool = false;
        }
        return bool;
    }

    //-------------------------------------------------------------
    //--------------------------PANTALLAS--------------------------
    //-------------------------------------------------------------
    /**
     * Abrir Pantalla de Seleccion de Documento
     *
     * @return Retorna "PANTALLA" dando el "OK" para cargar la Pagina
     */
    public String screenSelectDocumento() {
        documento = new Documento();
        return "PANTALLA";
    }

    /**
     * Abrir Pantalla de Modificacion de Documento
     *
     * @return Retorna "PANTALLA" dando el "OK" para cargar la Pagina
     */
    public String screenUpdateDocumento() {
        return "PANTALLA";
    }

    /**
     * Abrir Pantalla de Seleccion de Documento mediante Ejemplo
     *
     * @return Retorna "PANTALLA" dando el "OK" para cargar la Pagina
     */
    public String screenSelectByExampleDocumento() {
        documento = new Documento();
        return "PANTALLA";
    }
}
