/**
 * Incidencia Controller.
 *
 * Responde a eventos referente a la CLASE INCIDENCIA ,invoca peticiones al
 * 'modelo' cuando se hace alguna solicitud sobre la información (por ejemplo,
 * editar un documento o un registro en una base de datos).
 *
 * También puede enviar comandos a su 'vista' asociada si se solicita un cambio
 * en la forma en que se presenta de 'modelo' (por ejemplo, desplazamiento o
 * scroll por un documento o por los diferentes registros de una base de datos),
 * por tanto se podría decir que el 'controlador' hace de intermediario entre la
 * 'vista' y el 'modelo'.
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws Throws in DAO
 */
package controller;

import com.opensymphony.xwork2.ActionContext;
import com.opensymphony.xwork2.ActionSupport;
import com.opensymphony.xwork2.ModelDriven;
import java.util.ArrayList;
import java.util.Date;
import java.util.Map;
import model.dao.IncidenciaDAO;
import model.dao.LoginDAO;
import model.dao.UsuarioDAO;
import model.entities.Incidencia;
import model.entities.Login;
import model.entities.Usuario;
import model.pdf.ObjectPdf;
import model.util.MessageUtil;

public class IncidenciaController extends ActionSupport implements ModelDriven<Incidencia> {

    private Incidencia incidencia = new Incidencia();
    private ArrayList<Incidencia> listIncidencia = new ArrayList<Incidencia>();
    private IncidenciaDAO incidenciaDAO = new IncidenciaDAO();
    private Usuario current = null;
    private String dest = null;
    private String ori = null;
    private String option = null;
    private MessageUtil message = new MessageUtil();
    private String msg = null;

    /**
     * Implementacion del ModelDriven
     *
     * Permite enlazar tags de Struts 2 con el objeto del Action
     *
     * @return Retorna el Objeto del ModelDriven
     */
    @Override
    public Incidencia getModel() {
        return incidencia;
    }

    /**
     * Getter del Objeto Incidencia
     *
     * @return Retorna el incidencia
     */
    public Incidencia getIncidencia() {
        return incidencia;
    }

    /**
     * Setter del Objeto Incidencia
     *
     * @param incidencia Assignala Incidencia
     */
    public void setIncidencia(Incidencia incidencia) {
        this.incidencia = incidencia;
    }

    /**
     * Getter de la lista de Incidencias
     *
     * @return Retorna la lista de Incidencias
     */
    public ArrayList<Incidencia> getListIncidencia() {
        return listIncidencia;
    }

    /**
     * Setter de la lista de Incidencias
     *
     * @param listIncidencia Assigna la lista de Incidencias
     */
    public void setListIncidencia(ArrayList<Incidencia> listIncidencia) {
        this.listIncidencia = listIncidencia;
    }

    /**
     * Getter del Usuario Logueado
     *
     * @return Retorna el Usuario Logueado
     */
    public Usuario getCurrent() {
        return current;
    }

    /**
     * Setter del Usuario Logeado
     *
     * @param current Retorna el Usuario Logueado.
     */
    public void setCurrent(Usuario current) {
        this.current = current;
    }

    /**
     * Getter de Option
     *
     * @return
     */
    public String getOption() {
        return option;
    }

    /**
     * Setter de Option
     *
     * @param option Opcion Elegida por el Usuario
     */
    public void setOption(String option) {
        this.option = option;
    }

    /**
     * Getter de Mensage
     *
     * @return Retorna el mensage destinado al Usuario
     */
    public String getMsg() {
        return msg;
    }

    /**
     * Setter de Mensage
     *
     * @param msg Mensage destinado al Usuario
     */
    public void setMsg(String msg) {
        this.msg = msg;
    }

    /**
     * Getter del Usuario destino
     *
     * @return Retorna el Usuario Destino
     */
    public String getDest() {
        return dest;
    }

    /**
     * Setter del Usuario Destino
     *
     * @param dest Usuario Destino
     */
    public void setDest(String dest) {
        this.dest = dest;
    }

    /**
     * Getter del Usuario Origen
     *
     * @return Retorna el Usuario Origen
     */
    public String getOri() {
        return ori;
    }

    /**
     * Setter del Usuario Origen
     *
     * @param ori Usuario Origen
     */
    public void setOri(String ori) {
        this.ori = ori;
    }

    /**
     * Insertar Incidencia
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String insertIncidencia() {

        if (this.validateInsert()) {

            incidencia.setEstado("A");

            Date d = new Date();
            incidencia.setDataApertura(d);

            if (incidenciaDAO.insertIncidencia(incidencia)) {
                msg = "La Incidencia ha sido Insertada Correctamente.";
                incidencia = new Incidencia();
                return "OK";
            } else {
                msg = "Ha Ocurrido un Error al Insertar la Incidencia.";
                message = new MessageUtil(msg);
                return "KO";
            }
        } else {
            this.incidencia = new Incidencia();
            msg = "Ha Ocurrido un Error al Validar el Formulario.";
            message = new MessageUtil(msg);
            addActionError(msg);
            return "KO";
        }
    }

    /**
     * Valida los Campos de seleccion de la Incidencia
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateSelect() {
        if (this.incidencia.getIdIncidencia() == null) {
            addFieldError("", "Introduzca el ID");
            return false;
        } else {
            return true;
        }
    }

    /**
     * Seleccionala Incidencia
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectIncidencia() {
        if (this.validateSelect()) {

            Incidencia inci = incidenciaDAO.selectIncidencia(incidencia.getIdIncidencia());

            if (inci != null) {
                incidencia = inci;
                msg = "La Incidencia se ha Seleccionado Correctamente.";
                return "OK";
            } else {
                this.incidencia = null;
                msg = "Ha Ocurrido un Error al Consultar la Incidencia.";
                addActionError(msg);
                msg = "La Incidencia no Existe.";
                addFieldError("", msg);
                message = new MessageUtil(msg);
                return "KO";
            }
        } else {
            this.incidencia = null;
            msg = "Ha Ocurrido un Error al Validar el Formulario.";
            message = new MessageUtil(msg);
            addActionError(msg);
            return "KO";
        }
    }

    /**
     * Seleccionala Incidencia del Administrador
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectAdmIncidencia() {
        incidencia = new Incidencia();
        if (option.equals("Origen")) {
            incidencia.setOrigen("A");
        } else {
            incidencia.setDestino("A");
        }
        listIncidencia = incidenciaDAO.selectByExample(incidencia);

        if (listIncidencia.size() > 0) {
            msg = "La Incidencia se ha Seleccionado Correctamente.";
            return "OK";
        } else {
            msg = "No Existe una Incidencia con esas Caracteristicas.";
            message = new MessageUtil(msg);
            return "KO";
        }
    }

    /**
     * Selecciona todos los Incidenciaes
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectAllIncidencia() {
        listIncidencia = incidenciaDAO.selectAll();
        if (listIncidencia.size() > 0) {
            msg = "Las Incidencias se ha Seleccionado Correctamente.";
        } else {
            msg = "La base de datos de Incidencias esta Vacia.";
            message = new MessageUtil(msg);
            return "KO";
        }
        return "OK";
    }

    /**
     * Selecciona los Incidenciaes que coinciden con el Ejemplo
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectByExampleIncidencia() {
        if (this.validateExample()) {
            listIncidencia = incidenciaDAO.selectByExample(incidencia);

            if (listIncidencia.size() > 0) {
                msg = "La Incidencia se ha Seleccionado Correctamente.";
                return "OK";
            } else {
                incidencia = new Incidencia();
                msg = "No Existe una Incidencia con esas Caracteristicas.";
                message = new MessageUtil(msg);
                return "KO";
            }
        } else {
            msg = "Todos los Campos estan Vacios.";
            message = new MessageUtil(msg);
            addActionError(msg);
            return "KO";
        }
    }

    /**
     * Eliminala Incidencia
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String deleteIncidencia() {
        msg = "Desea Eliminar el Incidencia??";
        if (message.showConfirmationMessage(msg) != -1) {

            if (incidenciaDAO.deleteIncidencia(incidencia)) {
                msg = "La Incidencia ha sido Eliminado Correctamente.";
                message = new MessageUtil(msg);
                incidencia = new Incidencia();
                return "OK";
            } else {
                msg = "Ha Ocurrido un Error al Eliminar la Incidencia.";
                message = new MessageUtil(msg);
                return "KO";
            }
        }
        msg = "Processo Abortado.";
        return "KO";
    }

    /**
     * Imprime una incidencia
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printIncidencia() {
        if (incidencia != null) {
            ObjectPdf objPdf = new ObjectPdf(this.incidencia);

            return "OK";
        } else {

            message = new MessageUtil("No hay datos para imprimir");
            return "KO";
        }
    }

    /**
     * Imprime la lista de Incidencias
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printIncidencias() {
        if (this.selectByExampleIncidencia().equals("OK")) {
            if (listIncidencia != null) {
                ObjectPdf objPdf = new ObjectPdf(this.listIncidencia);

                return "OK";
            } else {

                message = new MessageUtil("No hay datos para imprimir");
                return "KO";
            }
        } else {
            message = new MessageUtil("No hay datos para imprimir");
            return "KO";
        }
    }

    /**
     * Imprime la lista de Incidencias
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printAllIncidencias() {
        if (this.printAllIncidencias().equals("OK")) {
            if (listIncidencia != null) {
                ObjectPdf objPdf = new ObjectPdf(this.listIncidencia);

                return "OK";
            } else {

                message = new MessageUtil("No hay datos para imprimir");
                return "KO";
            }
        } else {
            message = new MessageUtil("No hay datos para imprimir");
            return "KO";
        }
    }

    /**
     * Imprime la lista de Incidencias del Administrador
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printAdmIncidencia() {
        if (this.selectAdmIncidencia().equals("OK")) {
            if (listIncidencia != null) {
                ObjectPdf objPdf = new ObjectPdf(this.listIncidencia);

                return "OK";
            } else {

                message = new MessageUtil("No hay datos para imprimir");
                return "KO";
            }
        } else {
            message = new MessageUtil("No hay datos para imprimir");
            return "KO";
        }
    }

    /**
     * Imprime la lista de Incidencias del Responsable
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printRespIncidencias() {
        if (this.selectByExampleRespIncidencia().equals("OK")) {
            if (listIncidencia != null) {
                ObjectPdf objPdf = new ObjectPdf(this.listIncidencia);

                return "OK";
            } else {

                message = new MessageUtil("No hay datos para imprimir");
                return "KO";
            }
        } else {
            message = new MessageUtil("No hay datos para imprimir");
            return "KO";
        }
    }

    /**
     * Imprime la lista de Incidencias del Usuario
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printUserIncidencias() {
        if (this.selectByExampleUserIncidencia().equals("OK")) {
            if (listIncidencia != null) {
                ObjectPdf objPdf = new ObjectPdf(this.listIncidencia);
                return "OK";
            } else {

                message = new MessageUtil("No hay datos para imprimir");
                return "KO";
            }
        } else {
            message = new MessageUtil("No hay datos para imprimir");
            return "KO";
        }
    }

    /**
     * Seleccionala Incidencia del Usuario
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectUserIncidencia() {
        return selectTipoIncidencia("U");
    }

    /**
     * Seleccionala Incidencia del Responsable
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectRespIncidencia() {
        return selectTipoIncidencia("R");
    }

    /**
     * Seleccionala Incidencia por Tipo de Usuario
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    private String selectTipoIncidencia(String tipoUsuario) {
        if (this.validateSelect()) {

            Incidencia inci = incidenciaDAO.selectIncidencia(incidencia.getIdIncidencia());

            if (inci != null) {
                if (inci.getDestino().equals(tipoUsuario) || inci.getOrigen().equals(tipoUsuario)) {
                    incidencia = inci;
                    msg = "La Incidencia se ha Seleccionado Correctamente.";
                    return "OK";
                } else {
                    this.incidencia = null;
                    msg = "Esta incidencia no esta Relacionada con Usted.";
                    message = new MessageUtil(msg);
                    addActionError(msg);
                    return "KO";
                }
            } else {
                this.incidencia = null;
                msg = "Ha Ocurrido un Error al Consultar la Incidencia.";
                addActionError(msg);
                msg = "La Incidencia no Existe.";
                addFieldError("", msg);
                message = new MessageUtil(msg);
                return "KO";
            }
        } else {
            this.incidencia = null;
            msg = "Ha Ocurrido un Error al Validar el Formulario.";
            message = new MessageUtil(msg);
            addActionError(msg);
            return "KO";
        }
    }

    /**
     * Seleccionala Incidencia del Responsable
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectRespIncidencias() {
        incidencia = new Incidencia();
        if (option.equals("Origen")) {
            incidencia.setOrigen("R");
        } else {
            incidencia.setDestino("R");
        }
        listIncidencia = incidenciaDAO.selectByExample(incidencia);

        if (listIncidencia.size() > 0) {
            msg = "La Incidencia se ha Seleccionado Correctamente.";
            return "OK";
        } else {
            msg = "No Existe una Incidencia con esas Caracteristicas.";
            message = new MessageUtil(msg);
            return "KO";
        }
    }

    /**
     * Seleccionala Incidencia del Usuario
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectUserIncidencias() {
        incidencia = new Incidencia();
        if (option.equals("Origen")) {
            incidencia.setOrigen("U");
        } else {
            incidencia.setDestino("U");
        }
        listIncidencia = incidenciaDAO.selectByExample(incidencia);

        if (listIncidencia.size() > 0) {
            msg = "La Incidencia se ha Seleccionado Correctamente.";
            return "OK";
        } else {
            msg = "No Existe una Incidencia con esas Caracteristicas.";
            message = new MessageUtil(msg);
            return "KO";
        }
    }

    /**
     * Selecciona los Incidenciaes del Responsable que coinciden con el Ejemplo
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectByExampleRespIncidencia() {

        if (this.validateRespExample()) {
            listIncidencia = incidenciaDAO.selectByExample(incidencia);

            if (listIncidencia.size() > 0) {
                msg = "La Incidencia se ha Seleccionado Correctamente.";
                return "OK";
            } else {
                incidencia = new Incidencia();
                msg = "No Existe una Incidencia con esas Caracteristicas.";
                message = new MessageUtil(msg);
                return "KO";
            }
        } else {
            msg = "Todos los Campos estan Vacios.";
            message = new MessageUtil(msg);
            addActionError(msg);
            return "KO";
        }
    }

    /**
     * Selecciona los Incidenciaes del Usuario que coinciden con el Ejemplo
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectByExampleUserIncidencia() {

        if (this.validateUserExample()) {
            listIncidencia = incidenciaDAO.selectByExample(incidencia);

            if (listIncidencia.size() > 0) {
                msg = "La Incidencia se ha Seleccionado Correctamente.";
                return "OK";
            } else {
                incidencia = new Incidencia();
                msg = "No Existe una Incidencia con esas Caracteristicas.";
                message = new MessageUtil(msg);
                return "KO";
            }
        } else {
            msg = "Todos los Campos estan Vacios.";
            message = new MessageUtil(msg);
            addActionError(msg);
            return "KO";
        }
    }

    /**
     * Cierra una Incidencia
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String closeIncidencia() {

        if (validateClose()) {
            msg = "Desea Modificar el Incidencia??";
            if (message.showConfirmationMessage(msg) != -1) {
                if (incidenciaDAO.updateIncidencia(incidencia)) {
                    msg = "El Incidencia ha sido Modificado Correctamente.";
                    message = new MessageUtil(msg);
                    return "OK";
                } else {
                    msg = "Ha Ocurrido un Error al Modificado la Incidencia.";
                    message = new MessageUtil(msg);
                    return "KO";
                }
            }
        } else {
            msg = "Ha Ocurrido un Error al Validar el Formulario.";
            message = new MessageUtil(msg);
            return "KO";
        }
        msg = "Processo Abortado.";
        return "KO";
    }

    //-------------------------------------------------------------
    //------------------------VALIDATES----------------------------
    //-------------------------------------------------------------
    /**
     * Valida los campos de Insercion de datos
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateInsert() {
        boolean bool = true;

        if (dest.equals("Administrador")) {
            incidencia.setDestino("A");
        } else if (dest.equals("Responsable")) {
            incidencia.setDestino("R");
        } else {
            LoginDAO loginDAO = new LoginDAO();
            Login log = loginDAO.selectLogin(incidencia.getDestino());

            if (log != null) {
                UsuarioDAO userDAO = new UsuarioDAO();
                Usuario user = userDAO.selectUsuario(log);

                incidencia.setDestino(Integer.toString(user.getIdUsuario()));
            } else {
                addFieldError("", "El usuario Destino no Existe.");
                bool = false;
            }
        }

        if (incidencia.getOrigen() == null) {
            Map session = ActionContext.getContext().getSession();
            incidencia.setOrigen((String) session.get("usuario"));
        }

        if (this.incidencia.getOrigen().equals("")) {
            addFieldError("", "El campo Origen es Obligatorio.");
            bool = false;
        } else {

            LoginDAO loginDAO = new LoginDAO();
            Login log = loginDAO.selectLogin(incidencia.getOrigen());

            if (log != null) {
                UsuarioDAO userDAO = new UsuarioDAO();
                Usuario user = userDAO.selectUsuario(log);

                if (user.getTipoUsuario().equals("A")) {
                    incidencia.setOrigen("A");
                } else if (user.getTipoUsuario().equals("R")) {
                    incidencia.setOrigen("R");
                } else {
                    incidencia.setOrigen(Integer.toString(user.getIdUsuario()));
                }
            } else {
                addFieldError("", "El usuario Origen no Existe.");
                bool = false;
            }
        }


        if (this.incidencia.getAsunto().equals("")) {
            addFieldError("", "El campo Asunto es Obligatorio.");
            bool = false;
        }

        if (this.incidencia.getIncidencia().equals("")) {
            addFieldError("", "El campo Incidencia es Obligatorio.");
            bool = false;
        }

        return bool;
    }

    /**
     * Valida los campos de Cierre de Incidencia
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateClose() {
        if (this.incidencia.getRespuesta().equals("")) {
            addFieldError("", "Debe Introducir una Respuesta.");
            return false;
        } else {
            if (this.incidencia.getEstado().equals("C")) {
                addFieldError("", "La Incidencia Ya esta Cerrada.");
                return false;
            } else {
                return true;
            }
        }
    }

    /**
     * Funcion Auxiliar para la validacion de Incidenciaes del Responsable
     * elegidas por Ejemplo
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateRespExample() {
        boolean bool = true;
        int n = 0;

        this.incidencia.setIncidencia(null);
        this.incidencia.setRespuesta(null);

        if (this.incidencia.getEstado().equals("")) {
            this.incidencia.setEstado(null);
            n = n + 1;
        }

        if (dest.equals("")) {
            this.incidencia.setDestino(null);
            n = n + 1;
        } else if (dest.equals("Administrador")) {
            incidencia.setDestino("A");
            incidencia.setOrigen("R");
        } else if (dest.equals("Responsable")) {
            incidencia.setDestino("R");
        } else {
            incidencia.setOrigen("R");

            if (this.incidencia.getDestino().equals("")) {
                addFieldError("", "Debe introducir el Usuario Destino.");
                bool = false;
            } else {

                LoginDAO loginDAO = new LoginDAO();
                Login log = loginDAO.selectLogin(incidencia.getDestino());

                if (log != null) {
                    UsuarioDAO userDAO = new UsuarioDAO();
                    Usuario user = userDAO.selectUsuario(log);

                    incidencia.setDestino(Integer.toString(user.getIdUsuario()));
                } else {
                    addFieldError("", "El usuario Destino no Existe.");
                    bool = false;
                }
            }
        }

        if (ori.equals("")) {
            this.incidencia.setOrigen(null);
            n = n + 1;
        } else if (ori.equals("Administrador")) {
            incidencia.setOrigen("A");
            incidencia.setDestino("R");
        } else if (ori.equals("Responsable")) {
            incidencia.setOrigen("R");
        } else {
            incidencia.setDestino("R");

            if (this.incidencia.getOrigen().equals("")) {
                addFieldError("", "Debe introducir el Usuario Origen.");
                bool = false;
            } else {
                LoginDAO loginDAO = new LoginDAO();
                Login log = loginDAO.selectLogin(incidencia.getOrigen());

                if (log != null) {
                    UsuarioDAO userDAO = new UsuarioDAO();
                    Usuario user = userDAO.selectUsuario(log);

                    incidencia.setOrigen(Integer.toString(user.getIdUsuario()));
                } else {
                    addFieldError("", "El usuario Origen no Existe.");
                    bool = false;
                }
            }
        }

        if (this.incidencia.getAsunto().equals("")) {
            this.incidencia.setAsunto(null);
            n = n + 1;
        }

        if (this.incidencia.getDataApertura() == null) {
            n = n + 1;
        }

        if (this.incidencia.getDataCierre() == null) {
            n = n + 1;
        }

        if (n == 6) {
            addFieldError("", "Rellene almenos algun Campo.");
            bool = false;
        }

        return bool;
    }

    /**
     * Funcion Auxiliar para la validacion de Incidenciaes del Usuario elegidas
     * por Ejemplo
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateUserExample() {
        boolean bool = true;
        int n = 0;

        this.incidencia.setIncidencia(null);
        this.incidencia.setRespuesta(null);

        if (this.incidencia.getEstado().equals("")) {
            this.incidencia.setEstado(null);
            n = n + 1;
        }

        if (dest.equals("")) {
            this.incidencia.setDestino(null);
            n = n + 1;
        } else if (dest.equals("Administrador")) {
            incidencia.setDestino("A");
            incidencia.setOrigen("U");
        } else if (dest.equals("Responsable")) {
            incidencia.setOrigen("U");
            incidencia.setDestino("R");
        } else {
            incidencia.setOrigen("U");

            if (this.incidencia.getDestino().equals("")) {
                addFieldError("", "Debe introducir el Usuario Destino.");
                bool = false;
            } else {

                LoginDAO loginDAO = new LoginDAO();
                Login log = loginDAO.selectLogin(incidencia.getDestino());

                if (log != null) {
                    UsuarioDAO userDAO = new UsuarioDAO();
                    Usuario user = userDAO.selectUsuario(log);

                    incidencia.setDestino(Integer.toString(user.getIdUsuario()));
                } else {
                    addFieldError("", "El usuario Destino no Existe.");
                    bool = false;
                }
            }
        }

        if (ori.equals("")) {
            this.incidencia.setOrigen(null);
            n = n + 1;
        } else if (ori.equals("Administrador")) {
            incidencia.setOrigen("A");
            incidencia.setDestino("U");
        } else if (ori.equals("Responsable")) {
            incidencia.setOrigen("U");
        } else {
            incidencia.setDestino("U");

            if (this.incidencia.getOrigen().equals("")) {
                addFieldError("", "Debe introducir el Usuario Origen.");
                bool = false;
            } else {
                LoginDAO loginDAO = new LoginDAO();
                Login log = loginDAO.selectLogin(incidencia.getOrigen());

                if (log != null) {
                    UsuarioDAO userDAO = new UsuarioDAO();
                    Usuario user = userDAO.selectUsuario(log);

                    incidencia.setOrigen(Integer.toString(user.getIdUsuario()));
                } else {
                    addFieldError("", "El usuario Origen no Existe.");
                    bool = false;
                }
            }
        }

        if (this.incidencia.getAsunto().equals("")) {
            this.incidencia.setAsunto(null);
            n = n + 1;
        }

        if (this.incidencia.getDataApertura() == null) {
            n = n + 1;
        }

        if (this.incidencia.getDataCierre() == null) {
            n = n + 1;
        }

        if (n == 6) {
            addFieldError("", "Rellene almenos algun Campo.");
            bool = false;
        }

        return bool;
    }

    /**
     * Funcion Auxiliar para la validacion de Incidenciaes elegidos por Ejemplo
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateExample() {
        boolean bool = true;
        int n = 0;

        this.incidencia.setIncidencia(null);
        this.incidencia.setRespuesta(null);

        if (this.incidencia.getEstado().equals("")) {
            this.incidencia.setEstado(null);
            n = n + 1;
        }

        if (dest.equals("")) {
            this.incidencia.setDestino(null);
            n = n + 1;
        } else if (dest.equals("Administrador")) {
            incidencia.setDestino("A");
        } else if (dest.equals("Responsable")) {
            incidencia.setDestino("R");
        } else {
            if (this.incidencia.getDestino().equals("")) {
                addFieldError("", "Debe introducir el Usuario Destino.");
                bool = false;
            } else {
                LoginDAO loginDAO = new LoginDAO();
                Login log = loginDAO.selectLogin(incidencia.getDestino());

                if (log != null) {
                    UsuarioDAO userDAO = new UsuarioDAO();
                    Usuario user = userDAO.selectUsuario(log);

                    incidencia.setDestino(Integer.toString(user.getIdUsuario()));
                } else {
                    addFieldError("", "El usuario Destino no Existe.");
                    bool = false;
                }
            }
        }

        if (ori.equals("")) {
            this.incidencia.setOrigen(null);
            n = n + 1;
        } else if (ori.equals("Administrador")) {
            incidencia.setOrigen("A");
        } else if (ori.equals("Responsable")) {
            incidencia.setOrigen("R");
        } else {
            if (this.incidencia.getOrigen().equals("")) {
                addFieldError("", "Debe introducir el Usuario Origen.");
                bool = false;
            } else {
                LoginDAO loginDAO = new LoginDAO();
                Login log = loginDAO.selectLogin(incidencia.getOrigen());

                if (log != null) {
                    UsuarioDAO userDAO = new UsuarioDAO();
                    Usuario user = userDAO.selectUsuario(log);

                    incidencia.setOrigen(Integer.toString(user.getIdUsuario()));
                } else {
                    addFieldError("", "El usuario Origen no Existe.");
                    bool = false;
                }
            }
        }

        if (this.incidencia.getAsunto().equals("")) {
            this.incidencia.setAsunto(null);
            n = n + 1;
        }

        if (this.incidencia.getDataApertura() == null) {
            n = n + 1;
        }

        if (this.incidencia.getDataCierre() == null) {
            n = n + 1;
        }

        if (n == 6) {
            addFieldError("", "Rellene almenos algun Campo.");
            bool = false;
        }

        return bool;
    }

    //-------------------------------------------------------------
    //--------------------------PANTALLAS--------------------------
    //-------------------------------------------------------------
    /**
     * Abrir Pantalla de Seleccion de Incidencia
     *
     * @return Retorna "PANTALLA" dando el "OK" para cargar la Pagina
     */
    public String screenSelectIncidencia() {
        if (incidencia != null) {
            incidencia = new Incidencia();
        }
        return "PANTALLA";
    }

    /**
     * Abrir Pantalla de Seleccion de Incidencia del Responsable
     *
     * @return Retorna "PANTALLA" dando el "OK" para cargar la Pagina
     */
    public String screenSelectRespIncidencia() {
        if (incidencia != null) {
            incidencia = new Incidencia();
        }
        return "PANTALLA";
    }

    /**
     * Abrir Pantalla de Seleccion de Incidencia del Administrador
     *
     * @return Retorna "PANTALLA" dando el "OK" para cargar la Pagina
     */
    public String screenSelectAdmIncidencia() {
        if (incidencia != null) {
            incidencia = new Incidencia();
        }
        return "PANTALLA";
    }

    /**
     * Abrir Pantalla de Seleccion de Incidencia del Usuario
     *
     * @return Retorna "PANTALLA" dando el "OK" para cargar la Pagina
     */
    public String screenSelectUserIncidencia() {
        if (incidencia != null) {
            incidencia = new Incidencia();
        }
        return "PANTALLA";
    }

    /**
     * Abrir Pantalla de Seleccion de Incidencias del Administrador
     *
     * @return Retorna "PANTALLA" dando el "OK" para cargar la Pagina
     */
    public String screenSelectAdmIncidencias() {
        if (incidencia != null) {
            incidencia = new Incidencia();
        }
        return "PANTALLA";
    }

    /**
     * Abrir Pantalla de Seleccion de Incidencias del Usuario
     *
     * @return Retorna "PANTALLA" dando el "OK" para cargar la Pagina
     */
    public String screenSelectUserIncidencias() {
        if (incidencia != null) {
            incidencia = new Incidencia();
        }
        return "PANTALLA";
    }

    /**
     * Abrir Pantalla de Seleccion de Incidencias del Responsable
     *
     * @return Retorna "PANTALLA" dando el "OK" para cargar la Pagina
     */
    public String screenSelectRespIncidencias() {
        if (incidencia != null) {
            incidencia = new Incidencia();
        }
        return "PANTALLA";
    }

    /**
     * Abrir Pantalla de Seleccion de Incidencias mediante Ejemplo
     *
     * @return Retorna "PANTALLA" dando el "OK" para cargar la Pagina
     */
    public String screenSelectByExampleIncidencia() {
        if (incidencia != null) {
            incidencia = new Incidencia();
        }
        return "PANTALLA";
    }

    /**
     * Abrir Pantalla de Insercion de Incidencias
     *
     * @return Retorna "PANTALLA" dando el "OK" para cargar la Pagina
     */
    public String screenInsertIncidencia() {

        this.getCurrentUser();

        return "PANTALLA";
    }

    /**
     * Funcion Auxiliar para insertar el Usuario en la Sesion
     *
     */
    private void getCurrentUser() {
        Map session = ActionContext.getContext().getSession();

        LoginDAO logDAO = new LoginDAO();
        Login log = logDAO.selectLogin((String) session.get("usuario"));

        UsuarioDAO userDAO = new UsuarioDAO();
        current = userDAO.selectUsuario(log);
    }
}
