/**
 * Trabajo Controller.
 *
 * Responde a eventos referente a la CLASE TRABAJO ,invoca peticiones al
 * 'modelo' cuando se hace alguna solicitud sobre la información (por ejemplo,
 * editar un documento o un registro en una base de datos).
 *
 * También puede enviar comandos a su 'vista' asociada si se solicita un cambio
 * en la forma en que se presenta de 'modelo' (por ejemplo, desplazamiento o
 * scroll por un documento o por los diferentes registros de una base de datos),
 * por tanto se podría decir que el 'controlador' hace de intermediario entre la
 * 'vista' y el 'modelo'.
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws Throws in DAO
 */
package controller;

import com.opensymphony.xwork2.ActionContext;
import com.opensymphony.xwork2.ActionSupport;
import com.opensymphony.xwork2.ModelDriven;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;
import java.util.StringTokenizer;
import model.dao.ArticuloDAO;
import model.dao.CartelDAO;
import model.dao.EstadotrabajoDAO;
import model.dao.LoginDAO;
import model.dao.PlantillaDAO;
import model.dao.TrabajoDAO;
import model.dao.UsuarioDAO;
import model.entities.Articulo;
import model.entities.Cartel;
import model.entities.Estadotrabajo;
import model.entities.Login;
import model.entities.Plantilla;
import model.entities.Trabajo;
import model.entities.Usuario;
import model.util.MessageUtil;
import model.pdf.CartelPdf;
import model.pdf.ObjectPdf;

/**
 *
 * @author Portatil
 */
public class TrabajoController extends ActionSupport implements ModelDriven<Cartel> {

    private Trabajo trabajo = new Trabajo();
    private ArrayList<Trabajo> listTrabajo = new ArrayList<Trabajo>();
    private TrabajoDAO trabajoDAO = new TrabajoDAO();
    private Cartel cartel = new Cartel();
    private Articulo art = new Articulo();
    private ArrayList<Cartel> listCartel = new ArrayList<Cartel>();
    private CartelDAO carDAO = new CartelDAO();
    private String selected = new String();
    private ArrayList<String> listTipo = new ArrayList<String>();
    private ArrayList<String> listTamano = new ArrayList<String>();
    private String option = null;
    private MessageUtil message = new MessageUtil();
    private String msg = null;
    private Usuario current = null;

    /**
     * Implementacion del ModelDriven
     *
     * Permite enlazar tags de Struts 2 con el objeto del Action
     *
     * @return Retorna el Objeto del ModelDriven
     */
    @Override
    public Cartel getModel() {
        return cartel;
    }

    /**
     * Getter del Articulo
     * 
     * @return Retorna el Articulo.
     */
    public Articulo getArt() {
        return art;
    }

    /**
     * Setter del Articulo
     * 
     * @param art Assigna el Articulo.
     */
    public void setArt(Articulo art) {
        this.art = art;
    }

    /**
     * Getter del Trabajo/s seleccionado
     * 
     * @return Retorna el trabajo/s seleccionados.
     */
    public String getSelected() {
        return selected;
    }

    /**
     * Setter del Trabajo/s seleccionado
     * 
     * @param selected Assigna el trabajo/s seleccionados.
     */
    public void setSelected(String selected) {
        this.selected = selected;
    }

    /**
     * Getter del Cartel
     * 
     * @return Retorna el cartel.
     */
    public Cartel getCartel() {
        return cartel;
    }

    /**
     * Setter del cartel
     * 
     * @param cartel Assigna un cartel.
     */
    public void setCartel(Cartel cartel) {
        this.cartel = cartel;
    }

    /**
     * Getter de la lista de Carteles
     * 
     * @return Retorna la lista de Carteles.
     */
    public ArrayList<Cartel> getListCartel() {
        return listCartel;
    }

    /**
     * Setter de la lista de Carteles
     * 
     * @param listCartel Assigna la lista de Carteles.
     */
    public void setListCartel(ArrayList<Cartel> listCartel) {
        this.listCartel = listCartel;
    }

    /**
     * Getter de la lista de Tipos de plantilla
     * 
     * @return Retorna la lista de Tipos de plantilla.
     */
    public ArrayList<String> getListTipo() {
        return listTipo;
    }

    /**
     * Setter de la lista de Tipos de plantilla
     * 
     * @param listTipo Assigna la lista de Tipos de plantilla.
     */
    public void setListTipo(ArrayList<String> listTipo) {
        this.listTipo = listTipo;
    }

    /**
     * Getter de la lista de Tamaños de plantilla
     * 
     * @return Retorna la lista de Tamaños de plantilla.
     */
    public ArrayList<String> getListTamano() {
        return listTamano;
    }

    /**
     * Setter de la lista de Tamaños de plantilla
     * 
     * @param listTamano Assigna la lista de Tamaños de plantilla.
     */
    public void setListTamano(ArrayList<String> listTamano) {
        this.listTamano = listTamano;
    }

    /**
     * Getter del Trabajo
     * 
     * @return Retorna el trabajo.
     */ 
    public Trabajo getTrabajo() {
        return trabajo;
    }

    /**
     * Setter del Trabajo
     * 
     * @param trabajo Assigna el Trabajo.
     */
    public void setTrabajo(Trabajo trabajo) {
        this.trabajo = trabajo;
    }

    /**
     * Getter de la lista de trabajos
     * 
     * @return Retorna la lista de Trabajos.
     */
    public ArrayList<Trabajo> getListTrabajo() {
        return listTrabajo;
    }

    /**
     * Setter de la lista de trabajos
     * 
     * @param listTrabajo Assigna la lista de trabajos.
     */
    public void setListTrabajo(ArrayList<Trabajo> listTrabajo) {
        this.listTrabajo = listTrabajo;
    }

    /**
     * Getter del parametro option
     * 
     * @return Retorna la opcion elegida.
     */
    public String getOption() {
        return option;
    }

    /**
     * Setter del parametro option
     * 
     * @param option Assigna la opcion elegida.
     */
    public void setOption(String option) {
        this.option = option;
    }

    /**
     * Getter de mensaje
     * 
     * @return Retorna el mensaje.
     */
    public String getMsg() {
        return msg;
    }

    /**
     * Setter de mensaje
     * 
     * @param msg Assigna el mensaje.
     */
    public void setMsg(String msg) {
        this.msg = msg;
    }

    //-------------------sesions-----------------------
    
    /**
     * Añade los trabajos en la sesion
     * 
     */
    private void addTrabajoSession() {
        Map session = ActionContext.getContext().getSession();
        session.put("Trabajo", trabajo);
    }

        /**
     * Carga los trabajos a la sesion
     * 
     */
    private void loadTrabajoSession() {
        Map session = ActionContext.getContext().getSession();
        trabajo = (Trabajo) session.get("Trabajo");
    }

        /**
     * Elimina los trabajos de la sesion
     * 
     */
    private void removeTrabajoSession() {
        Map session = ActionContext.getContext().getSession();
        session.remove("Trabajo");
    }

        /**
     * Carga los carteles en la sesion
     * 
     */
    private void loadCartelSession() {
        Map session = ActionContext.getContext().getSession();
        listCartel = new ArrayList<Cartel>((ArrayList<Cartel>) session.get("Carteles"));
    }

        /**
     * Añade los carteles en la sesion
     * 
     */
    private void addCartelSession() {
        Map session = ActionContext.getContext().getSession();
        session.put("Carteles", listCartel);
    }

        /**
     * Elimina los carteles de la sesion
     * 
     */
    private void removeCartelSession() {
        Map session = ActionContext.getContext().getSession();
        listCartel = new ArrayList<Cartel>();
        session.put("Carteles", listCartel);
    }

    /**
     * Selecciona el articulo del trabajo.
     * 
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectArticuloTrabajo() {
        this.listTypes();
        this.loadCartelSession();


        int n = 0;

        ArticuloDAO artDAO = new ArticuloDAO();
        Articulo arti = new Articulo();

        if (!art.getCodInterno().equals("")) {
            n = n + 1;
            arti = artDAO.selectArticuloSms(art.getCodInterno());

            if (arti != null) {
                cartel.setArticulo(arti);
                msg = "El Articulo ha sido Seleccionado Correctamente.";
                return "OK";
            } else {
                msg = "El Sms del Articulo No existe.";
            }
        } else {
            msg = "Campo Sms esta vacio.";
        }

        if (!art.getEan().equals("")) {
            n = n + 1;
            arti = artDAO.selectArticulo(art);

            if (arti != null) {
                cartel.setArticulo(arti);
                msg = "El Articulo ha sido Seleccionado Correctamente.";
                return "OK";
            } else {
                msg = "El Ean del Articulo No existe.";
            }
        } else {
            msg = "Campo Ean esta vacio.";
        }

        if (n == 0) {
            msg = "Debe Introducir almenos un Codigo.";
            message = new MessageUtil(msg);
            addActionError(msg);
            return "KO";
        } else {
            msg = "El/Los Codigos Introducidos no Existen.";
            message = new MessageUtil(msg);
            addActionError(msg);
            return "KO";
        }

    }

    /**
     * Elimina el cartel seleccionado
     * 
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String deleteCartelTrabajo() {
        this.listTypes();
        this.loadCartelSession();

        listCartel = deleteEquals(selected);

        addCartelSession();
        return "OK";
    }

        /**
     * Funcion que elimina los carteles iguales
     * 
     * @return Retorna la lista de carteles sin repeticion
     */
    private ArrayList<Cartel> deleteEquals(String str) {
        StringTokenizer st = new StringTokenizer(str, ",");

        while (st.hasMoreElements()) {
            String element = ((String) st.nextElement());

            StringTokenizer st2 = new StringTokenizer(element);

            Integer id = Integer.parseInt((String) st2.nextElement());
            Cartel car = carDAO.selectCartel(id);

            if (listCartel.contains(car)) {
                listCartel.remove(car);
            }
        }

        return listCartel;
    }

    /**
     * Inserta el cartel en el trabajo
     * 
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String insertCartelTrabajo() {
        String result = null;

        this.listTypes();
        if (this.validateCartel()) {
            Cartel car = null;
            if ((car = carDAO.selectCartel(cartel)) == null) {
                Articulo art = null;
                ArticuloDAO artDAO = new ArticuloDAO();

                if ((art = artDAO.selectArticulo(cartel.getArticulo())) == null) {
                    artDAO.insertArticulo(cartel.getArticulo());
                } else {
                    msg = "El Articulo ya Existe. Tomado Valores de la BD";
                    cartel.setArticulo(art);
                }

                if (carDAO.insertCartel(cartel)) {

                    msg = "El Cartel ha sido Insertado Correctamente.";
                    result = "OK";
                } else {
                    msg = "Ha Ocurrido un Error al Insertar el cartel.";
                    message = new MessageUtil(msg);
                    addActionError(msg);
                    result = "KO";
                }
            } else {
                cartel = car;
                msg = "El Cartel ya Existe";
                result = "OK";
            }

        } else {
            msg = "Ha Ocurrido un Error al Validar el Formulario.";
            message = new MessageUtil(msg);
            addActionError(msg);
            result = "KO";
        }

        this.loadCartelSession();

        if (result.equals("OK")) {
            listCartel.add(cartel);
            this.addCartelSession();
            cartel = new Cartel();
            return "OK";
        }

        return "KO";
    }

    /**
     * Funcion que crea el trabajo
     * 
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String createTrabajo() {

        try {
            this.listTypes();
            this.loadTrabajoSession();
            this.loadCartelSession();


            if (listCartel.size() > 0) {

                //Ordenamos la lista de carteles por PLANTILLA
                Collections.sort(listCartel, Cartel.COMPARADOR_POR_PLANTILLA);
                String plan = listCartel.get(0).getPlantilla().toStr();

                String descripcion = trabajo.getDescripcion();
                Integer prioridad = trabajo.getPrioridad();

                Date d = new Date();
                Trabajo tra = new Trabajo(descripcion + " " + plan, prioridad, d);

                Iterator<Cartel> it = listCartel.listIterator();

                //insertamos el Trabajo
                TrabajoDAO traDAO = new TrabajoDAO();

                while (it.hasNext()) {

                    Cartel car = it.next();

                    if (car.getPlantilla().toStr().equals(plan)) {
                        tra.addCarteles(car);
                    } else {
                        if (traDAO.insertTrabajo(tra)) {
                            plan = car.getPlantilla().toStr();
                            tra = new Trabajo(descripcion + " " + plan, prioridad, d);
                            tra.addCarteles(car);
                        } else {
                            msg = "Ha Ocurrido un Error al Insertar el Trabajo.";
                            message = new MessageUtil(msg);
                            addActionError(msg);

                            return "KO";
                        }
                    }
                }
                //Añadimos el ultimo trabajo
                if (traDAO.insertTrabajo(tra)) {
                    msg = "Trabajos creados Correctamente.";
                    message = new MessageUtil(msg);
                    this.removeCartelSession();
                    this.removeTrabajoSession();
                    cartel = new Cartel();
                    trabajo = new Trabajo();
                    return "OK";
                } else {
                    msg = "Ha Ocurrido un Error al Insertar el Trabajo.";
                    message = new MessageUtil(msg);
                    addActionError(msg);
                    return "KO";
                }
            } else {
                msg = "La lista de carteles esta vacia.";
                message = new MessageUtil(msg);
                addActionError(msg);
                return "KO";
            }
        } catch (Exception e) {
            msg = "Ha Ocurrido un Error al Crear el Trabajo.";
            message = new MessageUtil(msg);
            addActionError(msg);
            return "KO";
        }
    }

    /**
     * Selecciona el trabajo
     * 
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectTrabajo() {

        if (this.validateSelect()) {

            Trabajo tra = trabajoDAO.selectTrabajo(trabajo.getIdTrabajo());

            if (tra != null) {
                this.trabajo = tra;
                msg = "La Trabajo se ha Seleccionado Correctamente.";
                return "OK";
            } else {
                this.trabajo = new Trabajo();
                msg = "Ha Ocurrido un Error al Consultar la Trabajo.";
                addActionError(msg);
                msg = "La Trabajo no Existe.";
                addFieldError("", msg);
                message = new MessageUtil(msg);
                return "KO";
            }
        } else {
            this.trabajo = new Trabajo();
            msg = "Ha Ocurrido un Error al Validar el Formulario.";
            message = new MessageUtil(msg);
            addActionError(msg);
            return "KO";
        }

    }

    /**
     * Selecciona todos los trbajos
     * 
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectAllTrabajo() {
        listTrabajo = trabajoDAO.selectAll();
        if (listTrabajo.size() > 0) {
            msg = "Las Trabajos se ha Seleccionado Correctamente.";
        } else {
            msg = "La base de datos de Trabajos esta Vacia.";
            message = new MessageUtil(msg);
            return "KO";
        }
        return "OK";
    }

    /**
     * Selecciona los trbajos con Ejemplo
     * 
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectByExampleTrabajo() {
        if (this.validateExample()) {
            listTrabajo = trabajoDAO.selectByExample(trabajo);

            if (listTrabajo != null) {
                msg = "La Trabajo se ha Seleccionado Correctamente.";
                return "OK";
            } else {
                msg = "No Existe una Trabajo con esas Caracteristicas.";
                message = new MessageUtil(msg);
                return "KO";
            }
        } else {
            msg = "Todos los Campos estan Vacios.";
            return this.selectAllTrabajo();
        }
    }

    /**
     * Elimina el trabajo
     * 
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String deleteTrabajo() {
        msg = "Desea Eliminar el Trabajo??";
        if (message.showConfirmationMessage(msg) != -1) {

            if (trabajoDAO.deleteTrabajo(trabajo)) {
                msg = "La Trabajo ha sido Eliminado Correctamente.";
                message = new MessageUtil(msg);
                trabajo = new Trabajo();
                return "OK";
            } else {
                msg = "Ha Ocurrido un Error al Eliminar la Trabajo.";
                message = new MessageUtil(msg);
                return "KO";
            }
        }
        msg = "Processo Abortado.";
        return "KO";
    }

    /**
     * Modifica el Trabajo
     * 
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String updateTrabajo() {
        msg = "Desea Modificar el Trabajo??";
        if (message.showConfirmationMessage(msg) != -1) {
            if (trabajoDAO.updateTrabajo(trabajo)) {
                msg = "El Trabajo ha sido Modificado Correctamente.";
                message = new MessageUtil(msg);
                return "OK";
            } else {
                msg = "Ha Ocurrido un Error al Modificado la Trabajo.";
                message = new MessageUtil(msg);
                return "KO";
            }
        }
        msg = "Processo Abortado.";
        return "KO";
    }

    /**
     * Imprime un trabajo (formato texto)
     * 
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printTrabajoList() {
        if (trabajo != null) {
            ObjectPdf objPdf = new ObjectPdf(this.trabajo);

            return "OK";
        } else {

            message = new MessageUtil("No hay datos para imprimir");
            return "KO";
        }
    }

    /**
     * Imprime la lista de trabajos
     * 
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printTrabajos() {
        if (this.selectByExampleTrabajo().equals("OK")) {

            if (listTrabajo != null) {
                ObjectPdf objPdf = new ObjectPdf(this.listTrabajo);

                return "OK";
            } else {

                message = new MessageUtil("No hay datos para imprimir");
                return "KO";
            }
        } else {
            message = new MessageUtil("No hay datos para imprimir");
            return "KO";
        }
    }

    /**
     * Imprime la lista de trabajos
     * 
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printAllTrabajos() {
        if (this.selectAllTrabajo().equals("OK")) {
            if (listTrabajo != null) {
                ObjectPdf objPdf = new ObjectPdf(this.listTrabajo);

                return "OK";
            } else {

                message = new MessageUtil("No hay datos para imprimir");
                return "KO";
            }
        } else {
            message = new MessageUtil("No hay datos para imprimir");
            return "KO";
        }
    }
    //-------------------------------------------------------------
    //------------------------VALIDATES----------------------------
    //-------------------------------------------------------------

        /**
     * Funcion Auxiliar para la validacion carteles
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateCartel() {
        boolean bool = true;

        if (this.trabajo.getDescripcion().equals("")) {
            addFieldError("", "El campo Descripcion del Trbajo es Obligatorio");
            bool = false;
        } else {
            if (trabajoDAO.selectTrabajo(trabajo) != null) {
                msg = "La descripcion del Trabajo Ya Existe.";
                message = new MessageUtil(msg);
                addActionError(msg);
                bool = false;
            } else {
                this.addTrabajoSession();
            }
        }

        if (validateArticulo() == false) {
            bool = false;
        }

        if (validatePlantilla() == false) {
            bool = false;
        }

        return bool;
    }

        /**
     * Funcion Auxiliar para la validacion de articulos
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateArticulo() {
        boolean bool = true;

        if (this.cartel.getArticulo().getDescripcion1().equals("")) {
            addFieldError("", "El campo Descripcion 1 es Obligatorio");
            bool = false;
        }

        if (this.cartel.getArticulo().getDescripcion2().equals("")) {
            addFieldError("", "El campo Descripcion 2 es Obligatorio");
            bool = false;
        }

        if (this.cartel.getArticulo().getDescripcion3().equals("")) {
            this.cartel.getArticulo().setDescripcion3(null);
        }

        if (this.cartel.getArticulo().getPrecio() == null) {
            addFieldError("", "El campo Precio es Obligatorio");
            bool = false;
        }

        if (this.cartel.getArticulo().getEan().equals("")) {
            addFieldError("", "El campo Ean es Obligatorio");
            bool = false;
        }

        if (this.cartel.getArticulo().getCodInterno().equals("")) {
            this.cartel.getArticulo().setCodInterno(null);
        }

        return bool;
    }

        /**
     * Funcion Auxiliar para la validacion de Plantillas
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validatePlantilla() {
        PlantillaDAO planDAO = new PlantillaDAO();

        Plantilla plan = planDAO.selectPlantilla(cartel.getPlantilla());
        if (plan == null) {
            addFieldError("", "La Plantilla No Existe.");
            this.cartel.getPlantilla().reset();
            return false;
        } else {
            cartel.setPlantilla(plan);
            return true;
        }
    }

        /**
     * Funcion Auxiliar para la validacion del ID trabajo
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateSelect() {

        if (this.trabajo.getIdTrabajo() == null) {
            addFieldError("", "Introduzca el ID");
            return false;
        }

        return true;
    }

        /**
     * Funcion Auxiliar para la validacion de trabajos elegidos por Ejemplo
     *
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateExample() {

        int n = 0;

        this.trabajo.setDescripcion(null);

        if (this.trabajo.getIdTrabajo() == null) {
            n = n + 1;
        }

        if (this.trabajo.getPrioridad() == null) {
            n = n + 1;
        }

        if (this.trabajo.getFechaCreacion() == null) {
            n = n + 1;
        }

        if (n != 0) {
            return true;
        }

        return false;
    }

        /**
     * Funcion Auxiliar para buscar las plantillas en la DB.
     *
     */
    private void listTypes() {
        PlantillaDAO planDAO = new PlantillaDAO();
        ArrayList<Plantilla> listPlantilla = planDAO.selectAll();

        Iterator it = listPlantilla.listIterator();

        while (it.hasNext()) {
            Plantilla plan = (Plantilla) it.next();

            if (!this.listTamano.contains(plan.getTamano())) {
                listTamano.add(plan.getTamano());
            }

            if (!this.listTipo.contains(plan.getTipo())) {
                listTipo.add(plan.getTipo());
            }
        }
    }

    //-------------------------------------------------------------
    //-----------------------ESTADO TRABAJOS-----------------------
    //-------------------------------------------------------------
    /**
     * Abre la pantalla Trabajos impresos
     * 
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printTrabajoImpreso() {
        this.getCurrentUser();
        this.selectTrabajosByState("I");

        return this.printTrabajo();
    }

    /**
     * Abre la pantalla Trabajos no Impresos
     * 
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printTrabajoNoImpreso() {
        this.getCurrentUser();
        this.selectTrabajosByState(null);
        return this.printTrabajo();
    }

    /**
     * Abre la pantalla de Trabajos Rechazados
     * 
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printTrabajoRechazado() {
        this.getCurrentUser();
        this.selectTrabajosByState("R");
        return this.printTrabajo();
    }

    /**
     * Imprime un trabajo (Formato cartel)
     * 
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printTrabajo() {
        Trabajo tra = trabajoDAO.selectTrabajo(trabajo.getIdTrabajo());

        if (tra != null) {
            //Si el trabajo existe mriamos que este en la lista adecuada
            if (listTrabajo.contains(tra)) {

                CartelPdf p = new CartelPdf(tra);

                this.getCurrentUser();
                Date d = new Date();
                Estadotrabajo est = new Estadotrabajo(tra, current, "I", d);
                EstadotrabajoDAO estDAO = new EstadotrabajoDAO();

                if (estDAO.selectEstadotrabajo(est) != null) {
                    //si Existe lo modificamos a impreso
                    if (estDAO.updateEstadotrabajo(est)) {
                        msg = "El Trabajo ha sido modificado Correctamente.";
                        return "OK";
                    } else {
                        addActionError("Ha Ocurrido un Error al Modificar el Trabajo");
                        msg = "No se ha podido modificar el Estado del Trabajo.";
                        message = new MessageUtil(msg);
                        addFieldError("", msg);
                        return "KO";
                    }
                } else {
                    //sino existe lo añadimos
                    if (estDAO.insertEstadotrabajo(est)) {
                        msg = "El Trabajo ha sido Insertado Correctamente.";
                        return "OK";
                    } else {
                        addActionError("Ha Ocurrido un Error al Insertar el Trabajo");
                        msg = "No se ha podido modificar el Estado del Trabajo.";
                        message = new MessageUtil(msg);
                        addFieldError("", msg);
                        return "KO";
                    }
                }

            } else {
                addActionError("Ha Ocurrido un Error al Comprobar el Trabajo");
                msg = "El Trabajo no Esta en la Lista de Disponibles.";
                message = new MessageUtil(msg);
                addFieldError("", msg);
                return "KO";
            }
        } else {
            addActionError("Ha Ocurrido un Error al Seleccionar el Trabajo");
            msg = "El Trabajo no Existe.";
            message = new MessageUtil(msg);
            addFieldError("", msg);
            return "KO";
        }
    }

    /**
     * Rechaza el trabajo
     * 
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String refuseTrabajo() {
        Trabajo tra = trabajoDAO.selectTrabajo(trabajo.getIdTrabajo());
        if (tra != null) {
            this.getCurrentUser();
            Date d = new Date();
            Estadotrabajo est = new Estadotrabajo(tra, current, "R", d);
            EstadotrabajoDAO estDAO = new EstadotrabajoDAO();

            if (estDAO.insertEstadotrabajo(est)) {
                msg = "El Trabajo ha sido Rechazado Correctamente.";
                return "OK";
            } else {
                addActionError("Ha Ocurrido un Error al Rechazar el Trabajo");
                msg = "El Trabajo no Esta en la Lista de Disponibles.";
                message = new MessageUtil(msg);
                addFieldError("", msg);
                return "KO";
            }
        } else {
            addActionError("Ha Ocurrido un Error al Seleccionar el Trabajo");
            msg = "El Trabajo no Existe.";
            message = new MessageUtil(msg);
            addFieldError("", msg);
            return "KO";
        }
    }

    /**
     * Selecciona trabajos por estado
     * 
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    private void selectTrabajosByState(String estado) {
        this.getCurrentUser();

        EstadotrabajoDAO estDAO = new EstadotrabajoDAO();

        if (estado != null) {
            ArrayList<Estadotrabajo> listEt = estDAO.selectEstadotrabajo(current, estado);

            Iterator<Estadotrabajo> it = listEt.listIterator();

            while (it.hasNext()) {
                Trabajo tra = it.next().getTrabajo();
                this.listTrabajo.add(tra);
            }
        } else {
            //seleccionamos todos los trabajos
            this.listTrabajo = trabajoDAO.selectAll();

            //seleccionamos los trabajos del usuario (rechazados y impresos)
            ArrayList<Estadotrabajo> listEt = estDAO.selectEstadotrabajo(current);
            Iterator<Estadotrabajo> it = listEt.listIterator();

            while (it.hasNext()) {
                Trabajo tra = it.next().getTrabajo();

                listTrabajo.remove(tra);
            }
        }
    }

    /**
     * Assigna el usuario logueado
     * 
     */
    private void getCurrentUser() {
        Map session = ActionContext.getContext().getSession();

        LoginDAO logDAO = new LoginDAO();
        Login log = logDAO.selectLogin((String) session.get("usuario"));

        UsuarioDAO userDAO = new UsuarioDAO();
        current = userDAO.selectUsuario(log);
    }
    //-------------------------------------------------------------
    //--------------------------PANTALLAS--------------------------
    //-------------------------------------------------------------

    /**
     * Abre la pantalla Insertar Trabajo
     * 
     * @return Retorna "PANTALLA" dependiendo del Exito del Procedimiento.
     */
    public String screenInsertTrabajo() {
        this.listTypes();
        Map session = ActionContext.getContext().getSession();
        session.put("Carteles", listCartel);

        return "PANTALLA";
    }

    /**
     * Abre la pantalla Seleccionar Trabajo
     * 
     * @return Retorna "PANTALLA" dependiendo del Exito del Procedimiento.
     */
    public String screenSelectTrabajo() {

        if (trabajo != null) {
            trabajo.reset();
        }
        return "PANTALLA";
    }

    /**
     * Abre la pantalla Modificar Usuario
     * 
     * @return Retorna "PANTALLA" dependiendo del Exito del Procedimiento.
     */
    public String screenUpdateTrabajo() {
        return "PANTALLA";
    }

    /**
     * Abre la pantalla Selecionar trabajo con filtro
     * 
     * @return Retorna "PANTALLA" dependiendo del Exito del Procedimiento.
     */
    public String screenSelectByExampleTrabajo() {
        trabajo = new Trabajo();
        return "PANTALLA";
    }

    /**
     * Abre la pantalla trabajos Impresos
     * 
     * @return Retorna "PANTALLA" dependiendo del Exito del Procedimiento.
     */
    public String screenSelectTrabajosImpresos() {
        this.getCurrentUser();
        this.selectTrabajosByState("I");
        return "PANTALLA";

    }

    /**
     * Abre la pantalla trabajos No impresos
     * 
     * @return Retorna "PANTALLA" dependiendo del Exito del Procedimiento.
     */
    public String screenSelectTrabajosNoImpresos() {
        this.getCurrentUser();
        this.selectTrabajosByState(null);
        return "PANTALLA";
    }

    /**
     * Abre la pantalla de trabajos Rechazados
     * 
     * @return Retorna "PANTALLA" dependiendo del Exito del Procedimiento.
     */
    public String screenSelectTrabajosRechazados() {
        this.getCurrentUser();
        this.selectTrabajosByState("R");
        return "PANTALLA";
    }
}
