/**
 * Usuario Controller.
 *
 * Responde a eventos referente a la CLASE USUARIO ,invoca peticiones al
 * 'modelo' cuando se hace alguna solicitud sobre la información (por ejemplo,
 * editar un documento o un registro en una base de datos).
 *
 * También puede enviar comandos a su 'vista' asociada si se solicita un cambio
 * en la forma en que se presenta de 'modelo' (por ejemplo, desplazamiento o
 * scroll por un documento o por los diferentes registros de una base de datos),
 * por tanto se podría decir que el 'controlador' hace de intermediario entre la
 * 'vista' y el 'modelo'.
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws Throws in DAO
 */
package controller;

import com.opensymphony.xwork2.ActionSupport;
import com.opensymphony.xwork2.ModelDriven;
import java.util.ArrayList;
import java.util.Date;
import model.dao.DepartamentoDAO;
import model.dao.DocumentoDAO;
import model.dao.IncidenciaDAO;
import model.dao.LoginDAO;
import model.dao.PersonaDAO;
import model.dao.UsuarioDAO;
import model.entities.Departamento;
import model.entities.Documento;
import model.entities.Incidencia;
import model.entities.Login;
import model.entities.Persona;
import model.entities.Usuario;
import model.pdf.ObjectPdf;
import model.util.MessageUtil;
import org.apache.commons.lang3.RandomStringUtils;

/**
 *
 * @author Portatil
 */
public class UsuarioController extends ActionSupport implements ModelDriven<Usuario> {

    private Usuario usuario = new Usuario();
    private UsuarioDAO usuarioDAO = new UsuarioDAO();
    private ArrayList<Usuario> listUsuario = new ArrayList<Usuario>();
    private Login login = new Login();
    private LoginDAO loginDAO = new LoginDAO();
    private String option = null;
    private MessageUtil message = new MessageUtil();
    private String msg = null;

    /**
     * Implementacion del ModelDriven
     *
     * Permite enlazar tags de Struts 2 con el objeto del Action
     *
     * @return Retorna el Objeto del ModelDriven
     */
    @Override
    public Usuario getModel() {
        return usuario;
    }

    /**
     * Constructor Vacio.
     */
    public UsuarioController() {
    }

    /**
     * Getter del usuario
     *
     * @return Retorna el usuario.
     */
    public Usuario getUsuario() {
        return usuario;
    }

    /**
     * Setter del usuario
     *
     * @param usuario Assigna el usuario.
     */
    public void setUsuario(Usuario usuario) {
        this.usuario = usuario;
    }

    /**
     * Getter de la lista de Usuarios
     *
     * @return Retorna la lista de Usuarios.
     */
    public ArrayList<Usuario> getListUsuario() {
        return listUsuario;
    }

    /**
     * Setter de la lista de Usuarios
     *
     * @param listUsuario Assigna la lista de Usuarios.
     */
    public void setListUsuario(ArrayList<Usuario> listUsuario) {
        this.listUsuario = listUsuario;
    }

    /**
     * Getter de mensaje
     *
     * @return Retorna el mensaje.
     */
    public String getMsg() {
        return msg;
    }

    /**
     * Setter de mensaje
     *
     * @param msg Assigna el mensaje.
     */
    public void setMsg(String msg) {
        this.msg = msg;
    }

    /**
     * Getter de option
     *
     * @return Retorna la opcion elegida.
     */
    public String getOption() {
        return option;
    }

    /**
     * Setter de option
     *
     * @param option Assigna la opcion eleginda.
     */
    public void setOption(String option) {
        this.option = option;
    }

    /**
     * Inserta el usuario
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String insertUsuario() {
        if (this.validateInsert()) {
            if (this.validateDocumento()) {
                if (this.validateDepartamento()) {
                    if (this.validateLogin()) {
                        //completamos el Usuario

                        usuario.setLogin(login);
                        usuario.setEstadoEnSistema("A");


                        if (usuarioDAO.insertUsuario(usuario)) {
                            msg = "El Usuario ha sido Insertado Correctamente.";

                            usuario.reset();
                            return "OK";
                        } else {
                            msg = "Ha Ocurrido un Error al Insertar el Usuario.";
                            message = new MessageUtil(msg);
                            return "KO";
                        }
                    } else {
                        msg = "Ha Ocurrido un Error al Validar el Usuario.";
                        message = new MessageUtil(msg);
                        addActionError(msg);
                        return "KO";
                    }
                } else {
                    msg = "Ha Ocurrido un Error al Validar el Departamento.";
                    message = new MessageUtil(msg);
                    addActionError(msg);
                    return "KO";
                }
            } else {
                msg = "Ha Ocurrido un Error al Validar el Documento.";
                message = new MessageUtil(msg);
                addActionError(msg);
                return "KO";
            }
        } else {
            msg = "Ha Ocurrido un Error al Validar el Formulario.";
            message = new MessageUtil(msg);
            addActionError(msg);
            return "KO";
        }
    }

    /**
     * Registra el usuario
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String registerUsuario() {
        return this.insertUsuario();
    }

    /**
     * Selecciona el Usuario
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectUsuario() {
        if (this.validateSelect()) {

            if (option.equals("Buscar por ID")) {
                usuario = usuarioDAO.selectUsuario(usuario.getIdUsuario());
                if (usuario != null) {
                    listUsuario.add(usuario);
                }

            } else if (option.equals("Buscar por Usuario")) {


                if (login != null) {
                    usuario = usuarioDAO.selectUsuario(login);

                    if (usuario != null) {

                        listUsuario.add(usuario);
                    }
                }
            } else {
                listUsuario = usuarioDAO.selectByExample(usuario);
            }

            if (listUsuario.size() > 0) {
                usuario = listUsuario.get(0);
                msg = "El Usuario se ha Seleccionado Correctamente.";

                return "OK";
            } else {
                this.usuario = null;
                msg = "Ha Ocurrido un Error al Consultar el Usuario.";
                addActionError(msg);
                msg = "El Usuario no Existe.";
                addFieldError("", msg);
                message = new MessageUtil(msg);
                return "KO";
            }
        } else {
            msg = "Ha Ocurrido un Error al Validar el Formulario.";
            message = new MessageUtil(msg);
            addActionError(msg);
            return "KO";
        }
    }

    /**
     * Selecciona todos los Usuarios
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectAllUsuario() {
        listUsuario = usuarioDAO.selectAll();
        if (listUsuario.size() > 0) {
            msg = "Los Usuarios se ha Seleccionado Correctamente.";
        } else {
            msg = "Ha Ocurrido un Error al Seleccionar los Usuarios.";
            message = new MessageUtil(msg);
            return "KO";
        }
        return "OK";
    }

    /**
     * Selecciona Usuarios segun ejemplo
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectByExampleUsuario() {
        if (this.validateExample()) {
            listUsuario = usuarioDAO.selectByExample(usuario);

            if (listUsuario.size() > 0) {
                msg = "El Usuario se ha Seleccionado Correctamente.";

                return "OK";
            } else {
                msg = "No Existen Usuarios con esas Caracteristicas.";
                message = new MessageUtil(msg);
                return "KO";
            }
        } else {
            msg = "Todos los Campos estan Vacios.";

            return this.selectAllUsuario();
        }
    }

    /**
     * Elimina el Usuario
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String deleteUsuario() {
        msg = "Desea Eliminar el Usuario??";
        if (message.showConfirmationMessage(msg) != -1) {

            if (usuarioDAO.deleteUsuario(usuario)) {

                msg = "El Usuario ha sido Eliminado Correctamente.";
                message = new MessageUtil(msg);
                usuario.reset();
                return "OK";
            } else {
                msg = "Ha Ocurrido un Error al Eliminar el Usuario.";
                message = new MessageUtil(msg);
                return "KO";
            }
        }
        msg = "Processo Abortado.";
        return "KO";
    }

    /**
     * Modifica el Usuario
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String updateUsuario() {
        msg = "Desea Modificar el Usuario??";
        if (message.showConfirmationMessage(msg) != -1) {

            DepartamentoDAO depDAO = new DepartamentoDAO();

            Departamento dep = depDAO.selectDepartamentoCod(usuario.getDepartamento().getCodigoDepartamento());

            usuario.setDepartamento(dep);

            if (usuarioDAO.updateUsuario(usuario)) {
                msg = "El Usuario ha sido Modificado Correctamente.";
                message = new MessageUtil(msg);
                return "OK";
            } else {
                msg = "Ha Ocurrido un Error al Modificado el Usuario.";
                message = new MessageUtil(msg);
                return "KO";
            }
        }
        msg = "Processo Abortado.";
        return "KO";
    }

    /**
     * Bloquea el Usuario
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String lockUsuario() {
        msg = "Desea Bloquear el Usuario??";
        if (message.showConfirmationMessage(msg) != -1) {

            usuario.setEstadoEnSistema("B");

            if (usuarioDAO.updateUsuario(usuario)) {
                msg = "El Usuario ha sido Modificado Correctamente.";
                message = new MessageUtil(msg);
                return "OK";
            } else {
                msg = "Ha Ocurrido un Error al Modificado el Usuario.";
                message = new MessageUtil(msg);
                return "KO";
            }
        }
        msg = "Processo Abortado.";
        return "KO";
    }

    /**
     * Desbloquea el Usuario
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String unlockUsuario() {
        msg = "Desea desbloquear el Usuario??";
        if (message.showConfirmationMessage(msg) != -1) {

            usuario.setEstadoEnSistema("A");

            if (usuarioDAO.updateUsuario(usuario)) {
                msg = "El Usuario ha sido Modificado Correctamente.";
                message = new MessageUtil(msg);
                return "OK";
            } else {
                msg = "Ha Ocurrido un Error al Modificado el Usuario.";
                message = new MessageUtil(msg);
                return "KO";
            }
        } else {
            msg = "Processo Abortado.";
            return "KO";
        }
    }

    /**
     * Modifica el login de forma Automatica
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String updateAutoLogin() {
        msg = "Desea Modificar el Password??";
        if (message.showConfirmationMessage(msg) != -1) {
            String pass = RandomStringUtils.randomAlphanumeric(8);

            login = loginDAO.selectLogin(usuario.getLogin().getUsuario());

            Date d = new Date();
            login.setDataPassWord(d);
            login.setPassword(pass);

            if (loginDAO.updateLogin(login)) {

                message = new MessageUtil("El Nuevo Password es: " + pass);

                //generamos incidencia para que el usuario pueda consultarla

                Incidencia inci = new Incidencia();
                inci.setOrigen("A");
                inci.setDestino(Integer.toString(usuario.getIdUsuario()));
                inci.setAsunto("Password");
                inci.setIncidencia("Password Perdido.");
                inci.setEstado("C");
                inci.setDataApertura(null);
                inci.setRespuesta("Password Generado correctamente.");
                inci.setDataCierre(d);

                IncidenciaDAO inciDAO = new IncidenciaDAO();

                if (inciDAO.insertIncidencia(inci)) {
                    msg = "Password Generado Correctamente.";
                } else {
                    msg = "No se ha podido Generear la Incidencia.";
                }
                return "OK";
            } else {
                msg = "No se ha podido Generar el Password.";
                message = new MessageUtil(msg);
                return "KO";
            }
        } else {
            msg = "Processo Abortado.";
            return "KO";
        }
    }

    /**
     * Imprime un Usuario
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printUsuario() {
        if (usuario != null) {
            ObjectPdf objPdf = new ObjectPdf(this.usuario);

            return "OK";
        } else {

            message = new MessageUtil("No hay datos para imprimir");
            return "KO";
        }
    }

    /**
     * Imprime la lista de Usuarios
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printUsuarios() {
        if (this.selectByExampleUsuario().equals("OK")) {
            if (listUsuario != null) {
                ObjectPdf objPdf = new ObjectPdf(this.listUsuario);

                return "OK";
            } else {

                message = new MessageUtil("No hay datos para imprimir");
                return "KO";
            }
        } else {
            message = new MessageUtil("No hay datos para imprimir");
            return "KO";
        }
    }

    /**
     * Imprime la lista de Usuarios
     *
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String printAllUsuarios() {
        if (this.selectAllUsuario().equals("OK")) {
            if (listUsuario != null) {
                ObjectPdf objPdf = new ObjectPdf(this.listUsuario);

                return "OK";
            } else {

                message = new MessageUtil("No hay datos para imprimir");
                return "KO";
            }
        } else {
            message = new MessageUtil("No hay datos para imprimir");
            return "KO";
        }
    }

    //-------------------------------------------------------------
    //------------------------VALIDATES----------------------------
    //-------------------------------------------------------------
    /**
     * Funcion Auxiliar para la validacion del login
     * 
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateTextfieldsLogin() {
        boolean bool = true;

        if (this.usuario.getLogin().getUsuario().equals("")) {
            addFieldError("", "Debe Introducir el Usuario.");
            bool = false;
        }

        if (this.usuario.getLogin().getPassword().equals("")) {
            addFieldError("", "Debe Introducir el Password.");
            bool = false;
        }

        return bool;
    }

    /**
     * Funcion Auxiliar para la validacion del estado de Usuario
     * 
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateStateUsuario() {
        if (usuario.getEstadoEnSistema().equals("A")) {
            return true;
        } else {
            addFieldError("", "El Usuario esta Bloqueado.");
            return false;
        }
    }

    /**
     * Funcion Auxiliar para la validacion del password
     * 
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validatePassword() {
        //primero validamos la contraseña correcta
        if (login.getPassword().equals(usuario.getLogin().getPassword())) {

            //validamos la caducidad de la contraseña
            final long MILLSECS_PER_DAY = 24 * 60 * 60 * 1000; //Milisegundos al día 

            Date now = new Date(); //Fecha de hoy 
            Date pass = login.getDataPassWord();

            long result = (now.getTime() - pass.getTime()) / MILLSECS_PER_DAY;

            if (result > 365) {
                addFieldError("", "El password esta Caducado.");
                return false;
            }
        } else {
            return false;
        }

        return true;
    }

    /**
     * Funcion Auxiliar para la validacion del insert
     * 
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateInsert() {
        boolean bool = true;

        if (this.usuario.getDepartamento().getCodigoDepartamento().equals("")) {
            addFieldError("", "El campo Departamento es Obligatorio");
            bool = false;
        }

        if (this.usuario.getPersona().getDocumento().getNumeroDocumento().equals("")) {
            addFieldError("", "El campo Documento es Obligatorio");
            bool = false;
        }

        if (this.usuario.getLogin().getUsuario().equals("")) {
            addFieldError("", "El campo Usuario es Obligatorio");
            bool = false;
        }

        if (this.usuario.getLogin().getPassword().equals("")) {
            addFieldError("", "El campo Password es Obligatorio");
            bool = false;
        }

        if (this.usuario.getEmailUsuario().equals("")) {
            addFieldError("", "El campo Email es Obligatorio");
            bool = false;
        }

        if (this.usuario.getTelefonoUsuario().equals("")) {
            addFieldError("", "El campo Telefono es Obligatorio");
            bool = false;
        }

        return bool;
    }

    /**
     * Funcion Auxiliar para la validacion del Documento
     * 
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateDocumento() {

        DocumentoDAO docDAO = new DocumentoDAO();
        Documento doc = docDAO.selectDocumento(usuario.getPersona().getDocumento());

        if (doc != null) {

            PersonaDAO perDAO = new PersonaDAO();
            Persona per = perDAO.selectPersona(doc);

            if (per != null) {
                usuario.setIdUsuario(per.getIdPersona());

                msg = "El Documento se ha Seleccionado Correctamente.";
                return true;
            } else {
                msg = "Ha ocurrido un Error al seleccionar la usuario.";
                addFieldError("", msg);
                return false;
            }
        } else {
            msg = "El Documento no Existe.";
            addFieldError("", msg);
            return false;
        }
    }

    /**
     * Funcion Auxiliar para la validacion del Departamento
     * 
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateDepartamento() {

        DepartamentoDAO depDAO = new DepartamentoDAO();
        Departamento dep = depDAO.selectDepartamentoCod(usuario.getDepartamento().getCodigoDepartamento());

        if (dep != null) {

            this.usuario.setDepartamento(dep);

            msg = "El Departamento se ha Seleccionado Correctamente.";
            return true;
        } else {
            msg = "El Departamento no Existe.";
            addFieldError("", msg);
            return false;
        }
    }

    /**
     * Funcion Auxiliar para la validacion del Login
     * 
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateLogin() {

        Login log = loginDAO.selectLogin(usuario.getLogin());

        if (log != null) {
            msg = "El Usuario ya Existe.";
            addFieldError("", msg);
            return false;
        } else {
            login = usuario.getLogin();
            Date d = new Date();
            login.setDataPassWord(d);
            msg = "El Login esta Libre.";
            return true;
        }
    }

    /**
     * Funcion Auxiliar para la validacion del Select
     * 
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateSelect() {
        boolean bool = true;

        usuario.onlyParameter(option);

        if (option.equals("Buscar por ID")) {
            if (this.usuario.getIdUsuario() == null) {
                addFieldError("", "Introduzca el ID.");
                bool = false;
            }
        }

        if (option.equals("Buscar por Usuario")) {
            if (this.usuario.getLogin().getUsuario().equals("")) {
                addFieldError("", "Introduzca el Usuario.");
                bool = false;
            } else {
                login = loginDAO.selectLogin(usuario.getLogin().getUsuario());

                if (login != null) {
                    usuario.setLogin(login);
                } else {
                    addFieldError("", "El Usuario no Existe.");
                    bool = false;
                }
            }

        }

        if (option.equals("Buscar por Telefono")) {
            if (this.usuario.getTelefonoUsuario().equals("")) {
                addFieldError("", "Introduzca el Telefono.");
                bool = false;
            }
        }

        if (option.equals("Buscar por Email")) {
            if (this.usuario.getEmailUsuario().equals("")) {
                addFieldError("", "Introduzca el Email.");
                bool = false;
            }
        }

        return bool;
    }

    /**
     * Funcion Auxiliar para la validacion de Seleccion de articulos por ejemplo
     * 
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateExample() {
        boolean bool = true;
        int n = 0;

        if (this.usuario.getTipoUsuario().equals("")) {
            n = n + 1;
            usuario.setTipoUsuario(null);
        }

        if (this.usuario.getEstadoEnSistema().equals("")) {
            n = n + 1;
            usuario.setEstadoEnSistema(null);
        }

        if (this.usuario.getDepartamento().getCodigoDepartamento().equals("")) {
            n = n + 1;
            usuario.setDepartamento(null);
        }

        if (n == 3) {
            bool = false;
        }
        return bool;
    }

    //-------------------------------------------------------------
    //--------------------------PANTALLAS--------------------------
    //-------------------------------------------------------------
    /**
     * Abre pantalla Seleccionar Usuario
     *
     * @return Retorna "PANTALLA" dependiendo del Exito del Procedimiento.
     */
    public String screenSelectUsuario() {
        usuario.reset();
        return "PANTALLA";
    }

    /**
     * Abre la pantalla Modificar Usuario
     *
     * @return Retorna "PANTALLA" dependiendo del Exito del Procedimiento.
     */
    public String screenUpdateUsuario() {
        return "PANTALLA";
    }

    /**
     * Abre la pantalla Selecionar usuario con filtro
     *
     * @return Retorna "PANTALLA" dependiendo del Exito del Procedimiento.
     */
    public String screenSelectByExampleUsuario() {
        usuario.reset();
        return "PANTALLA";
    }
}
