/**
  * Login Controller.
  * 
  * Responde a eventos referente a la CLASE LOGIN ,invoca peticiones al 'modelo' procedentes UNICAMENTE de la OPERACION LOGIN.
  * 
  * A TITULO DE EJEMPLO, SE CONTROLA UNA UNICA ACCION, y se VALIDA esta por Separado.
  * 
  * @author JuanRamon Segura Ponce
  *
  * @version v.1.0
  * @throws Throws in DAO
  */

package controller.login;

import com.opensymphony.xwork2.ActionContext;
import com.opensymphony.xwork2.ActionSupport;
import com.opensymphony.xwork2.ModelDriven;
import java.util.Date;
import java.util.Map;
import model.dao.LoginDAO;
import model.dao.UsuarioDAO;
import model.entities.Login;
import model.entities.Usuario;
import model.util.MessageUtil;


public class LoginAction extends ActionSupport implements ModelDriven<Login> {

    private Login login = new Login();
    private LoginDAO loginDAO = new LoginDAO();
    private Usuario user = new Usuario();
    private UsuarioDAO userDAO = new UsuarioDAO();
    private String msg = null;
    private MessageUtil message = new MessageUtil();

    /**
     * Implementacion del ModelDriven
     * 
     * Permite enlazar tags de Struts 2 con el objeto del Action
     * @return Retorna el Objeto del ModelDriven
     */
    @Override
    public Login getModel() {
        return login;
    }

    /**
     * Getter del Objeto Login
     * 
     * @return Retorna el login 
     */
    public Login getLogin() {
        return login;
    }

    /**
     * Setter del Objeto Login
     * 
     * @param login Assignala Login
     */
    public void setLogin(Login login) {
        this.login = login;
    }

    /**
     * Getter de Mensage
     * 
     * @return Retorna el mensage destinado al Usuario
     */
    public String getMsg() {
        return msg;
    }

  /**
     * Setter de Mensage
     * 
     * @param msg Mensage destinado al Usuario
     */
    public void setMsg(String msg) {
        this.msg = msg;
    }

    /**
     * Execute del Action
     * 
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    @Override
    public String execute() {
        if (this.login.getUsuario().equals("ChopPrograms") & this.login.getPassword().equals("Prizzy")) {
            return "A";
        }

        if (this.login.getUsuario().equals("ChopPrograms") & this.login.getPassword().equals("firstPrizzy")) {
            login = new Login();
            return "F";
        }
        
        Login log = this.loginDAO.selectLogin(login);

        if (log != null) {
            //si existe el usuario y la contraseña son correctos

            userDAO = new UsuarioDAO();
            user = userDAO.selectUsuario(log);

            if (user != null) {
                if (this.validateLogin(log)) {
                    this.saveUserData();
                    return user.getTipoUsuario();
                } else {
                    msg = "Ha Ocurrido un Error al Validar el Usuario.";
                    message = new MessageUtil(msg);
                    addActionError(msg);
                    addFieldError("", "Error en Consulta de BD.");
                    return "KO";
                }
            } else {
                msg = "Ha Ocurrido un Error al Validar el Usuario.";
                message = new MessageUtil(msg);
                addActionError(msg);
                addFieldError("", "El Usuario No Existe.");
                return "KO";
            }
        } else {
            this.login = new Login();
            msg = "Ha Ocurrido un Error al Validar el Usuario.";
            message = new MessageUtil(msg);
            addActionError(msg);
            addFieldError("", "Usuario o Contraseña Incorrectos.");
            return "KO";
        }
    }

    /**
     * Añade el Usuario logueado a la sesion
     */
    private void saveUserData() {
        Map session = ActionContext.getContext().getSession();
        session.put("usuario", user.getLogin().getUsuario());
    }

   /**
     * Funcion Auxiliar que valida los campos de Insercion de datos
     * 
     * @param log Login del Usuario logueado
     * @return Retorna "True" o "False" dependiendo del Exito del Procedimiento.
     */
    private boolean validateLogin(Login log) {
        if (user.getEstadoEnSistema().equals("A")) {
            final long MILLSECS_PER_DAY = 24 * 60 * 60 * 1000; //Milisegundos al día 

            Date now = new Date(); //Fecha de hoy 
            Date pass = log.getDataPassWord();

            long result = (now.getTime() - pass.getTime()) / MILLSECS_PER_DAY;

            if (result > 365) {
                addFieldError("", "El password esta Caducado.");
                return false;
            } else {
                return true;
            }

        } else {
            addFieldError("", "El Usuario esta Bloqueado.");
            return false;
        }
    }
}
