/**
  * Login Controller.
  * 
  * Responde a eventos referente a la CLASE LOGIN ,invoca peticiones al 'modelo' cuando se hace alguna solicitud sobre la información
  * (por ejemplo, editar un documento o un registro en una base de datos). 
  * 
  * También puede enviar comandos a su 'vista' asociada si se solicita un cambio en la forma en que se presenta de 'modelo' 
  * (por ejemplo, desplazamiento o scroll por un documento o por los diferentes registros de una base de datos), 
  * por tanto se podría decir que el 'controlador' hace de intermediario entre la 'vista' y el 'modelo'.
  * 
  * @author JuanRamon Segura Ponce
  *
  * @version v.1.0
  * @throws Throws in DAO
  */

package controller.login;

import com.opensymphony.xwork2.ActionContext;
import com.opensymphony.xwork2.ActionSupport;
import com.opensymphony.xwork2.ModelDriven;
import java.util.ArrayList;
import java.util.Map;
import model.dao.LoginDAO;
import model.dao.UsuarioDAO;
import model.entities.Login;
import model.entities.Usuario;
import model.util.MessageUtil;


public class LoginController extends ActionSupport implements ModelDriven<Login> {

    private Login login = new Login();
    private ArrayList<Login> listLogin = new ArrayList<Login>();
    private LoginDAO loginDAO = new LoginDAO();
    private Usuario current = null;
    private String pass1 = null;
    private String pass2 = null;
    private String msg = null;
    private MessageUtil message = new MessageUtil();

    /**
     * Implementacion del ModelDriven
     * 
     * Permite enlazar tags de Struts 2 con el objeto del Action
     * @return Retorna el Objeto del ModelDriven
     */
    @Override
    public Login getModel() {
        return login;
    }

    /**
     * Getter del Usuario Logueado
     * 
     * @return Retorna el Usuario Logueado
     */
    public Usuario getCurrent() {
        return current;
    }

    /**
     * Setter del Usuario Logeado
     * 
     * @param current Retorna el Usuario Logueado.
     */
    public void setCurrent(Usuario current) {
        this.current = current;
    }

     /**
     * Getter del Objeto Login
     * 
     * @return Retorna el login 
     */
    public Login getLogin() {
        return login;
    }

    /**
     * Setter del Objeto Login
     * 
     * @param login Assignala Login
     */
    public void setLogin(Login login) {
        this.login = login;
    }

     /**
     * Getter de la lista de Logins
     * 
     * @param listLogin Assigna la lista de Logins
     */
    public ArrayList<Login> getListLogin() {
        return listLogin;
    }

     /**
     * Setter de la lista de Logins
     * 
     * @param listLogin Assigna la lista de Logins
     */
    public void setListLogin(ArrayList<Login> listLogin) {
        this.listLogin = listLogin;
    }

     /**
     * Getter del password 1
     * 
     * @return Retorna el login 
     */
    public String getPass1() {
        return pass1;
    }

     /**
     * Setter del password 1
     * 
     * @return Retorna el login 
     */
    public void setPass1(String pass1) {
        this.pass1 = pass1;
    }

     /**
     * Getter del password 2
     * 
     * @return Retorna el login 
     */
    public String getPass2() {
        return pass2;
    }

     /**
     * Setter del password 2
     * 
     * @return Retorna el login 
     */
    public void setPass2(String pass2) {
        this.pass2 = pass2;
    }

   /**
     * Getter de Mensage
     * 
     * @return Retorna el mensage destinado al Usuario
     */
    public String getMsg() {
        return msg;
    }

   /**
     * Setter de Mensage
     * 
     * @param msg Mensage destinado al Usuario
     */
    public void setMsg(String msg) {
        this.msg = msg;
    }

      /**
     * Modifica manualmente el Login
     * 
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String updateManualLogin() {
        Login log = loginDAO.selectLogin(login);

        if (log != null) {
            if (pass1.equals(pass2)) {
                if (!pass1.equals(log.getPassword())) {
                    return this.updateLogin();
                } else {
                    msg = "La Contraseña Nueva Debe ser Diferente de la Anterior.";
                    addFieldError("", msg);
                    return "KO";
                }
            } else {
                msg = "La Contraseña de Confirmacion es Incorrecta.";
                addFieldError("", msg);
                return "KO";
            }
        } else {
            msg = "La Contraseña es Incorrecta.";
            addFieldError("", msg);
            return "KO";
        }
    }

    //----------------------------------------------------
    //------------------OPERACIONES LOGIN-----------------
    //----------------------------------------------------
        /**
     * Selecciona el Login mediante Usuario
     * 
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectLoginUsuario() {
        if ((login = loginDAO.selectLogin(login.getUsuario())) != null) {
            msg = "El login se ha Seleccionado Correctamente.";
        } else {
            msg = "El Usuario no Existe.";
            addFieldError("", msg);
            return "KO";
        }
        return "OK";
    }

    /**
     * Insertar Login
     * 
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String insertLogin() {
        msg = "Desea Insertar el Password??";
        if (message.showConfirmationMessage(msg) != -1) {
            if (loginDAO.insertLogin(login)) {
                msg = "El login ha sido Insertado Correctamente.";
                return "OK";
            } else {
                msg = "Ha Ocurrido un Error al Insertar el login.";
                return "KO";
            }
        } else {
            msg = "Processo Abortado.";
            return "KO";
        }
    }

    /**
     * Selecciona el Login mediante ID
     * 
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectLoginId() {
        if ((login = loginDAO.selectLogin(login.getIdLogin())) != null) {
            msg = "El login se ha Seleccionado Correctamente.";
        } else {
            msg = "Ha Ocurrido un Error al Seleccionar el login.";
            return "KO";
        }
        return "OK";
    }

    /**
     * Selecciona todos los Logines
     * 
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectAllLogin() {
        if ((listLogin = loginDAO.selectAll()) != null) {
            msg = "Los logins se ha Seleccionado Correctamente.";
        } else {
            msg = "Ha Ocurrido un Error al Seleccionar los logins.";
            return "KO";
        }
        return "OK";
    }

    /**
     * Selecciona los Logines que coinciden con el Ejemplo
     * 
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String selectByExampleLogin() {

        if ((listLogin = loginDAO.selectByExample(login)) != null) {
            msg = "El login se ha Seleccionado Correctamente.";
        } else {
            msg = "Ha Ocurrido un Error al Seleccionar el login.";
            return "KO";
        }
        return "OK";
    }

    /**
     * Elimina el Login mediante ID
     * 
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String deleteLoginId() {
        msg = "Desea Eliminar el Password??";
        if (message.showConfirmationMessage(msg) != -1) {
            if (loginDAO.deleteLogin(login)) {
                msg = "El login ha sido Eliminado Correctamente.";
                return "OK";
            } else {
                msg = "Ha Ocurrido un Error al Eliminar el login.";
                return "KO";
            }
        } else {
            msg = "Processo Abortado.";
            return "KO";
        }
    }

    /**
     * Elimina el Login
     * 
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String deleteLogin() {
        msg = "Desea Eliminar el Password??";
        if (message.showConfirmationMessage(msg) != -1) {
            if (loginDAO.deleteLogin(login)) {
                msg = "El login ha sido Eliminado Correctamente.";
                return "OK";
            } else {
                msg = "Ha Ocurrido un Error al Eliminar el login.";
                return "KO";
            }
        } else {
            msg = "Processo Abortado.";
            return "KO";
        }
    }

  /**
     * Modifica el Login
     * 
     * @return Retorna "OK" o "KO" dependiendo del Exito del Procedimiento.
     */
    public String updateLogin() {
        msg = "Desea Modificar el Password??";
        if (message.showConfirmationMessage(msg) != -1) {
            if (loginDAO.updateLogin(login)) {
                msg = "El login ha sido Eliminado Correctamente.";
                return "OK";
            } else {
                msg = "Ha Ocurrido un Error al Eliminar el login.";
                return "KO";
            }
        } else {
            msg = "Processo Abortado.";
            return "KO";
        }
    }

        /**
     * Abrir Pantalla de Modificacion de Login
     * 
     * @return Retorna "PANTALLA" dando el "OK" para cargar la Pagina
     */
    public String screenUpdateManualLogin() {
        Map session = ActionContext.getContext().getSession();

        login = loginDAO.selectLogin((String) session.get("usuario"));

        //borramos el password para obligar al usuario a introducirlo
        login.setPassword("");

        UsuarioDAO userDAO = new UsuarioDAO();
        current = userDAO.selectUsuario(login);
        return "PANTALLA";
    }
}
