/**
 * Articulo.
 *
 * Es la entidad que se encarga de encapsular toda la informacion referente al
 * ARTICULO
 *
 * Los ENTITIES se usan para encapsular varios objetos en un único objeto (la
 * vaina o Bean en inglés), para hacer uso de un solo objeto en lugar de varios
 * más simples, son clases simples y que no dependen de un framework en
 * especial.
 *
 * Para funcionar como una clase JavaBean, una clase debe obedecer ciertas
 * convenciones sobre nomenclatura de métodos, construcción y comportamiento.
 *
 * Estas convenciones permiten tener herramientas que puedan utilizar,
 * reutilizar, sustituir y conectar JavaBeans.
 *
 * Sus propiedades deben ser accesibles mediante métodos get y set que siguen
 * una convención de nomenclatura estándar y debe ser serializable.
 *
 * Dentro de un JavaBean podemos distinguir tres partes: Propiedades: Los
 * atributos que contiene. Métodos: Se establecen los métodos get y set para
 * acceder y modificar los atributos. Eventos: Permiten comunicar con otros
 * JavaBeans.
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws --> No Throws
 */
package model.entities;

import java.util.ArrayList;

public class Articulo implements java.io.Serializable {

    private Integer idArticulo;
    private String descripcion1;
    private String descripcion2;
    private String descripcion3;
    private Float precio;
    private Float precioOferta;
    private Integer tecla;
    private String unidad;
    private String codInterno;
    private String ean;

    /**
     * Constructor Vacio de la Clase.
     */
    public Articulo() {
    }

    /**
     * Constructor con los parametros minimos "NOT NULL"
     *
     * @param descripcion1 Propiedad del Articulo.
     * @param descripcion2 Propiedad del Articulo.
     * @param precio Propiedad del Articulo.
     * @param unidad Propiedad del Articulo.
     * @param ean Propiedad del Articulo.
     */
    public Articulo(String descripcion1, String descripcion2, Float precio, String unidad, String ean) {
        this.descripcion1 = descripcion1;
        this.descripcion2 = descripcion2;
        this.precio = precio;
        this.unidad = unidad;
        this.ean = ean;
    }

    /**
     * Constructor con todos los parametros del Objeto.
     *
     * @param descripcion1 Propiedad del Articulo.
     * @param descripcion2 Propiedad del Articulo.
     * @param descripcion3 Propiedad del Articulo.
     * @param precio Propiedad del Articulo.
     * @param precioOferta Propiedad del Articulo.
     * @param tecla Propiedad del Articulo.
     * @param unidad Propiedad del Articulo.
     * @param codInterno Propiedad del Articulo.
     * @param ean Propiedad del Articulo.
     */
    public Articulo(String descripcion1, String descripcion2, String descripcion3, Float precio, Float precioOferta, Integer tecla, String unidad, String codInterno, String ean) {
        this.descripcion1 = descripcion1;
        this.descripcion2 = descripcion2;
        this.descripcion3 = descripcion3;
        this.precio = precio;
        this.precioOferta = precioOferta;
        this.tecla = tecla;
        this.unidad = unidad;
        this.codInterno = codInterno;
        this.ean = ean;
    }

    /**
     * Getter de la propiedad IdArticulo.
     *
     * @return ID Propiedad del Articulo. (idArticulo)
     */
    public Integer getIdArticulo() {
        return this.idArticulo;
    }

    /**
     * Setter de la propiedad IdArticulo
     *
     * @param idArticulo Propiedad del Articulo.
     */
    public void setIdArticulo(Integer idArticulo) {
        this.idArticulo = idArticulo;
    }

    /**
     * Getter de la propiedad Descripcion 1.
     *
     * @return Retorna la propiedad.
     */
    public String getDescripcion1() {
        return this.descripcion1;
    }

    /**
     * Setter de la propiedad Descripcion 1
     *
     * @param descripcion1 Propiedad del Articulo.
     */
    public void setDescripcion1(String descripcion1) {
        this.descripcion1 = descripcion1;
    }

    /**
     *
     * Getter de la propiedad Descripcion 2.
     *
     * @return Retorna la propiedad.
     */
    public String getDescripcion2() {
        return this.descripcion2;
    }

    /**
     * Setter de la propiedad Descripcion 2.
     *
     * @param descripcion2 Propiedad del Articulo.
     */
    public void setDescripcion2(String descripcion2) {
        this.descripcion2 = descripcion2;
    }

    /**
     * Getter de la propiedad Descripcion 3.
     *
     * @return Retorna la propiedad.
     */
    public String getDescripcion3() {
        return this.descripcion3;
    }

    /**
     * Setter de la propiedad Descripcion 3.
     *
     * @param descripcion3 Propiedad del Articulo.
     */
    public void setDescripcion3(String descripcion3) {
        this.descripcion3 = descripcion3;
    }

    /**
     * Getter de la propiedad Precio.
     *
     * @return Retorna la propiedad.
     */
    public Float getPrecio() {
        return this.precio;
    }

    /**
     * Setter de la propiedad Precio.
     *
     * @param precio Propiedad del Articulo.
     */
    public void setPrecio(Float precio) {
        this.precio = precio;
    }

    /**
     * Getter de la propiedad Precio Oferta.
     *
     * @return Retorna la propiedad.
     */
    public Float getPrecioOferta() {
        return this.precioOferta;
    }

    /**
     * Setter de la propiedad Precio Oferta.
     *
     * @param precioOferta Propiedad del Articulo.
     */
    public void setPrecioOferta(Float precioOferta) {
        this.precioOferta = precioOferta;
    }

    /**
     * Getter de la propiedad Tecla
     *
     * @return Retorna la propiedad.
     */
    public Integer getTecla() {
        return this.tecla;
    }

    /**
     * Setter de la propiedad Tecla.
     *
     * @param tecla Retorna la propiedad.
     */
    public void setTecla(Integer tecla) {
        this.tecla = tecla;
    }

    /**
     * Getter de la propiedad Unidad.
     *
     * @return Retorna la propiedad.
     */
    public String getUnidad() {
        return this.unidad;
    }

    /**
     * Setter de la propiedad Unidad.
     *
     * @param unidad Propiedad del Articulo.
     */
    public void setUnidad(String unidad) {
        this.unidad = unidad;
    }

    /**
     * Getter de la propiedad Codigo Interno (SMS)
     *
     * @return Retorna la propiedad.
     */
    public String getCodInterno() {
        return this.codInterno;
    }

    /**
     * Setter de la propiedad Codigo Interno.
     *
     * @param codInterno Propiedad del Articulo.
     */
    public void setCodInterno(String codInterno) {
        this.codInterno = codInterno;
    }

    /**
     * Getter de la propiedad Ean.
     *
     * @return Retorna la propiedad.
     */
    public String getEan() {
        return this.ean;
    }

    /**
     * Setter de la propiedad Ean
     *
     * @param ean Propiedad del Articulo.
     */
    public void setEan(String ean) {
        this.ean = ean;
    }

    /**
     * Coloca todos los valores de las propiedades a Null
     */
    public void reset() {
        idArticulo = null;
        descripcion1 = null;
        descripcion2 = null;
        descripcion3 = null;
        precio = null;
        precioOferta = null;
        tecla = null;
        unidad = null;
        codInterno = null;
        ean = null;
    }

    /**
     * Utilizado en las consultas byExample. Coloca las propiedades a null
     * excepto la propiedad por la que se quiere buscar.
     *
     * @param parameter parametro de Seleccion.
     */
    public void onlyParameter(String parameter) {
        if (parameter.equals("Buscar por ID")) {
            descripcion1 = null;
            descripcion2 = null;
            descripcion3 = null;
            precio = null;
            precioOferta = null;
            tecla = null;
            unidad = null;
            codInterno = null;
            ean = null;
        }

        if (parameter.equals("Buscar por EAN")) {
            idArticulo = null;
            descripcion1 = null;
            descripcion2 = null;
            descripcion3 = null;
            precio = null;
            precioOferta = null;
            tecla = null;
            unidad = null;
            codInterno = null;
        }

        if (parameter.equals("Buscar por SMS")) {
            idArticulo = null;
            descripcion1 = null;
            descripcion2 = null;
            descripcion3 = null;
            precio = null;
            precioOferta = null;
            tecla = null;
            unidad = null;
            ean = null;
        }

        if (parameter.equals("Buscar por Tecla")) {
            idArticulo = null;
            descripcion1 = null;
            descripcion2 = null;
            descripcion3 = null;
            precio = null;
            precioOferta = null;
            unidad = null;
            codInterno = null;
            ean = null;
        }
    }


    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Articulo other = (Articulo) obj;
        if (this.idArticulo != other.idArticulo && (this.idArticulo == null || !this.idArticulo.equals(other.idArticulo))) {
            return false;
        }
        if ((this.descripcion1 == null) ? (other.descripcion1 != null) : !this.descripcion1.equals(other.descripcion1)) {
            return false;
        }
        if ((this.descripcion2 == null) ? (other.descripcion2 != null) : !this.descripcion2.equals(other.descripcion2)) {
            return false;
        }
        if ((this.descripcion3 == null) ? (other.descripcion3 != null) : !this.descripcion3.equals(other.descripcion3)) {
            return false;
        }
        if (this.precio != other.precio && (this.precio == null || !this.precio.equals(other.precio))) {
            return false;
        }
        if (this.precioOferta != other.precioOferta && (this.precioOferta == null || !this.precioOferta.equals(other.precioOferta))) {
            return false;
        }
        if (this.tecla != other.tecla && (this.tecla == null || !this.tecla.equals(other.tecla))) {
            return false;
        }
        if ((this.unidad == null) ? (other.unidad != null) : !this.unidad.equals(other.unidad)) {
            return false;
        }
        if ((this.codInterno == null) ? (other.codInterno != null) : !this.codInterno.equals(other.codInterno)) {
            return false;
        }
        if ((this.ean == null) ? (other.ean != null) : !this.ean.equals(other.ean)) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        return "Articulo{" + "idArticulo=" + idArticulo + ", descripcion1=" + descripcion1 + ", descripcion2=" + descripcion2 + ", descripcion3=" + descripcion3 + ", precio=" + precio + ", precioOferta=" + precioOferta + ", tecla=" + tecla + ", unidad=" + unidad + ", codInterno=" + codInterno + ", ean=" + ean + '}';
    }

    /**
     * To String adaptado para la visualizacion de datos por pantalla
     *
     * @return
     */
        public String toStr() {
        String str = descripcion1 + descripcion2 + " precio=" + precio;
        if (precioOferta != null) {
            str = str + " precio Oferta=" + precioOferta;
        }
        
        return str;
    }
}
