/**
 * Departamento.
 *
 * Es la entidad que se encarga de encapsular toda la informacion referente al
 * DEPARTAMENTO
 *
 * Los ENTITIES se usan para encapsular varios objetos en un único objeto (la
 * vaina o Bean en inglés), para hacer uso de un solo objeto en lugar de varios
 * más simples, son clases simples y que no dependen de un framework en
 * especial.
 *
 * Para funcionar como una clase JavaBean, una clase debe obedecer ciertas
 * convenciones sobre nomenclatura de métodos, construcción y comportamiento.
 *
 * Estas convenciones permiten tener herramientas que puedan utilizar,
 * reutilizar, sustituir y conectar JavaBeans.
 *
 * Sus propiedades deben ser accesibles mediante métodos get y set que siguen
 * una convención de nomenclatura estándar y debe ser serializable.
 *
 * Dentro de un JavaBean podemos distinguir tres partes: Propiedades: Los
 * atributos que contiene. Métodos: Se establecen los métodos get y set para
 * acceder y modificar los atributos. Eventos: Permiten comunicar con otros
 * JavaBeans.
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws --> No Throws
 */
package model.entities;

import java.util.ArrayList;

/**
 *
 * @author Portatil
 */
public class Departamento implements java.io.Serializable {

    private Integer idDepartamento;
    private String departamento;
    private String codigoDepartamento;

    /**
     * Constructor Vacio.
     */
    public Departamento() {
    }

    /**
     * Constructor con los parametros minimos "NOT NULL"
     * 
     * @param departamento Propiedad del Departamento.
     * @param codigoDepartamento Propiedad del Departamento.
     */
    public Departamento(String departamento, String codigoDepartamento) {
        this.departamento = departamento;
        this.codigoDepartamento = codigoDepartamento;
    }

    /**
     * Getter de la propiedad Id Departamento
     * 
     * @return ID del Departamento.
     */
    public Integer getIdDepartamento() {
        return idDepartamento;
    }

    /**
     * Setter de la propiedad Documento
     * 
     * @param idDepartamento Propiedad del Departamento.
     */
    public void setIdDepartamento(Integer idDepartamento) {
        this.idDepartamento = idDepartamento;
    }

    /**
     * Getter de la propiedad Departamento
     * 
     * @return Retorna la propiedad.
     */
    public String getDepartamento() {
        return departamento;
    }

    /**
     * Setter de la propiedad Documento
     * 
     * @param departamento Propiedad del Departamento.
     */
    public void setDepartamento(String departamento) {
        this.departamento = departamento;
    }

    /**
     * Getter de la propiedad Codigo Departamento
     * 
     * @return Retorna la propiedad.
     */
    public String getCodigoDepartamento() {
        return codigoDepartamento;
    }

    /**
     * Setter de la propiedad Codigo Departamento
     * 
     * @param codigoDepartamento Propiedad del Departamento.
     */
    public void setCodigoDepartamento(String codigoDepartamento) {
        this.codigoDepartamento = codigoDepartamento;
    }

    /**
     * Coloca todos los valores de las propiedades a Null
     */
    public void reset() {
        this.idDepartamento = null;
        this.departamento = null;
        this.codigoDepartamento = null;
    }

    /**
     * Utilizado en las consultas byExample. Coloca las propiedades a null
     * excepto la propiedad por la que se quiere buscar.
     * 
     * @param parameter parametro de Seleccion
     */
    public void onlyParameter(String parameter) {
        if (parameter.equals("Buscar por ID")) {
            this.departamento = null;
            this.codigoDepartamento = null;
        }

        if (parameter.equals("Buscar por CodigoDepartamento")) {
            this.departamento = null;
            this.idDepartamento = null;
        }

    }

        
    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Departamento other = (Departamento) obj;
        if (this.idDepartamento != other.idDepartamento && (this.idDepartamento == null || !this.idDepartamento.equals(other.idDepartamento))) {
            return false;
        }
        if ((this.departamento == null) ? (other.departamento != null) : !this.departamento.equals(other.departamento)) {
            return false;
        }
        if ((this.codigoDepartamento == null) ? (other.codigoDepartamento != null) : !this.codigoDepartamento.equals(other.codigoDepartamento)) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        return "Departamento{" + "idDepartamento=" + idDepartamento + ", departamento=" + departamento + ", codigoDepartamento=" + codigoDepartamento + '}';
    }
}
