/**
 * Documento.
 *
 * Es la entidad que se encarga de encapsular toda la informacion referente al
 * DOCUMENTO
 *
 * Los ENTITIES se usan para encapsular varios objetos en un único objeto (la
 * vaina o Bean en inglés), para hacer uso de un solo objeto en lugar de varios
 * más simples, son clases simples y que no dependen de un framework en
 * especial.
 *
 * Para funcionar como una clase JavaBean, una clase debe obedecer ciertas
 * convenciones sobre nomenclatura de métodos, construcción y comportamiento.
 *
 * Estas convenciones permiten tener herramientas que puedan utilizar,
 * reutilizar, sustituir y conectar JavaBeans.
 *
 * Sus propiedades deben ser accesibles mediante métodos get y set que siguen
 * una convención de nomenclatura estándar y debe ser serializable.
 *
 * Dentro de un JavaBean podemos distinguir tres partes: Propiedades: Los
 * atributos que contiene. Métodos: Se establecen los métodos get y set para
 * acceder y modificar los atributos. Eventos: Permiten comunicar con otros
 * JavaBeans.
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws --> No Throws
 */

package model.entities;

import java.util.ArrayList;

public class Documento implements java.io.Serializable {

    private Integer idDocumento;
    private String numeroDocumento;
    private String tipoDocumento;

    /**
     * Constructor Vacio
     */
    public Documento() {
    }

    /**
     * Constructor con los parametros minimos "NOT NULL"
     * 
     * @param numeroDocumento Propiedad del Documento
     * @param tipoDocumento Propiedad del Documento
     */
    public Documento(String numeroDocumento, String tipoDocumento) {
        this.numeroDocumento = numeroDocumento;
        this.tipoDocumento = tipoDocumento;
    }

    /**
     * Getter de la propiedad Documento
     * 
     * @return Retorna la propiedad.
     */
    public Integer getIdDocumento() {
        return this.idDocumento;
    }

    /**
     * Setter de la propiedad Id del Documento
     * 
     * @param idDocumento Propiedad del Documento
     */
    public void setIdDocumento(Integer idDocumento) {
        this.idDocumento = idDocumento;
    }

    /**
     * Getter de la propiedad Numero Documento
     * 
     * @return Retorna la propiedad.
     */
    public String getNumeroDocumento() {
        return this.numeroDocumento;
    }

    /** 
     * Setter de la propiedad Numero Documento
     * 
     * @param numeroDocumento Propiedad del Documento
     */
    public void setNumeroDocumento(String numeroDocumento) {
        this.numeroDocumento = numeroDocumento;
    }

    /**
     * Getter de la Tipo Documento
     * 
     * @return Retorna la propiedad.
     */
    public String getTipoDocumento() {
        return this.tipoDocumento;
    }

    /**
     * Setter de la propiedad Tipo Documento
     * 
     * @param tipoDocumento Propiedad del Documento
     */
    public void setTipoDocumento(String tipoDocumento) {
        this.tipoDocumento = tipoDocumento;
    }

    /**
     * Coloca todos los valores de las propiedades a Null
     */
    public void reset() {
        this.idDocumento = null;
        this.numeroDocumento = null;
        this.tipoDocumento = null;
    }

    /**
     * Utilizado en las consultas byExample. Coloca las propiedades a null
     * excepto la propiedad por la que se quiere buscar.
     * 
     * @param parameter parametro de Seleccion
     */
    public void onlyParameter(String parameter) {
        if (parameter.equals("Buscar por ID")) {
            this.numeroDocumento = null;
            this.tipoDocumento = null;
        }

        if (parameter.equals("Buscar por Identificacion")) {
            this.idDocumento = null;
            this.tipoDocumento = null;
        }

        if (parameter.equals("Buscar por Tipo Documento")) {
            this.idDocumento = null;
            this.numeroDocumento = null;

        }
    }

    
    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Documento other = (Documento) obj;
        if (this.idDocumento != other.idDocumento && (this.idDocumento == null || !this.idDocumento.equals(other.idDocumento))) {
            return false;
        }
        if ((this.numeroDocumento == null) ? (other.numeroDocumento != null) : !this.numeroDocumento.equals(other.numeroDocumento)) {
            return false;
        }
        if ((this.tipoDocumento == null) ? (other.tipoDocumento != null) : !this.tipoDocumento.equals(other.tipoDocumento)) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        return "Documento{" + "idDocumento=" + idDocumento + "," + "numeroDocumento=" + numeroDocumento + ", tipoDocumento=" + tipoDocumento + '}';

    }
}
