/**
 * Estadotrabajos.
 *
 * Es la entidad que se encarga de encapsular toda la informacion referente al
 * estado de un trabajo dependiento del usuario conectado
 *
 * Los ENTITIES se usan para encapsular varios objetos en un único objeto (la
 * vaina o Bean en inglés), para hacer uso de un solo objeto en lugar de varios
 * más simples, son clases simples y que no dependen de un framework en
 * especial.
 *
 * Para funcionar como una clase JavaBean, una clase debe obedecer ciertas
 * convenciones sobre nomenclatura de métodos, construcción y comportamiento.
 *
 * Estas convenciones permiten tener herramientas que puedan utilizar,
 * reutilizar, sustituir y conectar JavaBeans.
 *
 * Sus propiedades deben ser accesibles mediante métodos get y set que siguen
 * una convención de nomenclatura estándar y debe ser serializable.
 *
 * Dentro de un JavaBean podemos distinguir tres partes: Propiedades: Los
 * atributos que contiene. Métodos: Se establecen los métodos get y set para
 * acceder y modificar los atributos. Eventos: Permiten comunicar con otros
 * JavaBeans.
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws --> No Throws
 */
package model.entities;

import java.util.ArrayList;
import java.util.Date;

public class Estadotrabajo implements java.io.Serializable {

    private Integer idEstadoTrabajo;
    private Trabajo trabajo;
    private Usuario usuario;
    private String estado;
    private Date fechaEstado;

    /**
     * Constructor Vacio.
     */
    public Estadotrabajo() {
    }

    /**
     * Constructor con los parametros minimos "NOT NULL"
     * 
     * @param trabajo Propiedad del EstadoTrabajo.
     * @param usuario Propiedad del EstadoTrabajo.
     * @param estado Propiedad del EstadoTrabajo.
     * @param fechaEstado Propiedad del EstadoTrabajo.
     */
    public Estadotrabajo(Trabajo trabajo, Usuario usuario, String estado, Date fechaEstado) {
        this.trabajo = trabajo;
        this.usuario = usuario;
        this.estado = estado;
        this.fechaEstado = fechaEstado;
    }

    /**
     * Getter de la propiedad Id Estado Trabajo
     * 
     * @return Retorna la Propiedad.
     */
    public Integer getIdEstadoTrabajo() {
        return this.idEstadoTrabajo;
    }

    /**
     * Setter de la propiedad ID TRABAJO
     * 
     * @param idEstadoTrabajo Propiedad del EstadoTrabajo.
     */
    public void setIdEstadoTrabajo(Integer idEstadoTrabajo) {
        this.idEstadoTrabajo = idEstadoTrabajo;
    }

    /**
     * Getter de la propiedad Trabajo
     * 
     * @return Retorna la Propiedad.
     */
    public Trabajo getTrabajo() {
        return this.trabajo;
    }

    /**
     * Setter de la propiedad trabajo
     * 
     * @param trabajo Propiedad del EstadoTrabajo.
     */
    public void setTrabajo(Trabajo trabajo) {
        this.trabajo = trabajo;
    }

    /**
     * Getter de la propiedad Usuario
     * 
     * @return Retorna la Propiedad.
     */
    public Usuario getUsuario() {
        return this.usuario;
    }

    /**
     * Setter de la propiedad Usuario
     *
     * @param usuario Propiedad del EstadoTrabajo.
     */
    public void setUsuario(Usuario usuario) {
        this.usuario = usuario;
    }

    /**
     * Getter de la propiedad Estado
     * 
     * @return Retorna la Propiedad.
     */
    public String getEstado() {
        return this.estado;
    }

    /**
     * Setter de la propiedad Estado
     * 
     * @param estado Propiedad del EstadoTrabajo.
     */
    public void setEstado(String estado) {
        this.estado = estado;
    }

    /**
     * Getter de la propiedad Fecha Estado
     * 
     * @return Retorna la Propiedad.
     */
    public Date getFechaEstado() {
        return this.fechaEstado;
    }

    /**
     * Setter de la propiedad Fecha Estado
     * 
     * @param fechaEstado Propiedad del EstadoTrabajo.
     */
    public void setFechaEstado(Date fechaEstado) {
        this.fechaEstado = fechaEstado;
    }


    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Estadotrabajo other = (Estadotrabajo) obj;
        if (this.idEstadoTrabajo != other.idEstadoTrabajo && (this.idEstadoTrabajo == null || !this.idEstadoTrabajo.equals(other.idEstadoTrabajo))) {
            return false;
        }
        if (this.trabajo != other.trabajo && (this.trabajo == null || !this.trabajo.equals(other.trabajo))) {
            return false;
        }
        if (this.usuario != other.usuario && (this.usuario == null || !this.usuario.equals(other.usuario))) {
            return false;
        }
        if ((this.estado == null) ? (other.estado != null) : !this.estado.equals(other.estado)) {
            return false;
        }
        if (this.fechaEstado != other.fechaEstado && (this.fechaEstado == null || !this.fechaEstado.equals(other.fechaEstado))) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        return "Estadotrabajo{" + "idEstadoTrabajo=" + idEstadoTrabajo + ", trabajo=" + trabajo + ", usuario=" + usuario + ", estado=" + estado + ", fechaEstado=" + fechaEstado + '}';
    }
}
