/**
 * Incidencia.
 *
 * Es la entidad que se encarga de encapsular toda la informacion referente al
 * INCIDENCIA
 *
 * Los ENTITIES se usan para encapsular varios objetos en un único objeto (la
 * vaina o Bean en inglés), para hacer uso de un solo objeto en lugar de varios
 * más simples, son clases simples y que no dependen de un framework en
 * especial.
 *
 * Para funcionar como una clase JavaBean, una clase debe obedecer ciertas
 * convenciones sobre nomenclatura de métodos, construcción y comportamiento.
 *
 * Estas convenciones permiten tener herramientas que puedan utilizar,
 * reutilizar, sustituir y conectar JavaBeans.
 *
 * Sus propiedades deben ser accesibles mediante métodos get y set que siguen
 * una convención de nomenclatura estándar y debe ser serializable.
 *
 * Dentro de un JavaBean podemos distinguir tres partes: Propiedades: Los
 * atributos que contiene. Métodos: Se establecen los métodos get y set para
 * acceder y modificar los atributos. Eventos: Permiten comunicar con otros
 * JavaBeans.
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws --> No Throws
 */
package model.entities;

import java.util.Date;

public class Incidencia implements java.io.Serializable {

    private Integer idIncidencia;
    private String origen;
    private String asunto;
    private String incidencia;
    private String destino;
    private String estado;
    private Date dataApertura;
    private String respuesta;
    private Date dataCierre;

    /**
     * Constructor Vacio.
     */
    public Incidencia() {
    }

    /**
     * Constructor con los parametros minimos "NOT NULL"
     *
     * @param origen Propiedad de la Incidencia
     * @param asunto Propiedad de la Incidencia
     * @param incidencia Propiedad de la Incidencia
     * @param destino Propiedad de la Incidencia
     * @param estado Propiedad de la Incidencia
     * @param dataApertura Propiedad de la Incidencia
     */
    public Incidencia(String origen, String asunto, String incidencia, String destino, String estado, Date dataApertura) {
        this.origen = origen;
        this.asunto = asunto;
        this.incidencia = incidencia;
        this.destino = destino;
        this.estado = estado;
        this.dataApertura = dataApertura;
    }

    /**
     * Constructor con todos los parametros del Objeto.
     * 
     * @param origen Propiedad de la Incidencia
     * @param asunto Propiedad de la Incidencia
     * @param incidencia Propiedad de la Incidencia
     * @param destino Propiedad de la Incidencia
     * @param estado Propiedad de la Incidencia
     * @param dataApertura Propiedad de la Incidencia
     * @param respuesta Propiedad de la Incidencia
     * @param dataCierre Propiedad de la Incidencia
     */
    public Incidencia(String origen, String asunto, String incidencia, String destino, String estado, Date dataApertura, String respuesta, Date dataCierre) {
        this.origen = origen;
        this.asunto = asunto;
        this.incidencia = incidencia;
        this.destino = destino;
        this.estado = estado;
        this.dataApertura = dataApertura;
        this.respuesta = respuesta;
        this.dataCierre = dataCierre;
    }

    /**
     * Getter de la propiedad Id Incidencia
     * 
     * @return Retorna la Propiedad.
     */
    public Integer getIdIncidencia() {
        return idIncidencia;
    }

    /**
     * Setter de la propiedad ID Incidencia
     * 
     * @param idIncidencia Propiedad de la Incidencia
     */
    public void setIdIncidencia(Integer idIncidencia) {
        this.idIncidencia = idIncidencia;
    }

    /**
     * Getter de la propiedad Origen
     * 
     * @return Retorna la Propiedad.
     */
    public String getOrigen() {
        return origen;
    }

    /**
     * Setter de la propiedad Origen
     * 
     * @param origen Propiedad de la Incidencia
     */
    public void setOrigen(String origen) {
        this.origen = origen;
    }

    /**
     * Getter de la propiedad Asunto
     * 
     * @return Retorna la Propiedad.
     */
    public String getAsunto() {
        return asunto;
    }

    /**
     * Setter de la propiedad Asunto
     * 
     * @param asunto Propiedad de la Incidencia
     */
    public void setAsunto(String asunto) {
        this.asunto = asunto;
    }

    /**
     *
     * @return Retorna la Propiedad.
     */
    public String getIncidencia() {
        return incidencia;
    }

    /**
     * Setter de la propiedad Incidencia
     * 
     * @param incidencia Propiedad de la Incidencia
     */
    public void setIncidencia(String incidencia) {
        this.incidencia = incidencia;
    }

    /**
     * Getter de la propiedad Destino
     * 
     * @return Retorna la Propiedad.
     */
    public String getDestino() {
        return destino;
    }

    /**
     * Setter de la propiedad Destino
     * 
     * @param destino Propiedad de la Incidencia
     */
    public void setDestino(String destino) {
        this.destino = destino;
    }

    /**
     * Getter de la propiedad Estado
     * 
     * @return Retorna la Propiedad.
     */
    public String getEstado() {
        return estado;
    }

    /**
     * Setter de la propiedad Estado
     * 
     * @param estado Propiedad de la Incidencia
     */
    public void setEstado(String estado) {
        this.estado = estado;
    }

    /**
     * Getter de la propiedad Data Apertura
     * 
     * @return Retorna la Propiedad.
     */
    public Date getDataApertura() {
        return dataApertura;
    }

    /**
     * Setter de la propiedad Data Apertura
     * 
     * @param dataApertura Propiedad de la Incidencia
     */
    public void setDataApertura(Date dataApertura) {
        this.dataApertura = dataApertura;
    }

    /**
     * Getter de la propiedad Respuesta
     * 
     * @return Retorna la Propiedad.
     */
    public String getRespuesta() {
        return respuesta;
    }

    /**
     * Setter de la propiedad Respuesta
     * 
     * @param respuesta Propiedad de la Incidencia
     */
    public void setRespuesta(String respuesta) {
        this.respuesta = respuesta;
    }

    /**
     * Getter de la propiedad Data Cierre
     * 
     * @return Retorna la Propiedad.
     */
    public Date getDataCierre() {
        return dataCierre;
    }

    /**
     * Setter de la propiedad Data Cierre
     * 
     * @param dataCierre Propiedad de la Incidencia
     */
    public void setDataCierre(Date dataCierre) {
        this.dataCierre = dataCierre;
    }

    /**
     * Coloca todos los valores de las propiedades a Null.
     */
    public void reset() {
        this.idIncidencia = null;
        this.asunto = null;
        this.incidencia = null;
        this.origen = null;
        this.destino = null;
        this.estado = null;
        this.dataApertura = null;
        this.respuesta = null;
        this.dataCierre = null;
    }
    
    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Incidencia other = (Incidencia) obj;
        if (this.idIncidencia != other.idIncidencia && (this.idIncidencia == null || !this.idIncidencia.equals(other.idIncidencia))) {
            return false;
        }
        if ((this.origen == null) ? (other.origen != null) : !this.origen.equals(other.origen)) {
            return false;
        }
        if ((this.asunto == null) ? (other.asunto != null) : !this.asunto.equals(other.asunto)) {
            return false;
        }
        if ((this.incidencia == null) ? (other.incidencia != null) : !this.incidencia.equals(other.incidencia)) {
            return false;
        }
        if ((this.destino == null) ? (other.destino != null) : !this.destino.equals(other.destino)) {
            return false;
        }
        if ((this.estado == null) ? (other.estado != null) : !this.estado.equals(other.estado)) {
            return false;
        }
        if (this.dataApertura != other.dataApertura && (this.dataApertura == null || !this.dataApertura.equals(other.dataApertura))) {
            return false;
        }
        if (this.dataCierre != other.dataCierre && (this.dataCierre == null || !this.dataCierre.equals(other.dataCierre))) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        return "Incidencia{" + "idIncidencia=" + idIncidencia + ", asunto=" + asunto + ", incidencia=" + incidencia + ", origen=" + origen + ", destino=" + destino + ", estado=" + estado + ", dataApertura=" + dataApertura + ", respuesta=" + respuesta + ", dataCierre=" + dataCierre + '}';
    }


}
