/**
 * Login.
 *
 * Es la entidad que se encarga de encapsular toda la informacion referente al LOGIN
 * 
 * Los ENTITIES se usan para encapsular varios objetos en un único objeto (la vaina o Bean en inglés), 
 * para hacer uso de un solo objeto en lugar de varios más simples, son clases simples y que no dependen de un framework en especial.
 * 
 * Para funcionar como una clase JavaBean, una clase debe obedecer ciertas convenciones sobre nomenclatura de métodos, construcción y comportamiento.
 * 
 * Estas convenciones permiten tener herramientas que puedan utilizar, reutilizar, sustituir y conectar JavaBeans.
 * 
 * Sus propiedades deben ser accesibles mediante métodos get y set que siguen una convención de nomenclatura estándar y debe ser serializable.
 * 
 * Dentro de un JavaBean podemos distinguir tres partes:
 * Propiedades: Los atributos que contiene.
 * Métodos: Se establecen los métodos get y set para acceder y modificar los atributos.
 * Eventos: Permiten comunicar con otros JavaBeans.
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws --> No Throws
 */

package model.entities;

import java.util.Date;

public class Login implements java.io.Serializable {

    private Integer idLogin;
    private String usuario;
    private String password;
    private Date dataPassWord;

    /**
     * Constructor Vacio.
     */
    public Login() {
    }

    /**
     * Constructor con los parametros minimos "NOT NULL"
     * 
     * @param usuario Propiedad del Login
     * @param password Propiedad del Login
     * @param dataPassWord Propiedad del Login
     */
    public Login(String usuario, String password, Date dataPassWord) {
        this.usuario = usuario;
        this.password = password;
        this.dataPassWord = dataPassWord;
    }

    /**
     * Getter de la propiedad ID Login
     * 
     * @return Retorna la Propiedad.
     */
    public Integer getIdLogin() {
        return this.idLogin;
    }

    /**
     * Setter de la propiedad ID Login
     * 
     * @param idLogin Propiedad del Login
     */
    public void setIdLogin(Integer idLogin) {
        this.idLogin = idLogin;
    }

    /**
     * Getter de la propiedad Usuario
     * 
     * @return Retorna la Propiedad.
     */
    public String getUsuario() {
        return this.usuario;
    }

    /**
     * Setter de la propiedad Usuario
     * 
     * @param usuario Propiedad del Login
     */
    public void setUsuario(String usuario) {
        this.usuario = usuario;
    }

    /**
     * Getter de la propiedad Password
     * 
     * @return Retorna la Propiedad.
     */
    public String getPassword() {
        return this.password;
    }

    /**
     * Setter de la propiedad Password
     * 
     * @param password Propiedad del Login
     */
    public void setPassword(String password) {
        this.password = password;
    }

    /**
     * Getter de la propiedad Data Password
     * 
     * @return Retorna la Propiedad.
     */
    public Date getDataPassWord() {
        return this.dataPassWord;
    }

    /**
     * Setter de la propiedad Data Password
     * 
     * @param dataPassWord Propiedad del Login
     */
    public void setDataPassWord(Date dataPassWord) {
        this.dataPassWord = dataPassWord;
    }

    /**
     * Coloca todos los valores de las propiedades a Null.
     */
    public void reset() {
        idLogin = null;
        usuario = null;
        password = null;
        dataPassWord = null;
    }
    
    @Override
    public String toString() {
        return "Login{" + "idLogin=" + idLogin + ", usuario=" + usuario + ", password=" + password + ", dataPassWord=" + dataPassWord + '}';
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Login other = (Login) obj;
        if (this.idLogin != other.idLogin && (this.idLogin == null || !this.idLogin.equals(other.idLogin))) {
            return false;
        }
        if ((this.usuario == null) ? (other.usuario != null) : !this.usuario.equals(other.usuario)) {
            return false;
        }
        if ((this.password == null) ? (other.password != null) : !this.password.equals(other.password)) {
            return false;
        }
        if (this.dataPassWord != other.dataPassWord && (this.dataPassWord == null || !this.dataPassWord.equals(other.dataPassWord))) {
            return false;
        }
        return true;
    }


}
