/**
 * Persona.
 *
 * Es la entidad que se encarga de encapsular toda la informacion referente al
 * APERSONA
 *
 * Los ENTITIES se usan para encapsular varios objetos en un único objeto (la
 * vaina o Bean en inglés), para hacer uso de un solo objeto en lugar de varios
 * más simples, son clases simples y que no dependen de un framework en
 * especial.
 *
 * Para funcionar como una clase JavaBean, una clase debe obedecer ciertas
 * convenciones sobre nomenclatura de métodos, construcción y comportamiento.
 *
 * Estas convenciones permiten tener herramientas que puedan utilizar,
 * reutilizar, sustituir y conectar JavaBeans.
 *
 * Sus propiedades deben ser accesibles mediante métodos get y set que siguen
 * una convención de nomenclatura estándar y debe ser serializable.
 *
 * Dentro de un JavaBean podemos distinguir tres partes: Propiedades: Los
 * atributos que contiene. Métodos: Se establecen los métodos get y set para
 * acceder y modificar los atributos. Eventos: Permiten comunicar con otros
 * JavaBeans.
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws --> No Throws
 */
package model.entities;


public class Persona implements java.io.Serializable {

    private Integer idPersona;
    private Documento documento;
    private String nombre;
    private String apellido1;
    private String apellido2;
    private String idioma;
    private String telefono;
    private String email;

    /**
     * Constructor Vacio.
     */
    public Persona() {
    }

    /**
     * Constructor con los parametros minimos "NOT NULL"
     * 
     * @param documento Propiedad de la Persona
     * @param nombre Propiedad de la Persona
     * @param apellido1 Propiedad de la Persona
     * @param apellido2 Propiedad de la Persona
     * @param idioma Propiedad de la Persona
     * @param telefono Propiedad de la Persona
     * @param email Propiedad de la Persona
     */
    public Persona(Documento documento, String nombre, String apellido1, String apellido2, String idioma, String telefono, String email) {
        this.documento = documento;
        this.nombre = nombre;
        this.apellido1 = apellido1;
        this.apellido2 = apellido2;
        this.idioma = idioma;
        this.telefono = telefono;
        this.email = email;
    }

    /**
     * Getter de la propiedad ID Persona
     * 
     * @return Retorna la Propiedad.
     */
    public Integer getIdPersona() {
        return this.idPersona;
    }

    /**
     * Setter de la propiedad ID Persona
     * 
     * @param idPersona Propiedad de la Persona
     */ 
    public void setIdPersona(Integer idPersona) {
        this.idPersona = idPersona;
    }

    /**
     * Getter de la propiedad Documento
     * 
     * @return Retorna la Propiedad.
     */
    public Documento getDocumento() {
        return this.documento;
    }

    /**
     * Setter de la propiedad Documento
     * 
     * @param documento Propiedad de la Persona
     */
    public void setDocumento(Documento documento) {
        this.documento = documento;
    }

    /**
     * Getter de la propiedad Nombre
     * 
     * @return Retorna la Propiedad.
     */
    public String getNombre() {
        return this.nombre;
    }

    /**
     * Setter de la propiedad nombre
     * 
     * @param nombre Propiedad de la Persona
     */
    public void setNombre(String nombre) {
        this.nombre = nombre;
    }

    /**
     * Getter de la propiedad Apellido 1
     * 
     * @return Retorna la Propiedad.
     */
    public String getApellido1() {
        return this.apellido1;
    }

    /**
     * Setter de la propiedad nombre Apellido 1
     * 
     * @param apellido1 Propiedad de la Persona
     */
    public void setApellido1(String apellido1) {
        this.apellido1 = apellido1;
    }

    /**
     * Getter de la propiedad Apellido 2
     * 
     * @return Retorna la Propiedad.
     */
    public String getApellido2() {
        return this.apellido2;
    }

    /**
     * Setter de la propiedad Apellido 2
     * 
     * @param apellido2 Propiedad de la Persona
     */
    public void setApellido2(String apellido2) {
        this.apellido2 = apellido2;
    }

    /**
     * Getter de la propiedad Idioma
     *
     * @return Retorna la Propiedad.
     */
    public String getIdioma() {
        return this.idioma;
    }

    /**
     * Setter de la propiedad Idioma
     * 
     * @param idioma Propiedad de la Persona
     */
    public void setIdioma(String idioma) {
        this.idioma = idioma;
    }

    /**
     * Getter de la propiedad Telefono
     * 
     * @return Retorna la Propiedad.
     */
    public String getTelefono() {
        return this.telefono;
    }

    /**
     * Setter de la propiedad Telefono
     * 
     * @param telefono Propiedad de la Persona
     */
    public void setTelefono(String telefono) {
        this.telefono = telefono;
    }

    /**
     * Getter de la propiedad Email
     * 
     * @return Retorna la Propiedad.
     */
    public String getEmail() {
        return this.email;
    }

    /**
     * Setter de la propiedad Email
     * 
     * @param email Propiedad de la Persona
     */
    public void setEmail(String email) {
        this.email = email;
    }

    /**
     * Coloca todos los valores de las propiedades a Null
     */
    public void reset() {
        this.idPersona = null;
        this.documento = null;
        this.nombre = null;
        this.apellido1 = null;
        this.apellido2 = null;
        this.idioma = null;
        this.telefono = null;
        this.email = null;
    }

    /**
     * Utilizado en las consultas byExample. Coloca las propiedades a null
     * excepto la propiedad por la que se quiere buscar.
     * 
     * @param parameter parametro de Seleccion
     */
    public void onlyParameter(String parameter) {

        if (parameter.equals("Buscar por ID")) {

            this.documento = null;
            this.nombre = null;
            this.apellido1 = null;
            this.apellido2 = null;
            this.idioma = null;
            this.telefono = null;
            this.email = null;
        }

        if (parameter.equals("Buscar por Documento")) {

            this.idPersona = null;
            this.nombre = null;
            this.apellido1 = null;
            this.apellido2 = null;
            this.idioma = null;
            this.telefono = null;
            this.email = null;
        }

        if (parameter.equals("Buscar por Telefono")) {

            this.idPersona = null;
            this.documento = null;
            this.nombre = null;
            this.apellido1 = null;
            this.apellido2 = null;
            this.idioma = null;
            this.email = null;
        }

        if (parameter.equals("Buscar por Email")) {

            this.idPersona = null;
            this.documento = null;
            this.nombre = null;
            this.apellido1 = null;
            this.apellido2 = null;
            this.idioma = null;
            this.telefono = null;
        }
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Persona other = (Persona) obj;
        if (this.idPersona != other.idPersona && (this.idPersona == null || !this.idPersona.equals(other.idPersona))) {
            return false;
        }
        if (this.documento != other.documento && (this.documento == null || !this.documento.equals(other.documento))) {
            return false;
        }
        if ((this.nombre == null) ? (other.nombre != null) : !this.nombre.equals(other.nombre)) {
            return false;
        }
        if ((this.apellido1 == null) ? (other.apellido1 != null) : !this.apellido1.equals(other.apellido1)) {
            return false;
        }
        if ((this.apellido2 == null) ? (other.apellido2 != null) : !this.apellido2.equals(other.apellido2)) {
            return false;
        }
        if ((this.idioma == null) ? (other.idioma != null) : !this.idioma.equals(other.idioma)) {
            return false;
        }
        if ((this.telefono == null) ? (other.telefono != null) : !this.telefono.equals(other.telefono)) {
            return false;
        }
        if ((this.email == null) ? (other.email != null) : !this.email.equals(other.email)) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        return "Persona{" + "idPersona=" + idPersona + ", documento=" + documento + ", nombre=" + nombre + ", apellido1=" + apellido1 + ", apellido2=" + apellido2 + ", idioma=" + idioma + ", telefono=" + telefono + ", email=" + email + '}';
    }
}
