/**
 * Plantilla.
 *
 * Es la entidad que se encarga de encapsular toda la informacion referente al
 * PLANTILLA
 *
 * Los ENTITIES se usan para encapsular varios objetos en un único objeto (la
 * vaina o Bean en inglés), para hacer uso de un solo objeto en lugar de varios
 * más simples, son clases simples y que no dependen de un framework en
 * especial.
 *
 * Para funcionar como una clase JavaBean, una clase debe obedecer ciertas
 * convenciones sobre nomenclatura de métodos, construcción y comportamiento.
 *
 * Estas convenciones permiten tener herramientas que puedan utilizar,
 * reutilizar, sustituir y conectar JavaBeans.
 *
 * Sus propiedades deben ser accesibles mediante métodos get y set que siguen
 * una convención de nomenclatura estándar y debe ser serializable.
 *
 * Dentro de un JavaBean podemos distinguir tres partes: Propiedades: Los
 * atributos que contiene. Métodos: Se establecen los métodos get y set para
 * acceder y modificar los atributos. Eventos: Permiten comunicar con otros
 * JavaBeans.
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws --> No Throws
 */
package model.entities;

import java.util.ArrayList;

public class Plantilla implements java.io.Serializable {

    private Integer idPlantilla;
    private String tipo;
    private String tamano;

    /**
     * Constructor Vacio.
     */
    public Plantilla() {
    }

    /**
     * Constructor con los parametros minimos "NOT NULL"
     * 
     * @param tipo Propiedad de la Plantilla
     * @param tamano Propiedad de la Plantilla
     */
    public Plantilla(String tipo, String tamano) {
        this.tipo = tipo;
        this.tamano = tamano;
    }


    /**
     * Getter de la propiedad ID Plantilla
     * 
     * @return Retorna la Propiedad.
     */
    public Integer getIdPlantilla() {
        return this.idPlantilla;
    }

    /**
     * Setter de la propiedad ID Plantilla
     * 
     * @param idPlantilla Propiedad de la Plantilla
     */
    public void setIdPlantilla(Integer idPlantilla) {
        this.idPlantilla = idPlantilla;
    }

    /**
     * Getter de la propiedad Tipo
     * 
     * @return Retorna la Propiedad.
     */
    public String getTipo() {
        return this.tipo;
    }

    /**
     * Setter de la propiedad Tipo
     * 
     * @param tipo Propiedad de la Plantilla
     */
    public void setTipo(String tipo) {
        this.tipo = tipo;
    }

    /**
     * Getter de la propiedad Tamaño
     * 
     * @return Retorna la Propiedad.
     */
    public String getTamano() {
        return this.tamano;
    }

    /**
     * Setter de la propiedad Tamaño
     * 
     * @param tamano Propiedad de la Plantilla
     */
    public void setTamano(String tamano) {
        this.tamano = tamano;
    }

    

    /**
     * Coloca todos los valores de las propiedades a Null
     */
    public void reset() {
        this.idPlantilla = null;
        this.tipo = null;
        this.tamano = null;
    }

    /**
     * Utilizado en las consultas byExample. Coloca las propiedades a null
     * excepto la propiedad por la que se quiere buscar.
     * 
     * @param option parametro de Seleccion
     */
    public void onlyParameter(String option) {
        if (option.equals("Buscar por ID")) {
            this.tipo = null;
            this.tamano = null;
        }

        if (option.equals("Buscar Tipo/Tamaño")) {
            this.idPlantilla = null;
        }

    }

    
    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Plantilla other = (Plantilla) obj;
        if (this.idPlantilla != other.idPlantilla && (this.idPlantilla == null || !this.idPlantilla.equals(other.idPlantilla))) {
            return false;
        }
        if ((this.tipo == null) ? (other.tipo != null) : !this.tipo.equals(other.tipo)) {
            return false;
        }
        if ((this.tamano == null) ? (other.tamano != null) : !this.tamano.equals(other.tamano)) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        return "Plantilla{" + "idPlantilla=" + idPlantilla + ", tipo=" + tipo + ", tamano=" + tamano + '}';
    }



    /**
     * To String adaptado para la visualizacion de datos por pantalla
     * 
     * @return
     */
    public String toStr() {
        return tipo + " " + tamano;
    }
}
