/**
 * Trabajo.
 *
 * Es la entidad que se encarga de encapsular toda la informacion referente al
 * TRABAJO
 *
 * Los ENTITIES se usan para encapsular varios objetos en un único objeto (la
 * vaina o Bean en inglés), para hacer uso de un solo objeto en lugar de varios
 * más simples, son clases simples y que no dependen de un framework en
 * especial.
 *
 * Para funcionar como una clase JavaBean, una clase debe obedecer ciertas
 * convenciones sobre nomenclatura de métodos, construcción y comportamiento.
 *
 * Estas convenciones permiten tener herramientas que puedan utilizar,
 * reutilizar, sustituir y conectar JavaBeans.
 *
 * Sus propiedades deben ser accesibles mediante métodos get y set que siguen
 * una convención de nomenclatura estándar y debe ser serializable.
 *
 * Dentro de un JavaBean podemos distinguir tres partes: Propiedades: Los
 * atributos que contiene. Métodos: Se establecen los métodos get y set para
 * acceder y modificar los atributos. Eventos: Permiten comunicar con otros
 * JavaBeans.
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws --> No Throws
 */
package model.entities;

import java.util.Date;
import java.util.HashSet;
import java.util.Set;

public class Trabajo implements java.io.Serializable {

    private Integer idTrabajo;
    private String descripcion;
    private Integer prioridad;
    private Date fechaCreacion;
    private Set carteles = new HashSet(0);

    /**
     * Constructor Vacio.
     */
    public Trabajo() {
    }

    /**
     * Constructor con los parametros minimos "NOT NULL"
     * 
     * @param descripcion Propiedad de la Trabajo
     * @param prioridad Propiedad de la Trabajo
     * @param fechaCreacion Propiedad de la Trabajo
     */
    public Trabajo(String descripcion, Integer prioridad, Date fechaCreacion) {
        this.descripcion = descripcion;
        this.prioridad = prioridad;
        this.fechaCreacion = fechaCreacion;
    }

    /**
     * Constructor con los parametros minimos "NOT NULL" mediante Clonacion
     * 
     * @param tra Entrada de objeto crea una copia.
     */
    public Trabajo(Trabajo tra) {
        this.descripcion = tra.getDescripcion();
        this.prioridad = tra.getPrioridad();
        this.fechaCreacion = tra.getFechaCreacion();
    }

    /**
     * Getter de la propiedad ID Trabajo
     * 
     * @return Retorna la Propiedad.
     */
    public Integer getIdTrabajo() {
        return this.idTrabajo;
    }

    /**
     * Setter de la propiedad ID Trabajo
     * 
     * @param idTrabajo Propiedad de la Trabajo
     */
    public void setIdTrabajo(Integer idTrabajo) {
        this.idTrabajo = idTrabajo;
    }

    /**
     * Getter de la propiedad Descripcion
     * 
     * @return Retorna la Propiedad.
     */
    public String getDescripcion() {
        return this.descripcion;
    }

    /**
     * Setter de la propiedad Descripcion
     * 
     * @param descripcion Propiedad de la Trabajo
     */
    public void setDescripcion(String descripcion) {
        this.descripcion = descripcion;
    }

    /**
     * Getter de la propiedad Prioridad
     * 
     * @return Retorna la Propiedad.
     */
    public Integer getPrioridad() {
        return this.prioridad;
    }

    /**
     * Setter de la propiedad Prioridad
     * 
     * @param prioridad Propiedad de la Trabajo
     */
    public void setPrioridad(Integer prioridad) {
        this.prioridad = prioridad;
    }

    /**
     * Getter de la propiedad Fecha Creacion
     * 
     * @return Retorna la Propiedad.
     */
    public Date getFechaCreacion() {
        return this.fechaCreacion;
    }

    /**
     * Setter de la propiedad Fecha de Creacion
     * 
     * @param fechaCreacion Propiedad de la Trabajo
     */
    public void setFechaCreacion(Date fechaCreacion) {
        this.fechaCreacion = fechaCreacion;
    }

    /**
     * Getter de la propiedad Carteles (lista de carteles asociados)
     * 
     * @return Retorna la Propiedad.
     */
    public Set getCarteles() {
        return carteles;
    }

    /**
     * Setter de la propiedad Carteles (lista de carteles asociados)
     * 
     * @param carteles Propiedad de la Trabajo (lista de carteles Associados)
     */
    public void setCarteles(Set carteles) {
        this.carteles = carteles;
    }

    /**
     * Associa carteles al trabajo
     * 
     * @param car Cartel a asociar a trabajo
     */
    public void addCarteles(Cartel car) {
        this.carteles.add(car);
    }

    /**
     * Coloca todos los valores de las propiedades a Null
     */
    public void reset() {
        this.idTrabajo = null;
        this.descripcion = null;
        this.prioridad = null;
        this.fechaCreacion = null;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Trabajo other = (Trabajo) obj;
        if (this.idTrabajo != other.idTrabajo && (this.idTrabajo == null || !this.idTrabajo.equals(other.idTrabajo))) {
            return false;
        }
        if ((this.descripcion == null) ? (other.descripcion != null) : !this.descripcion.equals(other.descripcion)) {
            return false;
        }
        if (this.prioridad != other.prioridad && (this.prioridad == null || !this.prioridad.equals(other.prioridad))) {
            return false;
        }
        if (this.fechaCreacion != other.fechaCreacion && (this.fechaCreacion == null || !this.fechaCreacion.equals(other.fechaCreacion))) {
            return false;
        }

        return true;
    }

    @Override
    public String toString() {
        return "Trabajo{" + "idTrabajo=" + idTrabajo + ", descripcion=" + descripcion + ", prioridad=" + prioridad + ", fechaCreacion=" + fechaCreacion + '}';
    }
}
