/**
 * Usuario.
 *
 * Es la entidad que se encarga de encapsular toda la informacion referente al
 * USUARIO
 *
 * Los ENTITIES se usan para encapsular varios objetos en un único objeto (la
 * vaina o Bean en inglés), para hacer uso de un solo objeto en lugar de varios
 * más simples, son clases simples y que no dependen de un framework en
 * especial.
 *
 * Para funcionar como una clase JavaBean, una clase debe obedecer ciertas
 * convenciones sobre nomenclatura de métodos, construcción y comportamiento.
 *
 * Estas convenciones permiten tener herramientas que puedan utilizar,
 * reutilizar, sustituir y conectar JavaBeans.
 *
 * Sus propiedades deben ser accesibles mediante métodos get y set que siguen
 * una convención de nomenclatura estándar y debe ser serializable.
 *
 * Dentro de un JavaBean podemos distinguir tres partes: Propiedades: Los
 * atributos que contiene. Métodos: Se establecen los métodos get y set para
 * acceder y modificar los atributos. Eventos: Permiten comunicar con otros
 * JavaBeans.
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws --> No Throws
 */
package model.entities;

import java.util.HashSet;
import java.util.Set;

public class Usuario implements java.io.Serializable {

    private Integer idUsuario;
    private Persona persona;
    private Login login;
    private Departamento departamento;
    private String tipoUsuario;
    private String estadoEnSistema;
    private String emailUsuario;
    private String telefonoUsuario;
    private Set estadotrabajos = new HashSet(0);

    /**
     * Constructor Vacio.
     */
    public Usuario() {
    }

    /**
     * Constructor con los parametros minimos "NOT NULL"
     * 
     * @param idUsuario Propiedad del Usuario
     * @param login Propiedad del Usuario
     * @param persona Propiedad del Usuario
     * @param departamento Propiedad del Usuario
     * @param tipoUsuario Propiedad del Usuario
     * @param estadoEnSistema Propiedad del Usuario
     * @param emailUsuario Propiedad del Usuario
     * @param telefonoUsuario Propiedad del Usuario
     */
    public Usuario(Integer idUsuario, Login login, Persona persona, Departamento departamento, String tipoUsuario, String estadoEnSistema, String emailUsuario, String telefonoUsuario) {
        this.idUsuario = idUsuario;
        this.login = login;
        this.persona = persona;
        this.departamento = departamento;
        this.tipoUsuario = tipoUsuario;
        this.estadoEnSistema = estadoEnSistema;
        this.emailUsuario = emailUsuario;
        this.telefonoUsuario = telefonoUsuario;
    }
     
    /**
     * Getter de la propiedad ID Usuario
     * 
     * @return Retorna la Propiedad.
     */
    public Integer getIdUsuario() {
        return this.idUsuario;
    }

    /**
     * Setter de la propiedad ID Usuario
     * 
     * @param idUsuario Propiedad del Usuario
     */
    public void setIdUsuario(Integer idUsuario) {
        this.idUsuario = idUsuario;
    }

    /**
     * Getter de la propiedad Login
     * 
     * @return Retorna la Propiedad.
     */
    public Login getLogin() {
        return this.login;
    }

    /**
     * Setter de la propiedad Login
     * 
     * @param login Propiedad del Usuario
     */
    public void setLogin(Login login) {
        this.login = login;
    }

    /**
     * Getter de la propiedad Persona
     * 
     * @return Retorna la Propiedad.
     */
    public Persona getPersona() {
        return this.persona;
    }

    /**
     * Setter de la propiedad Persona
     * 
     * @param persona Propiedad del Usuario
     */
    public void setPersona(Persona persona) {
        this.persona = persona;
    }

    /**
     * Getter de la propiedad Departamento
     * 
     * @return Retorna la Propiedad.
     */
    public Departamento getDepartamento() {
        return this.departamento;
    }

    /**
     * Setter de la propiedad Departamento
     * 
     * @param departamento Propiedad del Usuario
     */
    public void setDepartamento(Departamento departamento) {
        this.departamento = departamento;
    }

    /**
     * Getter de la propiedad Tipo Usuario
     * 
     * @return Retorna la Propiedad.
     */
    public String getTipoUsuario() {
        return this.tipoUsuario;
    }

    /**
     * Setter de la propiedad Tipo Usuario
     * 
     * @param tipoUsuario Propiedad del Usuario
     */
    public void setTipoUsuario(String tipoUsuario) {
        this.tipoUsuario = tipoUsuario;
    }

    /**
     * Getter de la propiedad Estado en Sistema
     * 
     * @return Retorna la Propiedad.
     */
    public String getEstadoEnSistema() {
        return this.estadoEnSistema;
    }

    /** 
     * Setter de la propiedad Estado en Sistema
     * 
     * @param estadoEnSistema Propiedad del Usuario
     */
    public void setEstadoEnSistema(String estadoEnSistema) {
        this.estadoEnSistema = estadoEnSistema;
    }

    /**
     * Getter de la propiedad Email Usuario
     * 
     * @return Retorna la Propiedad.
     */
    public String getEmailUsuario() {
        return this.emailUsuario;
    }

    /**
     * Setter de la propiedad Email Usuario
     * 
     * @param emailUsuario Propiedad del Usuario
     */
    public void setEmailUsuario(String emailUsuario) {
        this.emailUsuario = emailUsuario;
    }

    /**
     * Getter de la propiedad Telefono Usuario
     * 
     * @return Retorna la Propiedad.
     */
    public String getTelefonoUsuario() {
        return this.telefonoUsuario;
    }

    /**
     * Setter de la propiedad Telefono Usuario
     * 
     * @param telefonoUsuario Propiedad del Usuario
     */
    public void setTelefonoUsuario(String telefonoUsuario) {
        this.telefonoUsuario = telefonoUsuario;
    }

    /**
     * Getter de la propiedad Esto de Trabajos
     * 
     * @return Retorna la Propiedad.
     */
    public Set getEstadotrabajos() {
        return estadotrabajos;
    }

    /**
     * Setter de la propiedad Estado Trabajos
     * 
     * @param estadotrabajos Propiedad del Usuario (Lista de carteles que el Usuario ha manipulado)
     */
    public void setEstadotrabajos(Set estadotrabajos) {
        this.estadotrabajos = estadotrabajos;
    }

    /**
     * Añade un Estado Trabajo a la lista
     * 
     * @param est Relacion entre Usuario y trabajo manipulado (impreso, rechazado)
     */
    public void addEstadotrabajos(Estadotrabajo est) {
        this.estadotrabajos.add(est);
    }


    /**
     * Coloca todos los valores de las propiedades a Null
     */
    public void reset() {
        this.idUsuario = null;
        this.persona = null;
        this.departamento = null;
        this.tipoUsuario = null;
        this.login = null;
        this.estadoEnSistema = null;
        this.emailUsuario = null;
        this.telefonoUsuario = null;

    }

    /**
     * Utilizado en las consultas byExample. Coloca las propiedades a null
     * excepto la propiedad por la que se quiere buscar.
     * 
     * @param parameter parametro de Seleccion
     */
    public void onlyParameter(String parameter) {
        if (parameter.equals("Buscar por ID")) {
            this.persona = null;
            this.departamento = null;
            this.tipoUsuario = null;
            this.login = null;
            this.estadoEnSistema = null;
            this.emailUsuario = null;
            this.telefonoUsuario = null;

        }

        if (parameter.equals("Buscar por Usuario")) {
            this.idUsuario = null;
            this.persona = null;
            this.departamento = null;
            this.tipoUsuario = null;
            this.estadoEnSistema = null;
            this.emailUsuario = null;

        }

        if (parameter.equals("Buscar por Telefono")) {
            this.idUsuario = null;
            this.persona = null;
            this.departamento = null;
            this.tipoUsuario = null;
            this.login = null;
            this.estadoEnSistema = null;
            this.emailUsuario = null;

        }

        if (parameter.equals("Buscar por Email")) {
            this.idUsuario = null;
            this.persona = null;
            this.departamento = null;
            this.tipoUsuario = null;
            this.login = null;
            this.estadoEnSistema = null;
            this.telefonoUsuario = null;

        }
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Usuario other = (Usuario) obj;
        if (this.idUsuario != other.idUsuario && (this.idUsuario == null || !this.idUsuario.equals(other.idUsuario))) {
            return false;
        }
        if (this.persona != other.persona && (this.persona == null || !this.persona.equals(other.persona))) {
            return false;
        }
        if (this.login != other.login && (this.login == null || !this.login.equals(other.login))) {
            return false;
        }
        if (this.departamento != other.departamento && (this.departamento == null || !this.departamento.equals(other.departamento))) {
            return false;
        }
        if ((this.tipoUsuario == null) ? (other.tipoUsuario != null) : !this.tipoUsuario.equals(other.tipoUsuario)) {
            return false;
        }
        if ((this.estadoEnSistema == null) ? (other.estadoEnSistema != null) : !this.estadoEnSistema.equals(other.estadoEnSistema)) {
            return false;
        }
        if ((this.emailUsuario == null) ? (other.emailUsuario != null) : !this.emailUsuario.equals(other.emailUsuario)) {
            return false;
        }
        if ((this.telefonoUsuario == null) ? (other.telefonoUsuario != null) : !this.telefonoUsuario.equals(other.telefonoUsuario)) {
            return false;
        }
        if (this.estadotrabajos != other.estadotrabajos && (this.estadotrabajos == null || !this.estadotrabajos.equals(other.estadotrabajos))) {
            return false;
        }
        return true;
    }
    
    @Override
    public String toString() {
        return "Usuario{" + "idUsuario=" + idUsuario + ", idLogin=" + login.getIdLogin() + ", tipoUsuario=" + tipoUsuario + ", departamento=" + departamento.getIdDepartamento() + ", estadoEnSistema=" + estadoEnSistema + ", emailUsuario=" + emailUsuario + ", telefonoUsuario=" + telefonoUsuario + '}';
    }
}
