/**
 * CartelPdf.
 *
 * Es la clase encargada de generar los Carteles PDF
 *
 * El sistema tendra encuenta que tipo de cartel debe de generar en funcion del
 * trabajo de la Entrada.
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws DocumentException saltara un mensage de Error cuando se produzca una
 * excepcion procedente de la manipulacion del Documento. Si es una excepcion de
 * tipo general se generara una excepcion capturadad por "Exception".
 */

package model.pdf;

import java.io.FileOutputStream;
import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.PageSize;
import com.itextpdf.text.Paragraph;
import com.itextpdf.text.pdf.BaseFont;
import com.itextpdf.text.pdf.PdfContentByte;
import com.itextpdf.text.pdf.PdfWriter;
import java.awt.Desktop;
import java.io.File;
import java.io.IOException;
import java.util.Iterator;
import model.entities.Cartel;
import model.entities.Plantilla;
import model.entities.Trabajo;
import model.util.MessageUtil;
import model.util.functions;


public class CartelPdf {

    private static String FILE;
    private static int NUMERO_DISPLAY = 3;
    private Trabajo trabajo = new Trabajo();
    private PdfWriter writer;

    /**
     * Constructor Vacio
     */
    public CartelPdf() {
    }

    /**
     * Funciones Auxiliares.
     * 
     * Trunca un Float
     * 
     * @param f Float a truncar
     * @return Float Truncado
     */
    public static int trunc(float f) {
        int i = (int) f;
        return i;
    }

    /**
     * Funciones Auxiliares.
     * 
     * Devuelve la parte decimal de un Float
     * 
     * @param f Float a extraer la parte Decimal
     * @return Parte decimal del Float
     */
    public static int decimal(float f) {
        int i = (int) f;
        return (int) (f - i) * 100;
    }

    /**
     * Crea el "FILE" (ruta y archivo temporal a Usar)
     * 
     * Sino existe el Directorio, lo crea.
     */
    private void createTemp() {

        File folder = new File(System.getProperty("user.dir") + "\\carteleriaTemp");
        folder.mkdir();

        FILE = folder.toString() + "\\temp.pdf";
    }

    /**
     * Constructor que Genera un UNICO Cartel PDF.
     * 
     * @param car Cartel a Generar.
     */
    public CartelPdf(Cartel car) {

        Trabajo tra = new Trabajo();
        tra.addCarteles(car);

        CartelPdf p = new CartelPdf(tra);
    }

    /**
     * Constructor que Genera los carteles Asociados a un Trabajo
     * 
     * @param tra Prabajo a Generar.
     */
    public CartelPdf(Trabajo tra) {
        try {
            trabajo = tra;

            Document document = new Document();

            Iterator<Cartel> it = tra.getCarteles().iterator();
            Plantilla plan = it.next().getPlantilla();

            //pagina en Horizontal
            if (plan.getTamano().equals("A4")) {
                document.setPageSize(PageSize.A4.rotate());
            }

            //pagina en Vertical
            if (plan.getTamano().equals("Display")) {
                document.setPageSize(PageSize.A4);
            }

            this.createTemp();

            writer = PdfWriter.getInstance(document, new FileOutputStream(FILE));
            document.open();
            addMetaData(document);


            if (plan.getTamano().equals("A4")) {
                if (plan.getTipo().equals("Frescos")) {
                    A4Frescos(document);
                }

                if (plan.getTipo().equals("Pgc")) {
                    A4Pgc(document);
                }
            } else if (plan.getTamano().equals("Display")) {
                if (plan.getTipo().equals("Frescos")) {
                    displayFrescos(document);
                }

                if (plan.getTipo().equals("Pgc")) {
                    displayPgc(document);
                }
            }

            document.close();
        } catch (DocumentException de) {
            MessageUtil message = new MessageUtil("Error: " + de.getMessage());
        } catch (Exception e) {
            MessageUtil message = new MessageUtil("Error: " + e.getMessage());
        } finally {
            this.openFile(FILE);
        }
    }

    /**
     * Añade los MetaDatos al DocumentoPDF
     * 
     * @param document Documento en USO.
     */
    private static void addMetaData(Document document) {
        document.addTitle("Carteleria by Choped");
        document.addSubject("Carteleria Online");
        document.addAuthor("ChopPrograms");
        document.addCreator("JuanRamon Segura Ponce");
    }

    /**
     * Funcion que Genera los Carteles Tamaño A4 y Formato PGC
     * 
     * @param document Documento en USO.
     */
    private void A4Pgc(Document document) throws IOException, DocumentException {

        try {
            float incX = document.getPageSize().getWidth() / 8;
            float incY = document.getPageSize().getHeight() / 8;


            for (Iterator<Cartel> it = trabajo.getCarteles().iterator(); it.hasNext();) {
                Cartel car = it.next();

                //Creamos el PDF

                //Precio
                float precio;

                if (car.getArticulo().getPrecioOferta() != null) {
                    precio = car.getArticulo().getPrecioOferta();
                } else {
                    precio = car.getArticulo().getPrecio();
                }

                int entera = functions.trunc(precio);
                String decimal = functions.decimal(precio);

                PdfContentByte cb = writer.getDirectContent();

                BaseFont bf = BaseFont.createFont(BaseFont.HELVETICA_BOLD, BaseFont.CP1252, BaseFont.NOT_EMBEDDED);
                cb.beginText();

                //Descripciones
                cb.setFontAndSize(bf, 32);
                cb.showTextAligned(PdfContentByte.ALIGN_CENTER, car.getArticulo().getDescripcion1(), incX * 4, incY * 7 + (incY / 2), 0);
                cb.showTextAligned(PdfContentByte.ALIGN_CENTER, car.getArticulo().getDescripcion2(), incX * 4, incY * 7, 0);


                cb.setFontAndSize(bf, 150);
                cb.showTextAligned(PdfContentByte.ALIGN_RIGHT, "€", incX * 6 + (incX / 4), incY * 3, 0);

                cb.setFontAndSize(bf, 200);
                cb.showTextAligned(PdfContentByte.ALIGN_RIGHT, decimal, incX * 7 + (incX / 2), incY, 0);

                cb.setFontAndSize(bf, 375);
                cb.showTextAligned(PdfContentByte.ALIGN_RIGHT, Integer.toString(entera), incX * 5 + (incX / 2), incY, 0);
                cb.endText();

                document.newPage();

            }
        } catch (Exception e) {
            MessageUtil message = new MessageUtil("Error al Crear el Fichero PDF.");
            document.close();
        }
    }

    /**
     * Funcion que Genera los Carteles Tamaño A4 y Formato Frescos
     * 
     * @param document Documento en USO.
     */
    private void A4Frescos(Document document) throws IOException, DocumentException {

        try {
            float incX = document.getPageSize().getWidth() / 8;
            float incY = document.getPageSize().getHeight() / 8;


            for (Iterator<Cartel> it = trabajo.getCarteles().iterator(); it.hasNext();) {
                Cartel car = it.next();


                float precio;

                if (car.getArticulo().getPrecioOferta() != null) {
                    precio = car.getArticulo().getPrecioOferta();
                } else {
                    precio = car.getArticulo().getPrecio();
                }

                int entera = functions.trunc(precio);
                String decimal = functions.decimal(precio);



                PdfContentByte cb = writer.getDirectContent();

                BaseFont bf = BaseFont.createFont(BaseFont.HELVETICA_BOLD, BaseFont.CP1252, BaseFont.NOT_EMBEDDED);
                cb.beginText();
                cb.setFontAndSize(bf, 32);

                float pos = incY * 6 + (incY / 2);

                cb.showTextAligned(PdfContentByte.ALIGN_LEFT, car.getArticulo().getDescripcion1(), document.leftMargin(), pos, 0);

                pos = pos - (incY / 2);
                cb.showTextAligned(PdfContentByte.ALIGN_LEFT, car.getArticulo().getDescripcion2(), document.leftMargin(), pos, 0);

                pos = pos - (incY / 2);
                if (car.getArticulo().getDescripcion3() != null) {
                    cb.showTextAligned(PdfContentByte.ALIGN_LEFT, car.getArticulo().getDescripcion3(), document.leftMargin(), pos, 0);
                    pos = pos - (incY / 2);
                }


                cb.setFontAndSize(bf, 24);
                if (car.getArticulo().getUnidad().equals("U")) {
                    cb.showTextAligned(PdfContentByte.ALIGN_LEFT, "La Unitat", document.leftMargin(), pos, 0);
                } else {
                    cb.showTextAligned(PdfContentByte.ALIGN_LEFT, "El Quilo", document.leftMargin(), pos, 0);
                }


                cb.setFontAndSize(bf, 150);
                cb.showTextAligned(PdfContentByte.ALIGN_RIGHT, "€", incX * 6 + (incX / 4), incY * 3, 0);

                cb.setFontAndSize(bf, 200);
                cb.showTextAligned(PdfContentByte.ALIGN_RIGHT, decimal, incX * 7 + (incX / 2), incY, 0);

                cb.setFontAndSize(bf, 375);
                cb.showTextAligned(PdfContentByte.ALIGN_RIGHT, Integer.toString(entera), incX * 5 + (incX / 2), incY, 0);
                cb.endText();

                document.newPage();

            }
        } catch (Exception e) {
            MessageUtil message = new MessageUtil("Error al Crear el Fichero PDF.");
            document.close();
        }
    }

    /**
     * Funcion que Genera los Carteles Tamaño Display y Formato Frescos
     * 
     * @param document Documento en USO.
     */
    private void displayFrescos(Document document) throws IOException, DocumentException {

        try {
            //Contador Elementos por pagina
            int n = 0;

            float incX = document.getPageSize().getWidth() / 8;
            float incY = document.getPageSize().getHeight() / 8;

            float posY = incY * 7 + (incY / 4);
            float posN;
            float pos;

            for (Iterator<Cartel> it = trabajo.getCarteles().iterator(); it.hasNext();) {

                Cartel car = it.next();

                float precio;

                if (car.getArticulo().getPrecioOferta() != null) {
                    precio = car.getArticulo().getPrecioOferta();
                } else {
                    precio = car.getArticulo().getPrecio();
                }

                int entera = functions.trunc(precio);
                String decimal = functions.decimal(precio);

                PdfContentByte cb = writer.getDirectContent();

                BaseFont bf = BaseFont.createFont(BaseFont.HELVETICA_BOLD, BaseFont.CP1252, BaseFont.NOT_EMBEDDED);
                cb.beginText();
                cb.setFontAndSize(bf, 18);

                pos = posY;
                cb.showTextAligned(PdfContentByte.ALIGN_LEFT, car.getArticulo().getDescripcion1(), document.leftMargin(), pos, 0);

                pos = pos - (incY / 5);
                cb.showTextAligned(PdfContentByte.ALIGN_LEFT, car.getArticulo().getDescripcion2(), document.leftMargin(), pos, 0);

                pos = pos - (incY / 5);
                if (car.getArticulo().getDescripcion3() != null) {
                    cb.showTextAligned(PdfContentByte.ALIGN_LEFT, car.getArticulo().getDescripcion3(), document.leftMargin(), pos, 0);
                    pos = pos - (incY / 5);
                }


                cb.setFontAndSize(bf, 12);
                if (car.getArticulo().getUnidad().equals("U")) {
                    cb.showTextAligned(PdfContentByte.ALIGN_LEFT, "La Unitat", document.leftMargin(), pos, 0);
                } else {
                    cb.showTextAligned(PdfContentByte.ALIGN_LEFT, "El Quilo", document.leftMargin(), pos, 0);
                }

                posN = posY - (2 * incY / 2 + (incY / 3));
                cb.setFontAndSize(bf, 75);
                cb.showTextAligned(PdfContentByte.ALIGN_RIGHT, "€", incX * 6 + (incX / 2), posN + (incY / 3), 0);

                cb.setFontAndSize(bf, 110);
                cb.showTextAligned(PdfContentByte.ALIGN_RIGHT, decimal, incX * 7 + (incX / 2), posN - (incY / 2), 0);

                cb.setFontAndSize(bf, 200);
                cb.showTextAligned(PdfContentByte.ALIGN_RIGHT, Integer.toString(entera), incX * 6, posN - (incY / 2), 0);

                cb.endText();


                if (n < NUMERO_DISPLAY) {
                    posY = posY - (incY * 2) - (incY / 2);
                } else {
                    document.newPage();
                    posY = incY * 7 + (incY / 4);
                }
            }


        } catch (Exception e) {
            MessageUtil message = new MessageUtil("Error al Crear el Fichero PDF.");
            document.close();
        }
    }

    /**
     * Funcion que Genera los Carteles Tamaño Display y Formato PGC
     * 
     * @param document Documento en USO.
     */
    private void displayPgc(Document document) throws IOException, DocumentException {

        try {
            //Contador Elementos por pagina
            int n = 0;

            float incX = document.getPageSize().getWidth() / 8;
            float incY = document.getPageSize().getHeight() / 8;

            float posY = incY * 7 + (incY / 4);
            float posN;
            float pos;

            for (Iterator<Cartel> it = trabajo.getCarteles().iterator(); it.hasNext();) {

                Cartel car = it.next();

                float precio;

                if (car.getArticulo().getPrecioOferta() != null) {
                    precio = car.getArticulo().getPrecioOferta();
                } else {
                    precio = car.getArticulo().getPrecio();
                }

                int entera = functions.trunc(precio);
                String decimal = functions.decimal(precio);

                PdfContentByte cb = writer.getDirectContent();

                BaseFont bf = BaseFont.createFont(BaseFont.HELVETICA_BOLD, BaseFont.CP1252, BaseFont.NOT_EMBEDDED);
                cb.beginText();
                cb.setFontAndSize(bf, 18);

                pos = posY + (incY / 3);
                cb.showTextAligned(PdfContentByte.ALIGN_CENTER, car.getArticulo().getDescripcion1(), incX * 4, pos, 0);

                pos = pos - (incY / 5);
                cb.showTextAligned(PdfContentByte.ALIGN_CENTER, car.getArticulo().getDescripcion2(), incX * 4, pos, 0);

                posN = posY - (2 * incY / 2 + (incY / 3));
                cb.setFontAndSize(bf, 75);
                cb.showTextAligned(PdfContentByte.ALIGN_RIGHT, "€", incX * 6 + (incX / 2), posN + (incY / 3), 0);

                cb.setFontAndSize(bf, 110);
                cb.showTextAligned(PdfContentByte.ALIGN_RIGHT, decimal, incX * 7 + (incX / 2), posN - (incY / 2), 0);

                cb.setFontAndSize(bf, 200);
                cb.showTextAligned(PdfContentByte.ALIGN_RIGHT, Integer.toString(entera), incX * 6, posN - (incY / 2), 0);

                cb.endText();


                if (n < NUMERO_DISPLAY) {
                    posY = posY - (incY * 2) - (incY / 2);
                } else {
                    document.newPage();
                    posY = incY * 7 + (incY / 4);
                }
            }

        } catch (Exception e) {
            MessageUtil message = new MessageUtil("Error al Crear el Fichero PDF.");
            document.close();
        }

    }
    
    /**
     * Funcion Auxiliar
     * 
     * Abre el Archivo Generado
     * 
     * @param ruta Ruta del Archivo a Abrir.
     */
    private static void openFile(String ruta) {

        try {

            if ((new File(ruta)).exists()) {

                Desktop desktop = Desktop.getDesktop();
                File file = new File(ruta);
                desktop.open(file);

            } else {
                System.out.println("El Archivo no Existe");
            }

        } catch (Exception ex) {
            MessageUtil message = new MessageUtil("Error: " + ex.getMessage());

        }

    }

    /**
     * Funcion Auxiliar
     * 
     * Introduce Lineas en Blanco
     * 
     * @param paragraph Paragraph en USO.
     * @param number Numero de lineas en Blanco a introducir.
     */
    private static void addEmptyLine(Paragraph paragraph, int number) {
        for (int i = 0; i < number; i++) {
            paragraph.add(new Paragraph(" "));
        }
    }

    /**
     * Funcion que Genera EL PROTOTIPO de los Carteles Tamaño A5 y Formato Frescos
     * 
     * @param document Documento en USO.
     */
    private void A5Frescos(Document document) throws IOException, DocumentException {

        try {
            float incX = document.getPageSize().getWidth() / 8;
            float incY = document.getPageSize().getHeight() / 8;

            for (Iterator<Cartel> it = trabajo.getCarteles().iterator(); it.hasNext();) {
                Cartel car = it.next();


                float precio;

                if (car.getArticulo().getPrecioOferta() != null) {
                    precio = car.getArticulo().getPrecioOferta();
                } else {
                    precio = car.getArticulo().getPrecio();
                }

                int entera = functions.trunc(precio);
                String decimal = functions.decimal(precio);

                PdfContentByte cb = writer.getDirectContent();

                BaseFont bf = BaseFont.createFont(BaseFont.HELVETICA_BOLD, BaseFont.CP1252, BaseFont.NOT_EMBEDDED);
                cb.beginText();
                cb.setFontAndSize(bf, 24);

                float pos = incY * 7;

                cb.showTextAligned(PdfContentByte.ALIGN_LEFT, car.getArticulo().getDescripcion1(), document.leftMargin(), pos, 0);

                pos = pos - (incY / 4);
                cb.showTextAligned(PdfContentByte.ALIGN_LEFT, car.getArticulo().getDescripcion2(), document.leftMargin(), pos, 0);

                pos = pos - (incY / 4);
                if (car.getArticulo().getDescripcion3() != null) {
                    cb.showTextAligned(PdfContentByte.ALIGN_LEFT, car.getArticulo().getDescripcion3(), document.leftMargin(), pos, 0);
                    pos = pos - (incY / 4);
                }


                cb.setFontAndSize(bf, 16);
                if (car.getArticulo().getUnidad().equals("U")) {
                    cb.showTextAligned(PdfContentByte.ALIGN_LEFT, "La Unitat", document.leftMargin(), pos, 0);
                } else {
                    cb.showTextAligned(PdfContentByte.ALIGN_LEFT, "El Quilo", document.leftMargin(), pos, 0);
                }


                cb.setFontAndSize(bf, 100);
                cb.showTextAligned(PdfContentByte.ALIGN_RIGHT, "€", incX * 6, incY * 6 - (incY / 3), 0);

                cb.setFontAndSize(bf, 150);
                cb.showTextAligned(PdfContentByte.ALIGN_RIGHT, decimal, incX * 7 + (incX / 2), incY * 5 - (incY / 2), 0);

                cb.setFontAndSize(bf, 275);
                cb.showTextAligned(PdfContentByte.ALIGN_RIGHT, Integer.toString(entera), incX * 5 + (incX / 3), incY * 5 - (incY / 2), 0);

                cb.endText();
                document.newPage();

            }


        } catch (Exception e) {
            MessageUtil message = new MessageUtil("Error al Crear el Fichero PDF.");
            document.close();
        }
    }
    
    
    /*
     * MAIN PARA TESTEAR LA CLASE DE FORMA MAS RAPIDA
     * 
     public static void main(String[] args) {
     try {
     Date d = new Date();
     Trabajo tra = new Trabajo("Prueva 1", 1, d);

     Articulo art1 = new Articulo("Beguda Energetica 33cl", "Carrefour", (float) 0.33, "U", "843187611111");
     Articulo art2 = new Articulo("Beguda Energetica 50cl", "Carrefour", (float) 0.50, "U", "843187622222");
     Articulo art3 = new Articulo("Beguda Energetica 1L", "Carrefour", (float) 1.20, "U", "843187633333");
            
     Plantilla plan1 = new Plantilla("Frescos", "Display");

     Cartel car1 = new Cartel(art1, plan1);
     Cartel car2 = new Cartel(art2, plan1);
     Cartel car3 = new Cartel(art3, plan1);
            
     tra.addCarteles(car1);
     tra.addCarteles(car2);
     tra.addCarteles(car3);
            
     Pdf p = new Pdf(tra);

     functions.openFile(FILE);
     } catch (Exception e) {
     MessageUtil message = new MessageUtil("Error: "+e.getMessage());
     }
     }*/
}
