/**
 * CartelPdf.
 *
 * Es la clase encargada de generar los Listados PDF
 *
 * El Sistema imprimira tanto una lista de objetos como el objeto
 *
 * @author JuanRamon Segura Ponce
 *
 * @version v.1.0
 * @throws DocumentException saltara un mensage de Error cuando se produzca una
 * excepcion procedente de la manipulacion del Documento. Si es una excepcion de
 * tipo general se generara una excepcion capturadad por "Exception".
 */
package model.pdf;

import java.io.FileOutputStream;
import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.PageSize;
import com.itextpdf.text.Paragraph;
import com.itextpdf.text.pdf.BaseFont;
import com.itextpdf.text.pdf.PdfContentByte;
import com.itextpdf.text.pdf.PdfWriter;
import java.awt.Desktop;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import model.entities.Cartel;
import model.entities.Plantilla;
import model.entities.Articulo;
import model.entities.Trabajo;
import model.util.MessageUtil;



public final class ObjectPdf {

    private static String FILE;
    private static int NUMERO_DISPLAY = 3;
    private Object obj = null;
    private PdfWriter writer;

    /**
     * Constructor Vacio
     */
    public ObjectPdf() {
    }

    /**
     * Constructor con Objeto
     */
    public ObjectPdf(Object object) {
        this.obj = object;
        this.createObjectPdf();
    }


    /**
     * Crea el "FILE" (ruta y archivo temporal a Usar)
     *
     * Sino existe el Directorio, lo crea.
     */
    private void createTemp() {

        File folder = new File(System.getProperty("user.dir") + "\\carteleriaTemp");
        folder.mkdir();

        Date d = new Date();
        
        FILE = folder.toString() + "\\"+d.getTime()+".pdf";

    }

    /**
     * Constructor que Genera los carteles Asociados a un Trabajo
     *
     * @param tra Prabajo a Generar.
     */
    public boolean createObjectPdf() {
        try {
            boolean bool = true;

            Document document = new Document();

            //pagina en Vertical
            document.setPageSize(PageSize.A4.rotate());

            this.createTemp();

            writer = PdfWriter.getInstance(document, new FileOutputStream(FILE));
            document.open();
            addMetaData(document);

            bool = this.printList(document);

            document.close();

            return bool;
        } catch (DocumentException de) {
            MessageUtil message = new MessageUtil("Error: " + de.getMessage());
            return false;
        } catch (Exception e) {
            MessageUtil message = new MessageUtil("Error: " + e.getMessage());
            return false;
        } finally {
            this.openFile(FILE);

        }
    }

    /**
     * Añade los MetaDatos al DocumentoPDF
     *
     * @param document Documento en USO.
     */
    private static void addMetaData(Document document) {
        document.addTitle("Carteleria by Choped");
        document.addSubject("Carteleria Online");
        document.addAuthor("ChopPrograms");
        document.addCreator("JuanRamon Segura Ponce");
    }

    /**
     * Funcion que Genera los listados en PDF
     *
     * @param document Documento en USO.
     */
    private boolean printList(Document document) throws IOException, DocumentException {
        
        ArrayList list = new ArrayList();
        
        try {
            float incX = document.getPageSize().getWidth() / 12;
            float incY = document.getPageSize().getHeight() / 25;
            
            float x,y;
            
            int i = 0;
            int pages = 1;
            
            if (obj instanceof ArrayList<?>) {   
                list = (ArrayList) obj;
            }else{
                list.add(obj);
            }
            
            PdfContentByte cb = writer.getDirectContent();
            BaseFont bf = BaseFont.createFont(BaseFont.HELVETICA_BOLD, BaseFont.CP1252, BaseFont.NOT_EMBEDDED);
            cb.beginText();
            
            x = incX*6;
            y = incY*2;
            
            cb.setFontAndSize(bf, 24);
            cb.showTextAligned(PdfContentByte.ALIGN_CENTER, "Listado de "+this.getPlural(list.get(0).getClass().getSimpleName()), x, document.getPageSize().getHeight()-y, 0);
            
            cb.endText();
            
            //System.out.println("size = " +list.size());
            
            x = incX;
            
            for (Iterator it = list.listIterator(); it.hasNext();) {
                //BaseFont bf = BaseFont.createFont(BaseFont.HELVETICA_BOLD, BaseFont.CP1252, BaseFont.NOT_EMBEDDED);
                cb = writer.getDirectContent();
                bf = BaseFont.createFont(BaseFont.HELVETICA_BOLD, BaseFont.CP1252, BaseFont.NOT_EMBEDDED);
                cb.beginText();
                cb.setFontAndSize(bf, 10);

                Object obj = it.next();
                
                y = y+incY;
                
                cb.showTextAligned(PdfContentByte.ALIGN_LEFT, obj.toString(), x,document.getPageSize().getHeight()- y, 0);
                i=i+1;
                
                if (i==20){
                    y = incY*23;
                    cb.showTextAligned(PdfContentByte.ALIGN_RIGHT, Integer.toString(pages), document.getPageSize().getWidth()-x,document.getPageSize().getHeight()- y, 0);
                }
                
                cb.endText();
                
                if (i==20){
                    i=0;
                    y = incY*2;
                    pages=pages+1;
                    document.newPage();
                }
            }
                
                 //introducimos el numero de pagina de la ultima pagina
                cb = writer.getDirectContent();
                bf = BaseFont.createFont(BaseFont.HELVETICA_BOLD, BaseFont.CP1252, BaseFont.NOT_EMBEDDED);
                cb.beginText();
                cb.setFontAndSize(bf, 12);
            
                y = incY*23;
                cb.showTextAligned(PdfContentByte.ALIGN_RIGHT, Integer.toString(pages), document.getPageSize().getWidth()-x,document.getPageSize().getHeight()- y, 0);
                
                cb.endText();    
            
                document.close();
            return true;
        } catch (Exception e) {
            MessageUtil message = new MessageUtil("Error al Crear el Fichero PDF.");
            document.close();
            return false;
        }
    }

    private String getPlural(String palabra){
        String s ="";
        
        String c = palabra.substring(palabra.length()-1);

        if ((c.charAt(0)=='a') || (c.charAt(0)=='e') || (c.charAt(0)=='i') || (c.charAt(0)=='o') || (c.charAt(0)=='u')){
            s = palabra+"s";
        }else{
            s = palabra + "es";
        }
        
        return s;
    }
    
    /**
     * Funcion Auxiliar
     *
     * Abre el Archivo Generado
     *
     * @param ruta Ruta del Archivo a Abrir.
     */
    private static void openFile(String ruta) {

        try {

            if ((new File(ruta)).exists()) {

                Desktop desktop = Desktop.getDesktop();
                File file = new File(ruta);
                desktop.open(file);

            } else {
                System.out.println("El Archivo no Existe");
            }

        } catch (Exception ex) {
            MessageUtil message = new MessageUtil("Error: " + ex.getMessage());

        }
    }
    
    
    /*
    public static void main(String[] args) {
     try {
     Date d = new Date();
     Trabajo tra = new Trabajo("Prueva 1", 1, d);

     Articulo art1 = new Articulo("Beguda Energetica 33cl", "Carrefour", (float) 0.33, "U", "843187611111");
     Articulo art2 = new Articulo("Beguda Energetica 50cl", "Carrefour", (float) 0.50, "U", "843187622222");
     Articulo art3 = new Articulo("Beguda Energetica 1L", "Carrefour", (float) 1.20, "U", "843187633333");
            
     Plantilla plan1 = new Plantilla("Frescos", "Display");

     Cartel car1 = new Cartel(art1, plan1);
     Cartel car2 = new Cartel(art2, plan1);
     Cartel car3 = new Cartel(art3, plan1);
            
     tra.addCarteles(car1);
     tra.addCarteles(car2);
     tra.addCarteles(car3);
            
     //ObjectPdf p = new ObjectPdf(tra);

     ArrayList<Cartel> lc= new ArrayList<Cartel>();
     
     for (int i=0;i<=29;i++){
         lc.add(car3);
         
     }
     
     ObjectPdf p2 = new ObjectPdf(lc);
     
     functions.openFile(FILE);
     } catch (Exception e) {
     MessageUtil message = new MessageUtil("Error: "+e.getMessage());
     }
     }
    */
    
    
        /**
     * Funcion Auxiliar
     * 
     * Introduce Lineas en Blanco
     * 
     * @param paragraph Paragraph en USO.
     * @param number Numero de lineas en Blanco a introducir.
     */
    private static void addEmptyLine(Paragraph paragraph, int number) {
        for (int i = 0; i < number; i++) {
            paragraph.add(new Paragraph(" "));
        }
    }
}
