DROP DATABASE TFC;

CREATE DATABASE TFC;
USE TFC;

CREATE TABLE DOCUMENTO (
    idDocumento integer unique NOT NULL AUTO_INCREMENT,
    numeroDocumento character varying(9) UNIQUE NOT NULL,
    tipoDocumento character varying(3) NOT NULL,
    CONSTRAINT pk_Documento_idDocumento PRIMARY KEY (idDocumento)  
);

CREATE TABLE PERSONA(
	idPersona integer NOT NULL AUTO_INCREMENT,	
	idDocumento integer UNIQUE NOT NULL,
	nombre character varying(16) NOT NULL,
    apellido1 character varying(16) NOT NULL,
    apellido2 character varying(16) NOT NULL,
    idioma character varying(16) NOT NULL,
    telefono character varying(10) UNIQUE NOT NULL,
	email character varying(32) UNIQUE NOT NULL,
	CONSTRAINT pk_Persona PRIMARY KEY (idPersona),
    CONSTRAINT fk_Persona_Documento FOREIGN KEY (idDocumento) REFERENCES DOCUMENTO (idDocumento)  
);


CREATE TABLE DEPARTAMENTO (
    idDepartamento integer NOT NULL AUTO_INCREMENT,
    departamento character varying(32) NOT NULL,
    codigoDepartamento character varying(4) UNIQUE NOT NULL,
    CONSTRAINT pk_Departamento_idDepartamento PRIMARY KEY (idDepartamento)  
);

CREATE TABLE LOGIN (
	idLogin integer unique NOT NULL AUTO_INCREMENT,
	usuario character varying(16) unique NOT NULL,
	password character(16) NOT NULL,
    dataPassWord date NOT NULL,
    CONSTRAINT pk_Login PRIMARY KEY (idLogin)
);

CREATE TABLE USUARIO (
    idUsuario integer unique NOT NULL,	
	idLogin integer NOT NULL,
	tipoUsuario character(1) NOT NULL, #A Administrador, R ResponsableCarteleria i U UsuarioFinal     
    idDepartamento integer NOT NULL,
	estadoEnSistema character(1) NOT NULL,  #A Activo i B Bloqueado
    emailUsuario character varying(64) UNIQUE NOT NULL,
    telefonoUsuario character varying(10) UNIQUE NOT NULL,
    CONSTRAINT pk_Usuario PRIMARY KEY (idUsuario),
    CONSTRAINT fk_Usuario_Persona FOREIGN KEY (idUsuario) REFERENCES PERSONA (idPersona),
	CONSTRAINT fk_Usuario_Departamento FOREIGN KEY (idDepartamento) REFERENCES DEPARTAMENTO (idDepartamento),
    CONSTRAINT fk_Usuario_Login FOREIGN KEY (idLogin) REFERENCES LOGIN (idLogin)
);



CREATE TABLE INCIDENCIA (
    idIncidencia integer NOT NULL AUTO_INCREMENT,
    origen character varying(3) NOT NULL,
    destino character varying(3) NOT NULL,
    asunto character varying(64) NOT NULL,
    incidencia character varying(320) NOT NULL,
    estado character varying(1) NOT NULL,
	dataApertura date NOT NULL,
	respuesta character varying(320),
    dataCierre date,
    CONSTRAINT ck_DataCartel CHECK (dataApertura <= dataCierre),
    CONSTRAINT pk_Incidencia PRIMARY KEY (idIncidencia)
);

CREATE TABLE ARTICULO (
    idArticulo integer NOT NULL AUTO_INCREMENT,
    descripcion1 character varying(64) NOT NULL,
    descripcion2 character varying(64) NOT NULL,
    descripcion3 character varying(64),
    precio float NOT NULL,
    precioOferta float,
    tecla integer,
    unidad character varying(1) NOT NULL,
    codInterno character varying(6) UNIQUE,
    ean character varying(13) UNIQUE NOT NULL,
    CONSTRAINT pk_Articulo PRIMARY KEY (idArticulo)
);

CREATE TABLE PLANTILLA (
	idPlantilla integer NOT NULL AUTO_INCREMENT,
    tipo character varying(16) NOT NULL,
    tamano character varying(16) NOT NULL,
    CONSTRAINT pk_Plantilla PRIMARY KEY (idPlantilla)
);


CREATE TABLE CARTEL (
    idCartel integer NOT NULL AUTO_INCREMENT,
    idPlantilla integer NOT NULL,
    idArticulo integer NOT NULL,
    dataIni date,
    dataFi date,
    CONSTRAINT ck_DataCartel CHECK (dataIni <= dataFi), 
    CONSTRAINT pk_Cartel PRIMARY KEY (idCartel),
    CONSTRAINT fk_Cartel_idPlantilla FOREIGN KEY (idPlantilla) REFERENCES PLANTILLA (idPlantilla),
    CONSTRAINT fk_Cartel_idArticulo FOREIGN KEY (idArticulo) REFERENCES ARTICULO (idArticulo)
);


CREATE TABLE TRABAJO (
    idTrabajo integer NOT NULL AUTO_INCREMENT,
    descripcion character varying(64) UNIQUE NOT NULL,
    prioridad integer NOT NULL,
    fechaCreacion date NOT NULL,
    CONSTRAINT pk_Trabajo PRIMARY KEY (idTrabajo)
);

CREATE TABLE CARTELES (
    idTrabajo integer NOT NULL,
    idCartel integer NOT NULL,
    CONSTRAINT pk_Carteles PRIMARY KEY (idTrabajo,idCartel),
    CONSTRAINT fk_Carteles_idTrabajo FOREIGN KEY (idTrabajo) REFERENCES TRABAJO (idTrabajo),
    CONSTRAINT fk_Carteles_idCartel FOREIGN KEY (idCartel) REFERENCES CARTEL (idCartel)
);

CREATE TABLE ESTADOTRABAJO (
	idEstadoTrabajo integer NOT NULL AUTO_INCREMENT,
    idTrabajo integer NOT NULL,
    idUsuario integer NOT NULL,
	estado character varying(1) NOT NULL,# N = no impreso, I = Impreso, R = Rechazado
    fechaEstado date NOT NULL, 
    CONSTRAINT pk_traImprimidos PRIMARY KEY (idEstadoTrabajo),
    CONSTRAINT fk_traImprimidos_idTrabajo FOREIGN KEY (idTrabajo) REFERENCES TRABAJO (idTrabajo),
    CONSTRAINT fk_traImprimidos_idUsuario FOREIGN KEY (idUsuario) REFERENCES USUARIO (idUsuario)
);
COMMIT;

